!====================================================================================================
! DIRECTORY: ver6/amsu_v6/
! FILE NAME: amsu_module.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION: Contains routines used to read the archived AMSU satellite orbit files (dot "a" files)
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE AMSU_MODULE
  implicit none
  real,parameter :: badvalue = -999. ! I get an error if try to use this in main
  
  integer,parameter :: num_scans=10800 ! Number of scans per day. 10800 = 3600*24/8. The 8 sec is time between scans.
  integer,parameter :: num_beam_pos = 30
  integer,parameter :: num_channels = 15

  integer,parameter :: nlat=72, nlon=144

  type day_of_data
     character(len=3) :: sat  ! Input, satellite, one of N15, N16, N18, N19, AQU, MET. Case is uppper only.
     integer          :: ksat ! Input, Roy's satellite designation
     integer          :: year ! Input, 4-digit year
     integer          :: jday ! Input, Julian day
     integer          :: month! Input, month 
     real(8) :: time_of_day(num_scans)       ! Output, time of day in seconds
     real(4) :: lat(num_beam_pos,num_scans)  ! Output, latitude
     real(4) :: lon(num_beam_pos,num_scans)  ! Output, longitude
     real(4) :: Tb( num_channels,num_beam_pos,num_scans)! Output, Raw brightness temperature(K)
  end type 

  ! Data limits per channel
  real ::  limits(2,15)     ! Channel
  data limits/100.,295.,  & ! 1
              100.,295.,  & ! 2
              140.,295.,  & ! 3
              150.,285.,  & ! 4
              150.,280.,  & ! 5
              150.,270.,  & ! 6
              150.,250.,  & ! 7
              150.,250.,  & ! 8
              150.,250.,  & ! 9
              150.,250.,  & ! 10
              150.,250.,  & ! 11
              150.,260.,  & ! 12
              150.,270.,  & ! 13
              150.,280.,  & ! 14
              150.,290./    ! 15

CONTAINS

!==================================================================================

subroutine read_aqua( x, status )
  use CALENDAR
  type(day_of_data) :: x
  logical           :: status  ! Output variable. Returned as "T" when data is found, returned as "F" otherwise.

  character(len=150),parameter :: path = "/rstor/braswell/aqua_amsu_data/"

  integer :: year, jday, month, day_of_month
  integer :: vec(3),nn
  character(len=150) :: infile
  integer :: iprint,iscan,ifile,time,index,ichannel


  type aqua_input
     real,allocatable :: lat(:,:,:)
     real,allocatable :: lon(:,:,:)
     real,allocatable :: time(:,:,:)
     real,allocatable :: Tb(:,:,:,:)
  end type
  type(aqua_input) :: aq

  year = x%year
  jday = x%jday

  vec = month_day_from_jday(year, jday)
  month = vec(2)
  day_of_month = vec(3)

  nn=1000
  if( year<2009 .or. (year==2009 .and. jday<274)) nn=240  ! 274 is October 1.

  allocate(aq%lat(30,45,nn))
  allocate(aq%lon(30,45,nn))
  allocate(aq%time(30,45,nn))
  allocate(aq%Tb(15,30,45,nn))


  ! Construct file name (e.g. /rstor/braswell/aqua_amsu_data/2013/aqua_amsu.2013.05.31.dat)
  write(infile,'(a,i4.4,a,i4.4,a,i2.2,a,i2.2,a)') trim(path),year,"/aqua_amsu.",year,".",month,".",day_of_month,".dat"

  inquire(FILE=trim(infile),EXIST=status)
  !write(6,*) "status = ",status ! DEBUG
  if( .not. status ) return

  ! Open and read input file
  open(1,file=trim(infile),form="unformatted",access="stream",action="read",status="old")
  read(1) aq%lat, aq%lon, aq%time, aq%TB
  close(1)

  ! Switch from -9999 to badvalue
  where(aq%lat ==-9999) aq%lat =badvalue
  where(aq%lon ==-9999) aq%lon =badvalue
  where(aq%time==-9999) aq%time=badvalue
  where(aq%Tb  ==-9999) aq%Tb  =badvalue


  ! Initialize x
  x%time_of_day = badvalue
  x%lat         = badvalue
  x%lon         = badvalue
  x%Tb          = badvalue

  ! AQUA HAS TIME FOR EACH PRINT. The time for print 15 will be used as the scan time.
  do ifile=1,size(aq%lat,3)
     do iscan=1,size(aq%lat,2)
        time = aq%time(15,iscan,ifile)
        if(time==badvalue) cycle
        index = floor(time/8.) + 1
        if(index<1 .or. index>size(x%time_of_day)) cycle ! Added 8/27/2013, wdb
        !write(6,*) ifile,iscan,time,index ! DEBUGa
        x%time_of_day(index) = time
        x%lat(:,index) = aq%lat(:,iscan,ifile)
        x%lon(:,index) = aq%lon(:,iscan,ifile)
        x%Tb(:,:,index)= aq%Tb(:,:,iscan,ifile)
     end do
  end do   

  deallocate(aq%lat,aq%lon,aq%time,aq%Tb)

  ! Check ranges on lat/lon
  where(x%lat/=badvalue .and. (x%lat>90 .or. x%lat<-90)   ) x%lat = badvalue
  where(x%lon/=badvalue .and. (x%lon>180 .or. x%lon<-180) ) x%lon = badvalue

  ! Verify Tb's are within data limits
  do iscan=1,num_scans
     do ichannel=1,15
        where(x%Tb(ichannel,:,iscan)<limits(1,ichannel) .or. &
              x%Tb(ichannel,:,iscan)>limits(2,ichannel)) x%Tb(ichannel,:,iscan) = badvalue
     end do
  end do   

end subroutine read_aqua
!==================================================================================

subroutine write_scans( x )
  type(day_of_data) :: x
  
  integer :: iscan
  integer,parameter :: ichannel = 5

  do iscan=1,num_scans
     write(6,10)iscan, x%sat, x%year, x%jday, x%Tb(ichannel,:,iscan)
     10 format(i5,1x,a3,i4,1x,i3,1x,30(1x,f8.3))
  end do   

end subroutine write_scans  
!==================================================================================

!==================================================================================

! This routine returns Tb's and associated data for the specified sat, year, day.
subroutine read_amsu( x, status )
  implicit none

  type(day_of_data) :: x
  logical           :: status  ! Output variable. Returned as "T" when data is found, returned as "F" otherwise.
  integer :: i,index,iscan,ichannel

  character(len=3) :: csat
  integer :: iyear, jday, n, iorbit

  ! Argument list variables for routine read_AMSU_A_orbitfile
  character(len=150) :: filename          ! Input:  Orbit file to read (including path)
  integer :: nscan                        ! Output: Number of scans
  integer,allocatable :: iyyyyddd(:)      ! Output: Compact format for year/day_of_year (i.e. 1998361) ;iyyyyddd(nscan)                    
  integer,allocatable :: year(:)          ! Output: Scan line year (4-digit) (e.g. 1998)               ;year(nscan)                                    
  integer,allocatable :: day_of_year(:)   ! Output: Scan line day of year (e.g. 365)                   ;day_of_year(nscan)                 
  real(8),allocatable :: time_of_day(:)   ! Output: Time of day (secs)                                 ;time_of_day(nscan)                 
  real(4),allocatable :: lat(:,:)         ! Output: latitude (degrees,lat:-90->+90)                    ;lat(num_beam_pos,nscan)            
  real(4),allocatable :: lon(:,:)         ! Output: longitude (degrees,lon:-180->+180)                 ;lon(num_beam_pos,nscan)            
  real(4),allocatable :: Tb(:,:,:)        ! Output: Brightness temperature (K)                         ;Tb(num_channels,num_beam_pos,nscan)
  real(4),allocatable :: cold_ref_tb(:,:) ! Output: Cold ref Tb (K) (avg of 1 and 2)                   ;cold_ref_tb(num_channels,nscan)    
  real(4),allocatable :: warm_ref_tb(:,:) ! Output: Warm ref Tb (K) (avg of 1 and 2)                   ;warm_ref_tb(num_channels,nscan)    
  real(4),allocatable :: view_angle(:,:)  ! Output: Effective view angles (deg) at surface             ;view_angle(num_beam_pos,nscan)     
  logical             :: error_flag       ! Output: returned as TRUE if error encountered, FALSE otherwise.
  character(len=150),allocatable :: orbit_files(:)

  csat  = x%sat
  iyear = x%year
  jday  = x%jday

  call get_AMSU_orbit_file_names( csat, iyear, jday, orbit_files ) ! Vector orbit_files is allocated within this call.

  ! Initialize output arrays
  x%time_of_day = badvalue
  x%lat         = badvalue
  x%lon         = badvalue
  x%Tb          = badvalue

  ! Set status
  if(size(orbit_files)==0) then
     status = .FALSE.
     return
  else
     status = .TRUE.
  end if   
  
  ORBIT_FILE_LOOP: do iorbit=1,size(orbit_files)
     filename = trim(orbit_files(iorbit))

     call read_AMSU_A_orbitfile( filename,nscan,iyyyyddd,year,day_of_year,time_of_day, &
          lat,lon,Tb,cold_ref_Tb,warm_ref_Tb,view_angle, error_flag )

     if(error_flag) cycle

     do i=1,size(time_of_day)
        if( year(i)/=x%year .or. day_of_year(i)/=x%jday) cycle
        index = FLOOR(time_of_day(i)/8) + 1  ! 8 is for 8 seconds per scan
        if( index<0 .or. index>num_scans ) cycle

        x%time_of_day(index) = time_of_day(i)
        x%lat(:,index)       = lat(:,i)    
        x%lon(:,index)       = lon(:,i)    
        x%Tb(:,:,index)      = Tb(:,:,i)
        
     end do   

  end do ORBIT_FILE_LOOP  


  ! Check ranges on lat/lon
  where(x%lat/=badvalue .and. (x%lat>90 .or. x%lat<-90) )   x%lat = badvalue
  where(x%lon/=badvalue .and. (x%lon>180 .or. x%lon<-180) ) x%lon = badvalue

  ! Verify Tb's are within data limits
  do iscan=1,num_scans
     do ichannel=1,15
        where(x%Tb(ichannel,:,iscan)<limits(1,ichannel) .or. &
              x%Tb(ichannel,:,iscan)>limits(2,ichannel)) x%Tb(ichannel,:,iscan) = badvalue
     end do
  end do   

end subroutine read_amsu
!==================================================================================

! This routine retrieves the orbit file names for the specified sat, year, and jday.
! It also gets the last orbit file from the previous day if that has orbits that cross into current day.
! Set up to work for AMSU satellites N15, N16, N18, and METOP. It does not work for AQUA.
subroutine get_AMSU_orbit_file_names( sat3, year, jday, orbit_files )
  character(len=3) :: sat3   ! Input, sat designation (one of N15, N16, N18, N19)
                             ! Note: N15-->NK, N16-->NL, N18-->NN, MET-->M2
  integer          :: year   ! Input, 4-digit year
  integer          :: jday   ! Input, Julian day

  integer          :: previous_jday, previous_jday_year


  character(len=150),allocatable :: orbit_files(:)  ! Output, Returned list of file names. Allocated in this routine.

  logical,save :: first_call = .TRUE.
  character(len=150),save,allocatable :: all_orbit_files(:) ! This hold all orbit file names read from orbit_file_list.
                                                            ! This is used in subsequent calls rather than reading the file again.
  integer,save :: num_files

  character(len=150) :: buffer,path
  character(len=2) :: sat2, sat2_e
  integer :: year_e, jday_e,icount
  integer,parameter :: max_num_orbit_files = 366*20 ! max number of orbit files allowed in a year, jday range.
  character(len=150) :: name_vec(max_num_orbit_files)
  integer :: i, start_hour, stop_hour

  ! Path and related variables
  character(len=150),parameter :: orbit_file_list = "/rgroup/amsu/orbit_file_list" ! File containing all available N15, N16, N18, N19, MET  orbit file names
  character(len=150),parameter :: path_noaa_15 = "/rgroup/amsu/dot_a_files/" ! Path to noaa_15 orbit files, with trailing "/".
  character(len=150),parameter :: path_noaa_16 = "/rgroup/amsu/dot_a_files/" ! Path to noaa_16 orbit files, with trailing "/".
  character(len=150),parameter :: path_noaa_18 = "/rgroup/amsu/dot_a_files/" ! Path to noaa_18 orbit files, with trailing "/".
  character(len=150),parameter :: path_noaa_19 = "/rgroup/amsu/dot_a_files/" ! Path to noaa_19 orbit files, with trailing "/".
  character(len=150),parameter :: path_metop   = "/rgroup/amsu/dot_a_files/" ! Path to metop orbit files, with trailing "/".

  ! If the first call, read and store the full orbit file list.
  if(first_call) then
     open(1,file=trim(orbit_file_list),action="read",status="old")
     read(1,*) num_files
     if(allocated(all_orbit_files)) deallocate(all_orbit_files)
     allocate(all_orbit_files(num_files))
     do i=1,num_files
        read(1,'(a150)') all_orbit_files(i)
     end do
     close(1)
     first_call = .FALSE.
  end if   


  ! Check jday
  if( leap_year(year) ) then
     if( jday<1 .or. jday>366) stop "Error - get_AMSU_orbit_file_names(): jday must be in range 1..366"
  else   
     if( jday<1 .or. jday>365) stop "Error - get_AMSU_orbit_file_names(): jday must be in range 1..365"
  end if   
  
  ! Check sat3 and make 2 character satellite assignment
  select case(sat3)
     case("N15"); sat2="NK"; path = trim(path_noaa_15)
     case("N16"); sat2="NL"; path = trim(path_noaa_16)
     case("N18"); sat2="NN"; path = trim(path_noaa_18)
     case("N19"); sat2="NP"; path = trim(path_noaa_19)
     case("MET"); sat2="M2"; path = trim(path_metop)
     case default; Stop "Error - get_AMSU_orbit_file_names(): Invalid value for sat3"
  end select      

  ! Get previous day
  call previous_day(jday,year, previous_jday, previous_jday_year)


  name_vec = ""

  ! Count and save .a files in list with specified sat3, year and jday. Also files in previous
  ! day that cross into current day.
  icount=0
  do i=1,num_files
     buffer = all_orbit_files(i)

     ! Extract sat (the _e is for "extracted")
     sat2_e = buffer(1:2)

     ! Extract year (this is a 2-digit year). Convert to 4 digit year.
     read(buffer,'(4x,i2)') year_e
     if( year_e>90) then
        year_e = year_e + 1900
     else
        year_e = year_e + 2000
     end if   

     ! Extract jday
     read(buffer,'(6x,i3)') jday_e


     ! Extract 1st digit of starting hour and stopping hour
     read(buffer,'(11x,i1,5x,i1)') start_hour, stop_hour

     ! Keep file if matches specified sat/day/year
     if(sat2_e==sat2 .and. year_e==year .and. jday_e==jday)then
        icount = icount + 1
        if(icount>max_num_orbit_files) stop "Error - get_AMSU_orbit_file_names(): Number of orbit files greater than max allowed"
        name_vec(icount) = trim(buffer)
     end if   

     ! Keep file if matches sat and previous jday/year and crosses midnight.
     ! Example orbit file name: NK.D04001.S0000.E0153.B2928586.GC.a
     if(sat2_e==sat2 .and. year_e==previous_jday_year .and. jday_e==previous_jday .and. start_hour==2 .and. stop_hour==0)then
        icount = icount + 1
        if(icount>max_num_orbit_files) stop "Error - get_AMSU_orbit_file_names(): Number of orbit files greater than max allowed"
        name_vec(icount) = trim(buffer)
     end if   

  end do

  ! Allocate and load orbit_files vector 
  if(allocated(orbit_files)) deallocate(orbit_files)
  allocate(orbit_files(icount))
  orbit_files(:) = trim(path)//name_vec(1:icount)

end subroutine get_AMSU_orbit_file_names
!==================================================================================

! Routine for reading .a files made from Nesdis orbit files
subroutine read_AMSU_A_orbitfile( filename,     &
                                  nscan,        &
                                  iyyyyddd,     &
                                  year,         &
                                  day_of_year,  &
                                  time_of_day,  &
                                  lat,          &
                                  lon,          &
                                  Tb,           &
                                  cold_ref_Tb,  &
                                  warm_ref_Tb,  &
                                  view_angle,   &
                                  error_flag    )
  implicit none

  ! Parameters
  integer,parameter ::  num_channels=15, num_beam_pos=30

  ! Arugment list variables.
  ! All output arrays are allocated within this routine.
  character(len=150) :: filename          ! Input:  Orbit file to read (including path)
  integer :: nscan                        ! Output: Number of scans
  integer,allocatable :: iyyyyddd(:)      ! Output: Compact format for year/day_of_year (i.e. 1998361)
  integer,allocatable :: year(:)          ! Output: Scan line year (4-digit) (e.g. 1998)                           
  integer,allocatable :: day_of_year(:)   ! Output: Scan line day of year (e.g. 365)
  real(8),allocatable :: time_of_day(:)   ! Output: Time of day (secs)
  real(4),allocatable :: lat(:,:)         ! Output: latitude (degrees,lat:-90->+90)
  real(4),allocatable :: lon(:,:)         ! Output: longitude (degrees,lon:-180->+180)
  real(4),allocatable :: Tb(:,:,:)        ! Output: Brightness temperature (K)  
  real(4),allocatable :: cold_ref_tb(:,:) ! Output: Cold ref Tb (K) (avg of 1 and 2) 
  real(4),allocatable :: warm_ref_tb(:,:) ! Output: Warm ref Tb (K) (avg of 1 and 2)
  real(4),allocatable :: view_angle(:,:)  ! Output: Effective view angles (deg) at surface
  logical             :: error_flag       ! Output: set to TRUE if error encountered, FALSE otherwise

  ! Internal variables
  integer :: iscan,i,j

  ! Open file
  open(1199,file=filename,form="unformatted",action="read",access="stream",convert="big_endian",status="old",err=899)  ! Linux on PC

  ! Read number of scans
  read(1199,err=899) nscan

  ! Allocate arrays
  if(allocated(iyyyyddd   )) deallocate(iyyyyddd   )
  if(allocated(year       )) deallocate(year       )
  if(allocated(day_of_year)) deallocate(day_of_year)
  if(allocated(time_of_day)) deallocate(time_of_day)
  if(allocated(lat        )) deallocate(lat        )
  if(allocated(lon        )) deallocate(lon        )
  if(allocated(Tb         )) deallocate(Tb         )
  if(allocated(cold_ref_tb)) deallocate(cold_ref_tb)
  if(allocated(warm_ref_tb)) deallocate(warm_ref_tb)
  if(allocated(view_angle )) deallocate(view_angle )    

  allocate(iyyyyddd(nscan)                     )
  allocate(year(nscan)                         )
  allocate(day_of_year(nscan)                  )
  allocate(time_of_day(nscan)                  )
  allocate(lat(num_beam_pos,nscan)             )
  allocate(lon(num_beam_pos,nscan)             )
  allocate(Tb(num_channels,num_beam_pos,nscan) )
  allocate(cold_ref_tb(num_channels,nscan)     )
  allocate(warm_ref_tb(num_channels,nscan)     )
  allocate(view_angle(num_beam_pos,nscan)      )

  do iscan=1,nscan

     ! read year (4 digit), day_of_year, time (of day - secs)
     read(1199,err=899,end=899)year(iscan),day_of_year(iscan),time_of_day(iscan)
     iyyyyddd(iscan) = year(iscan)*1000 + day_of_year(iscan)

     ! Read ref_Tb's for each channel
     read(1199,err=899,end=899) (cold_ref_Tb(j,iscan),j=1,num_channels),(warm_ref_Tb(j,iscan),j=1,num_channels)

     ! Read latitudes for each beam position
     read(1199,err=899,end=899) (lat(j,iscan),j=1,num_beam_pos)

     ! Read longitudes for each beam position
     read(1199,err=899,end=899) (lon(j,iscan),j=1,num_beam_pos)

     ! Read Tb's for each channel and beam position
     read(1199,err=899,end=899) ((Tb(i,j,iscan),i=1,num_channels),j=1,num_beam_pos)

     ! Read view angles for each channel and beam position
     read(1199,err=899,end=899) (view_angle(j,iscan),j=1,num_beam_pos)

  end do

  ! Normal shutdown and return
  error_flag = .FALSE.
  close(1199)   
  return

  ! Read error handling and return
  899 error_flag = .TRUE.
  close(1199)
  return

end subroutine read_AMSU_A_orbitfile
!==================================================================================

function leap_year( year )result(flag)
  integer :: year ! Year to check (e.g. 1998, 2001)
  logical :: flag ! Returned value, TRUE if leap year, FALSE otherwise.

  flag = .FALSE.
  if( (year/4)*4 == year) then
     flag = .TRUE.
     if( (year/100)*100 == year) then
        flag = .FALSE.
        if( (year/400)*400 == year) then
           flag = .TRUE.
        end if
     end if
  end if   

end function leap_year
!================================================================================== 

! Given a specified jday and year, this routine computes the jday and 
! year of the previous day.
subroutine previous_day(jday, year, previous_jday, previous_jday_year)
  integer :: jday, year ! Input: specified julian day(1..366) and 4-digit year
  integer :: previous_jday, previous_jday_year ! Output: jday and year of previous julian day.

  integer :: num_days_in_year  ! Number of days in current year
  logical :: flag

  ! Verify jday is within range
  if( jday<1 ) STOP "Error - previous_day(): jday < 1"
  if( leap_year(year) ) then
     if( jday > 366 ) STOP "Error - previous_day(): jday > 366"
  else
     if( jday > 365 ) STOP "Error - previous_day(): jday > 365"     
  end if   
        
  ! Compute jday/year for previous day
  if(jday==1) then
     previous_jday_year = year - 1
     if( leap_year(previous_jday_year) ) then
        previous_jday = 366
     else
        previous_jday = 365
     end if
  else
     previous_jday = jday - 1
     previous_jday_year = year
  end if   

end subroutine previous_day
!==================================================================================

! Routine to convert string to lower case.

elemental subroutine lower_case(word)
  implicit none
  ! convert a word to lower case
  character (len=*) , intent(in out) :: word
  integer                            :: i,ic,nlen
  nlen = len(word)
  do i=1,nlen
     ic = ichar(word(i:i))
     if (ic >= 65 .and. ic < 90) word(i:i) = char(ic+32)
  end do
end subroutine lower_case 


END MODULE AMSU_MODULE
