!====================================================================================================
! DIRECTORY: ver6/amsu_v6/
! FILE NAME: main.f90
!
! PURPOSE: Fortran 95 main program
!
! DESCRIPTION: Main program that computes and writes monthly gridded AMSU anomaly files.
!
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:  File whose name is in parameter variable orbit_file_list, set in routine amsu_module.mod.f90.
!          This file lists the "archived" satellite orbit files available for processing. Only those orbit files 
!          needed for the month or months being processed are used. The .a at the end of the name means "archived".
!          Example archived orbit file name: NK.D05275.S0956.E1140.B3840203.WI.a
!
!
! OUTPUTS: Monthly gridded anomaly files for each satellite are placed in directory ./monthly_grids. Example
!          file names are: amsu_n15_2015_06.dat, amsu_n19_2011_12.dat.
!
! MODIFICATION HISTORY:
!
!====================================================================================================

program main
  use AMSU_MODULE
  use CALENDAR
  use CROSSING_TIME
  use MEDIAN_MODULE

  implicit none

  integer,parameter :: num_angles=15
  integer,parameter :: num_channels_used = 4 ! Channels 5,6,7,9

  integer :: start_year, end_year, start_month, end_month

  character(len=150),parameter :: outpath = "./monthly_grids/" ! Path to output grid files
  character(len=150) :: outfile

  type(day_of_data) :: x

  integer :: imonth1,imonth2,jday1,jday2
  integer :: iyear,imonth,jday
  
  character(len=3) :: csat,sat_label
  integer :: vec3(3)

  logical :: status ! Flag returned by read_aqua and read_amsu. Returned as "T" when data is
                    ! found and returned. Returned as "F" otherwise. Any input value is not used.

  integer :: num_args,ichannel

  real(8),allocatable :: grid(:,:,:,:)  ! Tb grid. Dims are (nlat,nlon, 15 angles, channels 5,6,7,9)
  integer(2),allocatable :: igrid(:,:,:,:) ! Number observations grid. Dims are (nlat,nlon, 15 angles, channels 5,6,7,9)

  logical,allocatable :: used_flag(:,:) ! Dims are (jday1:jday2,time-of-day)

  integer :: iscan,channel,iprint,iangle,ilat,ilon,time_of_day,ilon2,ilon3,ilon_start,ilon_stop
  real :: lat,lon
  real :: tb

  character(len=4) :: args(5)

  real :: day_crossing_time
  real :: day_ct_vec(31)  ! Holds the crossing times for current month
  integer :: ii
  real(4) :: eq_cr_time

  !======================
  ! Read input argument
  !======================
  num_args = command_argument_count()
  if(num_args/=5) stop "Error - Usage: a.out satellite start_year start_month end_year end_month"
  call get_command_argument(1,args(1)) ! csat
  call get_command_argument(2,args(2)) ! start_year
  call get_command_argument(3,args(3)) ! start_month
  call get_command_argument(4,args(4)) ! end_year
  call get_command_argument(5,args(5)) ! end_month

  csat = trim(args(1))
  read(args(2),*) start_year
  read(args(3),*) start_month
  read(args(4),*) end_year
  read(args(5),*) end_month

  write(6,*) "Args: ",trim(csat), start_year, start_month, end_year, end_month ! Years are 4-digit

  if(ALL(csat/=(/"N15","N16","N18","N19","AQU","MET"/))) stop "Error: input sat arg must be one of N15,N16,N18,N19,AQU,MET"

  select case(csat)
     case("N15"); x%sat = "N15"; x%ksat = 10 ! Last column is Roy's AMSU satellite designations
     case("N16"); x%sat = "N16"; x%ksat = 11
     case("N18"); x%sat = "N18"; x%ksat = 13
     case("N19"); x%sat = "N19"; x%ksat = 14
     case("AQU"); x%sat = "AQU"; x%ksat = 21
     case("MET"); x%sat = "MET"; x%ksat = 20
     case default; STOP "Error - csat must be one of N15, N16, N18, N19, AQU, MET"
  end select

  YEAR_LOOP: do iyear = start_year, end_year
     imonth1=1
     imonth2= 12
     if(iyear==start_year) imonth1 = start_month
     if(iyear==end_year  ) imonth2 = end_month

     MONTH_LOOP: do imonth=imonth1,imonth2
        ! Get the start and end julian day for the current month
        jday1 = first_of_month(iyear,imonth)
        jday2 = last_of_month(iyear,imonth)

        ! Set flag array used to handle orbit file overlap
        if(allocated(used_flag)) deallocate(used_flag)
        allocate(used_flag(jday1:jday2,0:24*3600))
        used_flag = .FALSE.

        ! Allocate grids for current month
        if(allocated(grid)) deallocate(grid)
        allocate(grid(nlat,nlon,num_angles,num_channels_used))
        if(allocated(igrid)) deallocate(igrid)
        allocate(igrid(nlat,nlon,num_angles,num_channels_used))

        grid = 0
        igrid = 0

        ii = 0 ! Initialize day counter
        day_ct_vec(:) = badvalue ! Initialize vector used to accumulate the daily crossing times for the month.

        DAY_LOOP:  do jday=jday1,jday2
           ii = ii+1  ! Counter of days in month

           x%year = iyear  ! 4-digit year
           x%jday = jday

           if(x%sat == "AQU") then
              call read_aqua( x, status )
           else
              call read_amsu( x, status )
           end if

           if(status ) then
              write(6,'(a3,1x,i4.4,1x,i2.2,1x,i3.3,2x,a)') x%sat,iyear,imonth,jday, " -- FILES FOUND"
           else 
              write(6,'(a3,1x,i4.4,1x,i2.2,1x,i3.3,2x,a)') x%sat,iyear,imonth,jday, " -- NO FILES FOUND"
              cycle
           end if

           SCAN_LOOP: do iscan=1,num_scans
              time_of_day = INT(x%time_of_day(iscan)) 
              if(time_of_day==badvalue .or. time_of_day<0 .or. time_of_day>86400) cycle

              if(used_flag(x%jday,time_of_day)) cycle

              CHANNEL_LOOP: do ichannel=1,4
                 
                 select case(ichannel)
                    case(1); channel=5
                    case(2); channel=6
                    case(3); channel=7
                    case(4); channel=9
                 end select   

                 PRINT_LOOP: do iprint=1,num_beam_pos

                    if(iprint<=num_angles) then
                       iangle = iprint
                    else
                       iangle = num_beam_pos+1-iprint
                    end if

                    tb = x%Tb(channel,iprint,iscan)
                    if(tb==badvalue) cycle

                    lat = x%lat(iprint,iscan)
                    lon = x%lon(iprint,iscan)
                    if(lat==badvalue .or. lon==badvalue) cycle
                    if(lat==0.0 .and. lon==0.0) cycle
                    if(lat<-90. .or. lat>90. .or. lon<-180. .or. lon>190.) cycle

                    ilat = floor( (lat + 90.)/2.5) + 1
                    ilon = floor( (lon +180.)/2.5) + 1
                    if(ilon==nlon+1) ilon = 1
                    if(ilat==nlat+1) ilat = nlat

                    ! Add tb to grid cells.
                    ! Near poles the tb values are replicated over multiple cells to account for the smaller cell size (in latitude).
                    if(ilat<=2 .or. ilat>=71) then
                       select case(ilat)
                          case(1,72); ilon_start=ilon-4; ilon_stop=ilon+4
                          case(2,71); ilon_start=ilon-1; ilon_stop=ilon+1
                          case default; stop "Error: This statment should never be executed"   
                       end select      

                       do ilon2=ilon_start,ilon_stop
                          ilon3 = ilon2
                          if(ilon3<1) then 
                             ilon3 = nlon + ilon3
                          elseif(ilon3>nlon) then
                             ilon3 = ilon3 - nlon
                          end if

                          grid(ilat,ilon3,iangle,ichannel) = grid(ilat,ilon3,iangle,ichannel) + tb
                          igrid(ilat,ilon3,iangle,ichannel) = igrid(ilat,ilon3,iangle,ichannel) + 1
                       end do   
                    else
                       grid(ilat,ilon,iangle,ichannel) = grid(ilat,ilon,iangle,ichannel) + tb
                       igrid(ilat,ilon,iangle,ichannel) = igrid(ilat,ilon,iangle,ichannel) + 1
                    end if   

                 end do PRINT_LOOP

              end do CHANNEL_LOOP

              used_flag(x%jday,time_of_day) = .TRUE.

           end do SCAN_LOOP

           ! Compute equatorial crossing time of ascending pass (based on print 15). Routine returns median value.
           call compute_equatorial_crossing_time( x%lat(15,:), x%lon(15,:), real(x%time_of_day(:)), badvalue, day_crossing_time )
           day_ct_vec(ii) = day_crossing_time

        end do DAY_LOOP

        ! Cycle if no data found within the month
        if(ALL(igrid==0)) cycle

        ! Finalize grids
        where(igrid>0)
           grid = grid/igrid
        elsewhere
           grid = badvalue
        end where   
        
        ! Get the crossing time for the month. Use the median of 
        ! the daily crossing times.
        call find_median(day_ct_vec,eq_cr_time,badvalue)

        write(6,*) "equatorial crossing time = ",eq_cr_time

        ! Create output file name (e.g. amsu_n15_1985_07.dat)
        sat_label = csat
        call lower_case(sat_label)
        write(outfile,'("amsu_",a3,"_",i4.4,"_",i2.2,".dat")') sat_label,iyear,imonth

        ! Write grids to file
        open(11,file=trim(outpath)//trim(outfile),status="replace",form="unformatted",access="stream")
        write(11) eq_cr_time
        write(11) real(grid,4)
        write(11) igrid
        close(11)

     end do MONTH_LOOP
  end do YEAR_LOOP

  close(88)

end program main

