!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: calendar.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION: This module contains routines for peforming various date and calendar operations.
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

module calendar
private
public absolute_date, days_in_month, gregorian_date, day_of_year, days_in_year, days_from_ref
public secs_from_ref, date_from_secs, day_of_week, month_day_from_jday, yesterday, tomorrow, leap_year
public trenberth_day
public first_of_month, last_of_month, get_julian_days_in_month
public current_month, next_month, previous_month
public fraction_of_month_from_1st, fraction_of_month_to_end


INTERFACE absolute_date
   MODULE PROCEDURE absolute_date_3, absolute_date_2
END INTERFACE

INTERFACE secs_from_ref
   MODULE PROCEDURE secs_from_ref_1, secs_from_ref_2
END INTERFACE

contains

!--------------------------------------------------------------------
! This program returns a vector of the julian days in the specified year/month.
! The jday vector is allocated within the routine.

subroutine get_julian_days_in_month( year, month, jday_vec )
  integer :: year  ! Input: 4-digit year
  integer :: month ! Input: month
  integer,allocatable :: jday_vec(:) ! Output: returned vector of julian day (allocated within routine).

  integer :: num_days, first, i, jday

  num_days = days_in_month(year,month)
  
  if(allocated(jday_vec)) deallocate(jday_vec)
  allocate(jday_vec(num_days))

  ! Get julian day of first day of month
  first = day_of_year(year,month,1)

  jday = first
  do i=1,num_days
     jday_vec(i) = jday
     jday = jday + 1
  end do   

end subroutine get_julian_days_in_month

!--------------------------------------------------------------------
! Returns number of seconds to specified time beginning with 00Z on Jan 1 of
! the given reference year.

function secs_from_ref_2(time_string,ref_year) result( nsecs )
   implicit none
   character(len=17),intent(IN) :: time_string  ! Format is "year:jday:hour:minute:second", e.g. "1998:243:09:31:58"
   integer,          intent(IN) :: ref_year
   integer                      :: nsecs


   integer :: year,jday,hour,minute,second

   ! Dissect time string
   read(time_string,10) year,jday,hour,minute,second
   10 format(i4,1x,i3,1x,i2,1x,i2,1x,i2)

   ! Verify values
   if(year<1 .or. ref_year<1 .or. ref_year>year) then
      stop "Error - secs_from_ref(): Invalid year or ref_year"
   endif

   ! Verify jday,hour,minute,second
   if(jday  <1 .or. jday  >days_in_year(year) .or.  &
      hour  <0 .or. hour  >23                 .or.  &
      minute<0 .or. minute>59                 .or.  &
      second<0 .or. second>59                       ) then
      stop "Error - secs_from_ref(): Invalid jday, hour, minute, or second"
   endif

   nsecs = (absolute_date(year,jday) - absolute_date(ref_year,1))*86400 + hour*3600 + minute*60 + second

end function secs_from_ref_2

!--------------------------------------------------------------------
! Returns number of seconds to specified time beginning with 00Z on Jan 1 of
! the given reference year.


function secs_from_ref_1(year,jday,hour,minute,second,ref_year) result( nsecs )
   implicit none
   integer,intent(IN) :: year,jday,hour,minute,second,ref_year
   integer            :: nsecs

   ! Verify years
   if(year<1 .or. ref_year<1 .or. ref_year>year) then
      stop "Error - secs_from_ref(): Invalid year or ref_year"
   endif

   ! Verify jday,hour,minute,second
   if(jday  <1 .or. jday  >days_in_year(year) .or.  &
      hour  <0 .or. hour  >23                 .or.  &
      minute<0 .or. minute>59                 .or.  &
      second<0 .or. second>59                       ) then
      stop "Error - secs_from_ref(): Invalid jday, hour, minute, or second"
   endif

   nsecs = (absolute_date(year,jday) - absolute_date(ref_year,1))*86400 + hour*3600 + minute*60 + second

end function secs_from_ref_1

!--------------------------------------------------------------------
! Returns the number of days since Jan 1 of 1978.
! Jan 1 1978 has a trenberth day of 0. Jan 2 1978 is 1 etc.
function trenberth_day(year,jday) result( tday )
   implicit none
   integer,intent(IN) :: year,jday
   integer            :: tday

   tday = days_from_ref(year,jday,1978) - 1

end function trenberth_day

!--------------------------------------------------------------------
! Returns number of days to current day beginning with Jan 1 of
! the given reference year (i.e. Jan 1 of reference year is day 1).


function days_from_ref(year,jday,ref_year) result( ndate )
   implicit none
   integer,intent(IN) :: year,jday,ref_year
   integer            :: ndate

   ! Verify years
   if(year<1 .or. ref_year<1 .or. ref_year>year) then
      stop "Error - days_from_ref(): Invalid year or ref_year"
   endif

   ! Verify day
   if(jday<1 .or. jday>days_in_year(year) ) then
      stop "Error - days_from_ref(): Invalid julian day"
   endif

   ndate = absolute_date(year,jday) - absolute_date(ref_year,1) + 1


end function days_from_ref

!--------------------------------------------------------------------
! Returns number of days since the day before 1 Jan 1 A.D., assuming
! the Gregorian calendar. 
! Uses 4-digit year.

function absolute_date_3(year,month,day) result( adate )
   implicit none
   integer year,month,day,adate

!..Internal declarations
   integer n,imonth

!..Verify year and month
   if(year<1 .or. month<1 .or. month>12) then
      stop "Error - absolute_date_3(): Invalid year or month--"
   endif

!..Verify day
   if(day<1 .or. day>days_in_month(year,month) ) then
      stop "Error - absolute_date_3(): Invalid day--"
   endif

!..Days so far in this month
   n = day

!..Add days in prior months this year
   do imonth=month-1,1,-1
      n=n+days_in_month(year,imonth)
   enddo

!..Add days for prior years
   n = n + 365 * (year - 1)  &  ! days in previous years ignoring leap days
         + (year - 1)/4      &  ! Julian leap days before this year...
         - (year - 1)/100    &  ! ...minus prior century years...
         + (year - 1)/400       ! ...plus prior years divisible by 400

   adate=n
end function absolute_date_3

!--------------------------------------------------------------------
! Returns number of days since the day before 1 Jan 1 A.D., assuming
! the Gregorian calendar. 
! Uses 4-digit year.

function absolute_date_2(year,jday) result( adate )
   implicit none
   integer year,jday,adate ! jday is Julian day

!..Internal declarations
   integer n

!..Verify year
   if(year<1) then
      stop "Error - absolute_date_2(): Invalid year--"
   endif

!..Verify day
   if(jday<1 .or. jday>days_in_year(year) ) then
      stop "Error - absolute_date_2(): Invalid julian day--"
   endif

!..Days so far in this year
   n = jday

!..Add days for prior years
   n = n + 365 * (year - 1)  &  ! days in previous years ignoring leap days
         + (year - 1)/4      &  ! Julian leap days before this year...
         - (year - 1)/100    &  ! ...minus prior century years...
         + (year - 1)/400       ! ...plus prior years divisible by 400

   adate=n
end function absolute_date_2

!--------------------------------------------------------------------

function leap_year( year ) result( flag )
   integer :: year ! 4 digit year
   logical :: flag ! Set to TRUE if leap year, FALSE otherwise.

   if( days_in_month(year, 2) == 29 ) then
      flag = .TRUE.
   else
      flag = .FALSE.
   end if

end function leap_year
!--------------------------------------------------------------------
function days_in_month(year,month) result (ndays)
   implicit none
   integer year,month,ndays

!..Verify year and month
   if(year<1 .or. month<1 .or. month>12) then
      stop "Error - days_in_month(): Invalid year or month--"
   endif

   select case(month)
      case(1,3,5,7,8,10,12)
         ndays=31
      case(4,6,9,11)
         ndays=30
      case(2)
         if (((mod(year,4) == 0) .and. (mod(year,100) /= 0)) &
            .or. (mod(year,400) == 0)) then
            ndays=29
         else
            ndays=28
         endif
   end select

end function days_in_month

!--------------------------------------------------------------------
function days_in_year(year) result (ndays)
   implicit none
   integer year,ndays

   if(year<1) then
      stop "Error - days_in_year(): Invalid year"
   endif

   if( days_in_month(year,2) == 29 ) then
      ndays=366
   else
      ndays=365
   end if

end function days_in_year

!--------------------------------------------------------------------
function gregorian_date( adate ) result (date_vec)
   implicit none
   integer adate  ! absolute date
   integer,dimension(3) :: date_vec  ! year,month,day

!..Internal declarations
   integer year,month,day

!.. Verify date
   if( adate<1 ) stop "Error - gregorian_date(): Invalid absolute date given as input"


!---------------------------------------------
! Compute year,month, day from absolute date
!---------------------------------------------
!..Search forward year by year from approximate year
   year = adate/366
   do while( adate >= absolute_date(year+1,1,1) )
      year=year+1
   enddo

!..Search forward month by month from January
   month = 1
   do while( adate > absolute_date(year,month,days_in_month(year,month)) )
      month=month+1
   enddo

!..Get day
   day = adate - absolute_date(year,month,1) + 1

!..Load results
   date_vec(1) = year
   date_vec(2) = month
   date_vec(3) = day

end function gregorian_date


!--------------------------------------------------------------------
! Returns the julian day of the specified date.
! Uses 4-digit year.

function day_of_year(year,month,day) result( adate )
   implicit none
   integer year,month,day,adate

!..Internal declarations
   integer n,imonth

!..Verify year and month
   if(year<1 .or. month<1 .or. month>12) then
      stop "Error - day_of_year(): Invalid year or month--"
   endif

!..Verify day
   if(day<1 .or. day>days_in_month(year,month) ) then
      stop "Error - day_of_year(): Invalid day--"
   endif

!..Days so far in this month
   n = day

!..Add days in prior months this year
   do imonth=month-1,1,-1
      n=n+days_in_month(year,imonth)
   enddo

   adate=n
end function day_of_year

!--------------------------------------------------------------------
! Returns the year, jday, and secs in jday for an
! inputed number of seconds since 0Z on Jan 1 of the input reference year

function date_from_secs( secs,ref_year ) result( date_vec )
   integer(8) :: secs         ! secs since 0Z on Jan 1 of reference year (ref_year)
   integer    :: ref_year     ! 4-digit reference year
   integer    :: date_vec(3)  ! Output: (4-digit year,jday,secs in jday)

   ! Internal variables
   integer :: abs_date_of_ref ! Absolute date of reference year
   integer :: abs_date        ! Absolute date of day represented by input secs and input ref_year
   integer :: secs_in_day     ! Number of seconds into current day
   integer :: year,month,day,jday
   integer :: ivec(3)

   ! Get absolute date of Jan 1 of reference year
   abs_date_of_ref = absolute_date(ref_year,1) 

   ! Compute absolute date represented by input secs and input reference year
   abs_date = abs_date_of_ref + secs/86400

   ! Compute number of seconds into the current day
   secs_in_day = MOD(secs,86400)

   ! Compute year, month, and day of abs_date
   ivec = gregorian_date(abs_date)  ! ivec ends up with (year,month,day)
   year = ivec(1)
   month= ivec(2)
   day  = ivec(3)

   ! Compute jday
   jday = day_of_year(year,month,day)

   ! Load date_vec for return
   date_vec(1) = year
   date_vec(2) = jday
   date_vec(3) = secs_in_day

end function date_from_secs

!--------------------------------------------------------------------

! This program uses Zeller's Rule to find the day of the week for any
! given date.
! Ref: http://mathforum.org/dr.math/faq/faq.calendar.html
subroutine day_of_week( year, month, day, dow)
   implicit none
   integer :: year, month, day ! Input : year (4 digit), month (1..12), day of month (1...31)
   character(len=3) :: dow     ! Output: day of week, one of SUN, MON, etc.

   integer :: f,d,m,c,ii

   character(len=3) :: days_of_week(0:6) = (/"SUN","MON","TUE","WED","THU","FRI","SAT"/)

   if(month == 1) then
      month = 11
      year = year - 1
   else if(month==2) then
      month = 12
      year = year - 1
   else
      month = month - 2
   end if
   
   m = month            

   d = MOD(year,100)
   c = year/100

   f = day + (13*m-1)/5 + d + d/4 + c/4 - 2*c
   ii = MOD(f,7)

   if(ii<0) ii = ii + 7

   dow = days_of_week(ii)

end subroutine day_of_week

!--------------------------------------------------------------------
function month_day_from_jday(year, jday) result( date_vec )
   integer :: year, jday   ! Input: 4-digit year, jday   
   integer :: date_vec(3)  ! Output: 4-digit year,month,day

   integer :: adate

   adate = absolute_date(year, jday)
   date_vec = gregorian_date( adate ) 

end function month_day_from_jday
!--------------------------------------------------------------------
! Input year, month, day for today and it returns date_vec for yesterday.
function yesterday( year, month, day ) result(date_vec)
   integer :: year, month, day ! Input: 4 digit year, month, day
   integer :: date_vec(3)      ! Output: 4-digit year,month,date
   integer :: adate            ! Absolute date (days)

   adate = absolute_date_3(year, month, day)
   adate = adate - 1                            ! Move to yesterday
   date_vec(:) = gregorian_date(adate)          ! Get date of yesterday

end function yesterday

! Input year, month, day for today and it returns date_vec for tomorrow.
function tomorrow( year, month, day ) result(date_vec)
   integer :: year, month, day ! Input: 4 digit year, month, day
   integer :: date_vec(3)      ! Output: 4-digit year,month,date
   integer :: adate            ! Absolute date (days)

   adate = absolute_date_3(year, month, day)
   adate = adate + 1                            ! Move to tomorrow
   date_vec(:) = gregorian_date(adate)          ! Get date of tomorrow

end function tomorrow


!--------------------------------------------------------------------
! Returns the julian day of the 1st day of the specified year, month.
function first_of_month(year, month) result( jday )
  integer :: year, month ! Input: 4 digit year, month
  integer :: jday        ! Output: computed julian day of first day of specified month

  jday = day_of_year(year,month,1)

end function first_of_month

!--------------------------------------------------------------------
! Returns the julian day of the last day of the specified year, month.
function last_of_month(year, month) result( jday )
  integer :: year, month ! Input: 4 digit year, month
  integer :: jday        ! Output: computed julian day of first day of specified month

  integer :: num_days, first

  num_days = days_in_month(year, month)
  first = day_of_year(year,month,1)
  jday = first + num_days - 1

end function last_of_month  

!--------------------------------------------------------------------
! Returns the month containing the specified julian day/year.
function current_month(year,jday) result(month)
  integer :: year, jday ! Input: 4 digit year, julian day
  integer :: month      ! Output: computed month

  integer :: date_vec(3)

  date_vec = month_day_from_jday(year, jday) ! Returns 4-digit year, month, day

  month = date_vec(2)

end function current_month

!--------------------------------------------------------------------
! Returns the year/month following the month containing the specified
! julian day/year.
function next_month(year,jday) result(date_vec)
  integer :: year, jday ! Input: 4 digit year, julian day
  integer :: date_vec(2)! Output: computed (year,month) for next month

  integer :: month
  month = current_month(year,jday)

  if(month==12) then
     date_vec(1) = year+1
     date_vec(2) = 1
  else   
     date_vec(1) = year
     date_vec(2) = month+1
  end if   

end function next_month
!--------------------------------------------------------------------
! Returns the year/month prior to the month containing the specified
! julian day/year.
function previous_month(year,jday) result(date_vec)
  integer :: year, jday ! Input: 4 digit year, julian day
  integer :: date_vec(2)! Output: computed (year,month) for next month

  integer :: month
  month = current_month(year,jday)

  if(month==1) then
     date_vec(1) = year-1
     date_vec(2) = 12
  else   
     date_vec(1) = year
     date_vec(2) = month-1
  end if   

end function previous_month
!--------------------------------------------------------------------
function fraction_of_month_from_1st(year, jday) result(frac)
  integer :: year, jday ! Input: 4 digit year, julian day
  real    :: frac       ! Output: fraction of month from the 1st of month to and including current day
  
  integer :: day_of_month, month, num_days
  integer :: vec(3)

  vec(:) = month_day_from_jday(year, jday) ! Returns year, month, day-of-month
  day_of_month = vec(3)
  month = vec(2)

  num_days = days_in_month(year, month) ! Number of days in month

  frac = real(day_of_month)/real(num_days)

end function fraction_of_month_from_1st
!--------------------------------------------------------------------
function fraction_of_month_to_end(year, jday) result(frac)
  integer :: year, jday ! Input: 4 digit year, julian day
  real    :: frac       ! Output: fraction of month from (and including current day) to end of month

  integer :: day_of_month, month, num_days
  integer :: vec(3)

  vec(:) = month_day_from_jday(year, jday) ! Returns year, month, day-of-month
  day_of_month = vec(3)
  month = vec(2)

  num_days = days_in_month(year, month) ! Number of days in month

  frac = real(num_days-day_of_month+1)/real(num_days)

end function fraction_of_month_to_end
!--------------------------------------------------------------------

end module calendar

!! MAIN PROGRAM FOR TESTING
!Program main
!   use calendar
!   implicit none
!   integer i,gdate(3),adate,y,m,d,count, jday, year
!
!   write(6,*)"May 1 1993     =",absolute_date(1993,5,1) -absolute_date(1992,12,31)
!   write(6,*)"May 31 1993    =",absolute_date(1993,5,31)-absolute_date(1992,12,31)
!   write(6,*)"August 1 1993  =",absolute_date(1993,8,1) -absolute_date(1992,12,31)
!   write(6,*)"August 31 1993 =",absolute_date(1993,8,31)-absolute_date(1992,12,31)
!
!   write(6,*)"January 1 1993 =",absolute_date(1993,1,1)-absolute_date(1992,12,31)
!   write(6,*)"Dec 31 1993    =",absolute_date(1993,12,31)-absolute_date(1992,12,31)
!
!
!   write(6,*)"Dec 31 1900    =",absolute_date(1900,12,31)-absolute_date(1899,12,31)
!   write(6,*)"Dec 31 2000    =",absolute_date(2000,12,31)-absolute_date(1999,12,31)
!   write(6,*)"Dec 31 1996    =",absolute_date(1996,12,31)-absolute_date(1995,12,31)
!
!   
!   count=0
!   do y=1,3000
!   do m=1,12
!   do d=1,days_in_month(y,m)
!      count=count+1
!      adate=absolute_date(y,m,d)
!      gdate=gregorian_date(adate)
!      if(y/=gdate(1) .or. m/=gdate(2) .or. d/=gdate(3) .or.  &
!         count/=adate) stop 'ERROR: unexpected value'
!   enddo
!   enddo
!   enddo
!
!   do 
!      !y = 2012
!      !jday = 1
!      write(6,*) "year, jday? "
!      read(5,*) year,jday
!      write(6,*) year,jday
!      write(6,*) "current month = ",current_month(year,jday)
!      write(6,*) "next month = ",next_month(year,jday)
!      write(6,*) "previous month = ",previous_month(year,jday)
!      write(6,10) "frac of month since 1st: ",fraction_of_month_from_1st(year,jday)
!      write(6,10) "frac of month remaining: ",fraction_of_month_to_end(year,jday)
!      10 format(a,f5.3)
!   end do
!
!
!end

