!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: interpolate.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION: Collection of various interpolation routines and supporting routines.
!
!
! AUTHOR(S): Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE INTERPOLATE
implicit none
integer,parameter :: k12=selected_real_kind(12)
private :: k12

interface linear_interpolate
   module procedure linear_interpolate_scalar,linear_interpolate_vector
end interface

interface log_interpolate
   module procedure log_interpolate_scalar,log_interpolate_vector
end interface


CONTAINS

!--------------------------------------------------------
subroutine fill_missing_data_2d(arr, badvalue) 
  real(8) :: arr(:,:) ! The array to be filled. It is assumed that varible 2 varies fastest (e.g. dims of (year, month))
  real(8) :: badvalue

  integer :: n,n1,n2,i,i1,i2
  real(8),allocatable :: fvec(:)

  n1=size(arr,1)
  n2=size(arr,2)
  n=n1*n2

  allocate( fvec(n) )

  i=0
  do i1=1,n1
     do i2=1,n2
        i=i+1
        fvec(i) = arr(i1,i2)
     end do
  end do   

  call fill_missing_data(fvec,real(badvalue,8))


  ! Put interpolated data back into original array
  i=0
  do i1=1,n1
     do i2=1,n2
        i=i+1
        arr(i1,i2) = fvec(i)
     end do
  end do     

  deallocate(fvec)

end subroutine fill_missing_data_2d

!--------------------------------------------------------
! Linear interpolation for case where the vector f1 "wraps" on
! itself when it cross the boundaries defined by low and high.
! The valid interior region is defined to be   low <= f1 < high.

! I THINK THIS ROUTINE IS FAULTY. THE WRAP SHOULD BE ON X2 NOT F1 - WDB, 12/17/01.
! USE NEW VERSION - linear_interpolate_vector_wrap_2()

function linear_interpolate_vector_wrap( f1,x1,x2,low,high ) result( f2 )
   real(k12), dimension(:)        :: f1,x1,x2
   real(k12), dimension(size(x2)) :: f2
   real(k12)                      :: low, high ! Valid region boundaries

!..Internal variables
   integer i,n1,n2,jlower
   real(k12) df,dx
   real(k12) diff,half_diff

!..Verify input vectors conform
   if(size(f1) .ne. size(x1)) then
      stop "linear_interpolate_vector_wrap() - Nonconformal input vectors"
   endif

   n1=size(x1)
   n2=size(x2)

!..Verify high > low
   if(high <= low ) stop "linear_interpolate_vector_wrap() - Invalid low,high defining valid region"   

!..Verify f1 is confined by low and high
   if(ANY(f1<low) .or. ANY(f1>=high)) stop "linear_interpolate_vector_wrap() - f has values outside valid region"   

   diff = high - low
   half_diff = diff/2.d0

   do i=1,n2
      call locate(x1,x2(i),jlower)
      if(jlower==0 .or. jlower==n1) then
         write(6,*)"x1=",x1
         write(6,*)"i=",i
         write(6,*)"x2(i)=",x2(i)
         stop "linear_interpolate_vector_wrap(): element out of bounds"
      endif
      df = f1(jlower+1)-f1(jlower)

      ! Adjust df if wrap occurs
      if( df > half_diff ) then ! Crossed lower boundary (i.e. moving in negative direction)
         df = df - diff
      else if( df < -half_diff ) then ! Crossed upper boundary (i.e. moving in positive direction)
         df = df + diff
      end if

      dx = x1(jlower+1)-x1(jlower)
      f2(i) = f1(jlower) + df*(x2(i)-x1(jlower))/dx

      if(f2(i) >= high) then
         f2(i) = f2(i) - diff
      else if(f2(i) < low) then
         f2(i) = f2(i) + diff
      end if

   enddo

end function linear_interpolate_vector_wrap

!--------------------------------------------------------
! This version corrects the problem I think is in the above version. I.e. the 
! wrap should be on x1 not f1.

function linear_interpolate_vector_wrap_2( f1,x1,x2,low,high ) result( f2 )
   real(k12), dimension(:)        :: f1,x1,x2
   real(k12), dimension(size(x2)) :: f2
   real(k12)                      :: low, high ! Valid region boundaries

!..Internal variables
   integer i,n1,n2,jlower
   real(k12) df,dx
   real(k12) diff,half_diff
   real(k12) :: effective_x2

!..Verify input vectors conform
   if(size(f1) .ne. size(x1)) then
      stop "linear_interpolate_vector_wrap_2() - Nonconformal input vectors"
   endif

   n1=size(x1)
   n2=size(x2)

!..Verify high > low
   if(high <= low ) stop "linear_interpolate_vector_wrap_2() - Invalid low,high defining valid region"   

!..Verify f1 is confined by low and high
   !$$$if(ANY(f1<low) .or. ANY(f1>=high)) stop "linear_interpolate_vector_wrap() - f has values outside valid region"   

   !$$$diff = high - low
   !$$$half_diff = diff/2.d0

   do i=1,n2

      effective_x2 = x2(i)
      if (x2(i) >= high ) effective_x2 = x2(i) - high + low
      if (x2(i) <  low  ) effective_x2 = high - ( low - x2(i) )
      call locate(x1,effective_x2,jlower)
      if(jlower==0 .or. jlower==n1) then
         write(6,*)"x1=",x1
         write(6,*)"i=",i
         write(6,*)"x2(i)=",x2(i)
         stop "linear_interpolate_vector_wrap_2(): element out of bounds"
      endif
      df = f1(jlower+1)-f1(jlower)
      dx = x1(jlower+1)-x1(jlower)
      f2(i) = f1(jlower) + df*(effective_x2-x1(jlower))/dx

   enddo

end function linear_interpolate_vector_wrap_2


!--------------------------------------------------------
function linear_interpolate_vector( f1,x1,x2 ) result( f2 )
   real(k12), dimension(:)        :: f1,x1,x2
   real(k12), dimension(size(x2)) :: f2

!..Internal variables
   integer i,n1,n2,jlower
   real(k12) df,dx

!..Verify input vectors conform
   if(size(f1) .ne. size(x1)) then
      stop "linear_interpolate() - Nonconformal input vectors"
   endif

   n1=size(x1)
   n2=size(x2)

   do i=1,n2
      call locate(x1,x2(i),jlower)
      if(jlower==0 .or. jlower==n1) then
         write(6,*) "linear_interpolate(): lower, upper, x = ", x1(1),x1(n1), x2(i)
         stop "linear_interpolate(): element out of bounds"
      endif
      df = f1(jlower+1)-f1(jlower)
      dx = x1(jlower+1)-x1(jlower)
      f2(i) = f1(jlower) + df*(x2(i)-x1(jlower))/dx
   enddo

end function linear_interpolate_vector

!--------------------------------------------------------
function linear_interpolate_scalar( f1,x1,x2 ) result( f2 )
   real(k12), dimension(:)  :: f1,x1
   real(k12)                :: x2
   real(k12)                :: f2
   real(k12),dimension(1) :: aa,bb
   aa(1) = x2
   bb = linear_interpolate_vector(f1,x1,aa)
   f2=bb(1)
end function linear_interpolate_scalar

function log_interpolate_scalar( f1,x1,x2 ) result( f2 )
   real(k12), dimension(:)  :: f1,x1
   real(k12)                :: x2
   real(k12)                :: f2
   real(k12),dimension(1) :: aa,bb
   aa(1) = x2
   bb = log_interpolate_vector(f1,x1,aa)
   f2=bb(1)
end function log_interpolate_scalar


!--------------------------------------------------------
function log_interpolate_vector( f1,x1,x2 ) result( f2 )
   real(k12), dimension(:)        :: f1,x1,x2
   real(k12), dimension(size(x2)) :: f2

!..Internal variables
   integer i,n1,n2,jlower
   real(k12) df,dx

!..Verify input vectors conform
   n1=size(x1)
   n2=size(x2)
   if(size(f1) .ne. size(x1)) then
      stop "log_interpolate() - Nonconformal input vectors"
   endif

!..Verify x1 > 0
   n1=size(x1)
   n2=size(x2)
   do i=1,n1
      if(x1(i) <= 0) stop "log_interpolate(): negative or zero values in input vector x1 --"
   enddo

   do i=1,n2
      call locate(x1,x2(i),jlower)
      if(jlower==0 .or. jlower==n1) then
         stop "log_interpolate(): element out of bounds"
      endif
      df = f1(jlower+1)-f1(jlower)
      dx = log(x1(jlower+1))-log(x1(jlower))
      f2(i) = f1(jlower) + df*(log(x2(i))-log(x1(jlower)))/dx
   enddo

end function log_interpolate_vector


!--------------------------------------------------------
! Returns in jlower the smallest index of two (nearest)
! neighbors in xvec that bound x.
! Returns 0 or n if x is out of bounds, where n is number of elements in xvec.

subroutine locate(xvec,x,jlower)
   real(k12) xvec(:),x
   integer jlower

!..Internal variables
   integer jl,jm,ju,n
   n=size(xvec)

!..Check for ends
   if(x==xvec(1)) then
      jlower=1
      return
   else if(x==xvec(n)) then
      jlower=n-1
      return
   endif

   jl=0
   ju=n+1
   do while (ju-jl.gt.1)
      jm=(ju+jl)/2
      if((xvec(n).gt.xvec(1)).eqv.(x.gt.xvec(jm)))then
         jl=jm
      else
         ju=jm
      endif
   enddo
   jlower=jl
   return
end subroutine locate
!--------------------------------------------------------
! Use linear interpolation to fill all missing values. No extrapolation
! is done, only missing values internal to good values are replaced.
subroutine fill_missing_data( x, badvalue )
   real(k12) :: x(:)
   real(k12) :: badvalue
   integer   :: n,i,j
   real(k12) :: rlower,rupper ! Bounding values for current interpolation
   integer   :: ilower,iupper ! Indices of rlower,rupper

   real(k12),allocatable :: x2(:)

   ! At least two good values are required for interpolation.
   if(COUNT(x/=badvalue) < 2 ) return

   n=size(x)
   allocate(x2(n))
   x2 = x


   ilower=0
   iupper=0
   do i=2,n-1
      if(x(i)/=badvalue) cycle

      ! Find rlower
      rlower = badvalue
      do j=i-1,1,-1
         if(x(j)/=badvalue) then
            rlower=x(j)
            ilower=j
            exit
         end if
      end do
      if(rlower==badvalue) cycle ! No lower value found

      ! Find rupper
      rupper = badvalue
      do j=i+1,n
         if(x(j)/=badvalue) then
            rupper=x(j)
            iupper=j
            exit
         end if
      end do
      if(rupper==badvalue) cycle ! No upper value found

      ! Do the interpolation. Put result in x2.
      x2(i) = rlower + (real(i)-ilower)/(iupper-ilower) * (rupper-rlower)

   end do  

   ! Overwrite x with x2
   x = x2

   ! Clean-up
   deallocate(x2)

end subroutine fill_missing_data
!--------------------------------------------------------

function compute_layer_average(f,x, xlower, xupper, npts ) result(average)
   real(k12) :: f(:)  ! Values at levels from which layer average is computed
   real(k12) :: x(:)  ! Level values
   real(k12) :: xlower, xupper ! Levels bounding the layer for which average is computed.
   integer   :: npts  ! number of points interpolated across the layer used in computing the average (npts must be 2 or greater)
   real(k12) :: average


   real(k12) :: xnew(npts) ! Level values across the layer at which f will be evaluated in computing the average
   real(k12) :: fnew(npts) ! Values of f interpolated to xnew

   integer :: i
   real(k12) :: dx

   ! Verify x is increasing
   do i=2,size(x)
      if(x(i)<=x(i-1)) STOP "ERROR: compute_layer_averages() - input vector x must be increasing"
   end do

   if( npts<2 ) STOP "ERROR: compute_layer_averages() - npts must be >= 2"

   if( xupper <= xlower ) STOP "ERROR: compute_layer_averages() - xupper must be greater than xlower"

   ! Load xnew
   dx = (xupper - xlower)/(npts-1)
   do i=1,npts
      xnew(i) = xlower + (i-1)*dx
   end do

   ! Load fnew
   fnew(:) = linear_interpolate_vector(f,x,xnew)

   ! Compute layer average
   average = (fnew(1)*0.5 + SUM(fnew(2:npts-1)) + fnew(npts)*0.5)/(npts-1)

end function compute_layer_average
!--------------------------------------------------------

END MODULE INTERPOLATE










