!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: linear_algebra.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION: This module contains a quadratic curve fitting routine and a simplified interface
!              to the LAPACK routine DGELSS (double precision version of SGELSS) which solves the
!              linear least squares problem set up as A*X = B.
!
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE LINEAR_ALGEBRA
  implicit none

CONTAINS

! This routine fits quadratic to f vs x (e.g. f = c0 + c1*x + c2*x**2)
subroutine curve_fit(f,x,iorder,c)
  real(8),intent(IN) :: f(:)
  real(8),intent(IN) :: x(:)
  integer,intent(IN) :: iorder ! order of fit 1 <= order <= 5
  real(8),allocatable,intent(OUT) :: c(:)

  integer :: num_unknowns
  integer :: n,i
  real(8),allocatable :: A0(:,:),B0(:), x0(:)


  if(iorder<1 .or. iorder>5) stop "Error - curve_fit(): order must be >=1 and <=5"

  num_unknowns = iorder + 1

  n = size(f)
  if(size(x)/=n) stop "Error - curve_fit(): Input vectors f,x have different lengths"

  if(allocated(A0)) deallocate(A0)
  allocate(A0(n,num_unknowns))
  if(allocated(B0)) deallocate(B0)
  allocate(B0(n))
  if(allocated(X0)) deallocate(X0)
  allocate(X0(num_unknowns))
  
  do i=1,num_unknowns
     A0(:,i) = x**(i-1)
  end do   
  B0 = f

  call regress(A0,B0,X0)

  if(allocated(c)) deallocate(c)
  allocate(c(num_unknowns))
  C = X0

  deallocate(A0,B0,X0)
  
end subroutine curve_fit

! This routine is a simple interface to the LAPACK routine DGELSS (double precision version of SGELSS)
! which solves the linear least squares problem set up as A*X = B.
subroutine regress(a0,b0,x0)
   real(8),INTENT(IN)  :: a0(:,:)
   real(8),INTENT(IN)  :: b0(:)
   real(8),INTENT(OUT) :: x0(:) ! Solution vector

   integer :: m0,n0,IS

   ! Lapack variables
   integer :: M,N,NRHS,LDA,LDB,RANK,LWORK,INFO
   real(8) :: RCOND
   real(8),allocatable :: A(:,:),B(:),S(:),WORK(:)

   ! Get and check array dimensions
   m0 = size(A0,1)
   n0 = size(A0,2)
   if(size(B0)/=m0) stop "Error - regress(): Unexpected size for input vector B"
   if(size(x0)/=n0) stop "Error - regress(): Unexpected size for input vector X"

   ! Assign and allocate Lapack variables
   M = m0
   N = n0
   NRHS = 1
   LDA = M
   LDB = M
   LWORK = 3*minVal((/M,N/)) + maxVal( (/2*minVal((/M,N/)), maxVal((/M,N/)), NRHS/) ) ! Minimum size for LWORK
   LWORK = 2*LWORK ! Performance is better if LWORK is bigger than minimum
   if(allocated(A))    deallocate(A)
   if(allocated(B))    deallocate(B)
   if(allocated(WORK)) deallocate(WORK)
   if(allocated(S))    deallocate(S)

   IS = minVal((/M,N/))

   allocate(A(M,N))
   allocate(B(M))
   allocate(WORK(LWORK))
   allocate(S(IS))

   ! Load data into Lapack variables A,B
   A = A0
   B = B0

   ! Call Lapack routine to solve system of equations A*X = B.
   ! On return from this call the solution X is in the first N locations of B.
   call DGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,INFO)

   ! Check solution status
   if(info/=0) stop "Error - regress(): No solution found, info/=0"

   ! Load solution into X0 for return. Solution is in first N locations of B.
   X0 = B(1:N)

   deallocate(A,B,WORK,S)

end subroutine regress

END MODULE LINEAR_ALGEBRA


!////////////////////////////////////////////////
! MAIN PROGRAM FOR STAND-ALONE TESTING
!////////////////////////////////////////////////
!$$$program main
!$$$  use LINEAR_ALGEBRA
!$$$  implicit none
!$$$
!$$$  integer,parameter :: n1=100
!$$$  real :: A(n1,4),X(4),B(n1)
!$$$  integer :: i
!$$$  real,parameter :: r0=5.5, r1=1.1, r2=3.14, r3=3.3
!$$$
!$$$  ! Create data
!$$$  do i=1,n1
!$$$     B(i) = r0 + r1*i + r2*i**2 + r3*i**3
!$$$     A(i,1) = 1
!$$$     A(i,2) = i
!$$$     A(i,3) = i**2
!$$$     A(i,4) = i**3
!$$$  end do  
!$$$
!$$$  call regress(A,B,X)
!$$$  write(6,*) "X=",x
!$$$
!$$$end program main
