!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: main.f90
!
! PURPOSE: Fortran 95 driver (main) program for Version 6.0 of the University of Alabama in
!          Huntsville (UAH) Lower, Mid and Upper Tropospheric Microwave Sounding Unit(MSU/AMSU)
!          Processing Software.
!
! DESCRIPTION: This program reads monthly gridded inputs files for a range of MSU and AMSU 
!              satellites, processes the data, and creates a single merged monthly gridded record for three
!              different atmosphertic layers: lower troposphere(LT), middle troposphere (MT), and 
!              lower stratosphere (LS).
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS: This program generates output productfiles in the following directories:
!          * lt_data_products_6.0
!          * mt_data_products_6.0
!          * tp_data_products_6.0
!          * ls_data_products_6.0
!
!          The output product files placed in the "lt" directory are:
!          * tltglhmam_6.0        ! Monthly mean file
!          * tltmonacg_6.0        ! Annual cycle file
!          *  
!          * tltmonamg.1978_6.0   ! Gridded monthly anomaly files,
!          * tltmonamg.1979_6.0   ! one file for each year processed.
!          * ...
!          * tltmonamg.2015_6.0
!          * tltmonamg.2015_6.0
!          *  
!          * uahncdc_lt_6.0b      ! File containing time series of anomalies for various regions.
!                                 ! Trends for those regions are at the bottom of the file
!
!          The other output directories are filled with similar files with 
!          the "lt" in the file names changed to "mt" or "tp" or "ls", depending 
!          on the channel.
!
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE MERGE_MODULE
  use REGIONAL_MASKS
  implicit none
  private :: ilat

  !===========================
  ! Global variables

  character(len=2) :: channel_label   ! Set to one of "c2","c3", or "c4". This is set in calling program.
  logical :: make_trend_adjustments   ! Set to .TRUE. for trend adjustments to be preformed, .FALSE.otherwise.
  !integer :: max_elevation            ! Maximum land elevation that is processed (meters). This is set in calling program.

  character(len=15) :: version  ! Set in main program

  logical :: make_diurnal_correction  ! Set in main program to TRUE or FALSE.
  logical :: use_precip_in_diurnal_correction  ! Set in main program to .TRUE. if precip is to be used. Only used if above is TRUE.
  !===========================


  integer,parameter :: iorder = 2 ! 1<= iorder <=5.  Order of fit to use. This is set in calling program. Default is 2.

  real,parameter :: am_ref_time = 19.5 
  real,parameter :: pm_ref_time = 15.0

  real,parameter :: badvalue = -999
  integer,parameter :: nlat=72, nlon=144 ! Lats run S..N, Lons run -180..180
  character(len=150) :: grid_path_0  = "./grid_files_0/" ! Path to grid files
  character(len=150) :: grid_path_1  = "./grid_files_1/" ! Path to grid files
  character(len=150) :: grid_path_2  = "./grid_files_2/" ! Path to grid files
  character(len=150) :: grid_path_ac = "./grid_files_2/" ! Path to grid files (annual cycles)
  character(len=150) :: grid_path_3  = "./grid_files_3/" ! Path to grid files
  character(len=150) :: grid_path_merged  = "./grid_files_merged/" ! Path to merged grid files

  integer :: ilat
  real(8),parameter :: dlat = 180.d0/nlat
  real(8),parameter :: latvec(nlat) = (/ (-90.d0 + dlat/2 + (ilat-1)*dlat,ilat=1,nlat) /)
  real(8),parameter :: pi = 3.141592654d0
  real(8),parameter :: degrees = pi/180.d0

  ! Overall parameters
  integer,parameter :: start_year=1978, end_year=2017
  integer,parameter :: max_num_prints = 15
  integer,parameter :: num_sats = 14
  character(len=3),parameter :: sat_label(num_sats) = (/"trn","n06","n07","n08","n09","n10","n11","n12","n14", & ! isat=1..9 are MSU
                                                        "n15","aqu","n18","met","n19"/) ! isat=10..14 are AMSU
  ! MSU parameters
  integer,parameter :: msu_start_year=1978, msu_end_year=2005
  integer,parameter :: num_msu_prints = 6 ! Angle 1 is limb most
  integer,parameter :: num_msu_channels = 3 ! channels 2,3,4
  character(len=150),parameter :: msu_path = "../msu_v6/monthly_grids/"

  ! AMSU parameters
  integer,parameter :: amsu_start_year=1998, amsu_end_year=2017
  integer,parameter :: num_amsu_prints = 15 ! Angle 1 is limb most
  integer,parameter :: num_amsu_channels = 4 ! channels 5,6,7,9
  character(len=150),parameter :: amsu_path = "../amsu_v6/monthly_grids/"

  type annual_cycles
     real :: am(12,nlat,nlon) = badvalue
     real :: pm(12,nlat,nlon) = badvalue
     real :: final(12,nlat,nlon) = badvalue
  end type annual_cycles   
     
  type sat_structure
     logical :: success  ! TRUE if grid filled successfully, FALSE otherwise.
     !integer :: isat     ! Satellite number (JC's numbering system)
     !character(len=3) :: sat_label ! trn, n06, ...., n15, aqu, etc.
     real(4) :: crossing_time = badvalue ! Monthly averaged crossing time (fractional hour)
     character(len=4) :: instr_type  ! msu or amsu
     integer :: num_prints
     integer :: num_channels
     !integer :: iyear
     !integer :: imonth
     real(4),allocatable :: grid(:,:,:,:)     ! Dims are  grid(nlat,nlon,num_prints,num_channels)
     integer(2),allocatable :: igrid(:,:,:,:) ! Counts per grid value. Dims are igrid(nlat,nlon,#prints,#channels)
     real(4),allocatable :: ia(:,:)           ! Incidence angle. Dims are (nlat,#prints)

     real(4) :: anom(nlat,nlon) = badvalue  ! This holds the mt or lt or ls anomaly product as it evolves.
  end type sat_structure   

  type merged_sat_structure
     real(4) :: anom(nlat,nlon) = badvalue
  end type merged_sat_structure

  type date
     integer :: year
     integer :: month
  end type date 

  type monthly_regional_product_structure
     real :: all(   12, start_year:end_year ) ! Anomalies (land & ocean)
     real :: land(  12, start_year:end_year ) ! Anomalies (land)
     real :: ocean( 12, start_year:end_year ) ! Anomalies (ocean)
     real :: all_12(12, start_year:end_year ) ! 12-month running avg of "all" anomaly
     !integer :: num_days_all( initial%iyear:final%iyear, 1:12 ) ! Number of days used in computation of "all" monthly average
     !integer :: num_days_all_12( initial%iyear:final%iyear, 1:12 ) ! Number of days used in computation of the 12-month running avg
  end type monthly_regional_product_structure

  type monthly_regional_product_collection
     type(monthly_regional_product_structure) :: global, NH, SH, trpcs, NoExt, SoExt, NoPol, SoPol
     real,dimension( 12, start_year:end_year ) :: usa48, usa49, aus
  end type monthly_regional_product_collection

CONTAINS

subroutine add_trend_to_grids(iyear,imonth,x,ich,iyear_ref,imonth_ref,adj)
  integer :: iyear,imonth   ! Input: year and month of grid
  type(sat_structure) :: x  ! Input/Output: grid trend is added to
  integer :: ich            ! Input: channel trend is added to (1,2,3,4 => AMSU 5,6,7,9)
  integer :: iyear_ref, imonth_ref  ! Input: year/month trend addition is referenced to.
  real :: adj               ! Input: amount of trend addition (K/decade)
  real :: adj2
  
  integer :: num_months

  ! Compute number of months since reference year/month
  num_months = iyear*12 + imonth - (iyear_ref*12 + imonth_ref)

  ! Compute adjustment
  adj2 = num_months*adj/120. ! 120 converts from per decade to per month

  ! Apply trend adjustment
  where(x%grid(:,:,:,ich)/=badvalue)
     x%grid(:,:,:,ich) = x%grid(:,:,:,ich) + adj2
  end where   

  write(6,'(i4.4,1x,i2.2,1x,i3.3,1x,f10.5)') iyear,imonth,num_months,adj2

end subroutine add_trend_to_grids

subroutine setup()

  call load_regional_masks()

end subroutine setup  


subroutine remove_residual_annual_cycle( msat, year1, year2, ac0 )
  type(merged_sat_structure) :: msat(:,start_year:) ! Dims are month,year
  integer :: year1, year2 ! Bounds on the years the annual cycle is computed for (year2>year1)
  type(annual_cycles):: ac0

  real(4):: ac(12,nlat,nlon)

  integer :: icount(12,nlat,nlon)
  integer :: iyear,imonth,ilat,ilon

  if(year1>year2) stop "Error - remove_residual_annual_cycle(): year2 must be >= year1"
  if(year1<start_year .or. year1>end_year) stop "Error - remove_residual_annual_cycle(): invalid value for year1"
  if(year2<start_year .or. year2>end_year) stop "Error - remove_residual_annual_cycle(): invalid value for year2"


  ! Compute residual annual cycle
  ac = 0
  icount = 0
  do imonth=1,12
     do iyear=year1,year2
        
        where(msat(imonth,iyear)%anom /= badvalue)
           ac(imonth,:,:) = ac(imonth,:,:) + msat(imonth,iyear)%anom 
           icount(imonth,:,:) = icount(imonth,:,:) + 1
        end where   

     end do   
  end do      

  do imonth=1,12

     where(icount(imonth,:,:)>0)
        ac(imonth,:,:) = ac(imonth,:,:)/icount(imonth,:,:)
     elsewhere
        ac(imonth,:,:) = badvalue
     end where   

  end do   

  ! Remove residual annual cycle
  do iyear=start_year,end_year
     do imonth=1,12

        where(ac(imonth,:,:)/=badvalue .and. msat(imonth,iyear)%anom/=badvalue)
           msat(imonth,iyear)%anom = msat(imonth,iyear)%anom - ac(imonth,:,:)
        elsewhere
           msat(imonth,iyear)%anom = badvalue
        end where   

     end do
  end do   

  ! Update final annual cycle in ac0
  where(ac/=badvalue .and. ac0%final/=badvalue)
     ac0%final = ac0%final + ac
  elsewhere
     ac0%final = badvalue
  endwhere   

end subroutine remove_residual_annual_cycle


subroutine smooth_ocean_or_land( msat, surface )
  type(merged_sat_structure) :: msat(:,start_year:) ! Dims are month,year
  character(len=5) :: surface ! Must be set to "land" or "ocean"

  integer :: iyear,imonth,ilat,ilon
  real(4),save :: land_mask(nlat,nlon),ocean_mask(nlat,nlon),mask(nlat,nlon)

  logical,save :: first_call = .TRUE.
  
  real(4) :: fanom(nlat,nlon)
  real(8) :: wt,value
  integer :: ilon2,ii

  real,parameter :: filter(-2:2) = (/1.,1.,1.,1.,1./)

  if(first_call) then
     land_mask = masks%global%land
     ocean_mask = masks%global%ocean
     first_call = .FALSE.
  end if   

  if(trim(surface) == "land") then
     mask = land_mask
  elseif(trim(surface) == "ocean") then
     mask = ocean_mask
  else
     stop "Error - smooth_ocean_or_land(): Invalid surface type"
  end if   


  YEAR_LOOP: do iyear=start_year,end_year
     MONTH_LOOP: do imonth=1,12

        fanom = msat(imonth,iyear)%anom
        LAT_LOOP: do ilat=1,nlat
           LON_LOOP: do ilon=1,nlon

              if(mask(ilat,ilon)/=1 .or. msat(imonth,iyear)%anom(ilat,ilon)==badvalue) cycle

              wt = 0
              value = 0
              FILTER_LOOP: do ii=-2,+2
                 ilon2 = ilon+ii
                 if(ilon2<1) then
                    ilon2 = ilon2 + nlon
                 elseif(ilon2>nlon) then
                    ilon2 = ilon2 - nlon
                 end if
                 
                 if(mask(ilat,ilon2)==1 .and. msat(imonth,iyear)%anom(ilat,ilon2)/=badvalue) then
                    value = value + msat(imonth,iyear)%anom(ilat,ilon2)*filter(ii)
                    wt = wt + filter(ii)
                 end if   
              
              end do FILTER_LOOP
              
              if( wt==0) stop "Error - smooth_ocean_or_land(): This should never be encountered"

              fanom(ilat,ilon) = value/wt

           end do LON_LOOP
        end do LAT_LOOP   

        msat(imonth,iyear)%anom = fanom

     end do MONTH_LOOP
  end do YEAR_LOOP

end subroutine smooth_ocean_or_land


subroutine automated_trend_adjustment(sat,isat0,isat1,isat2)
  type(sat_structure),target,intent(INOUT) :: sat(:,start_year:,:) ! Dims are (months,years,sats)
  integer,intent(IN) :: isat0  ! Satellite to be adjusted
  integer,intent(IN) :: isat1  ! First satellite overlapping isat0
  integer,intent(IN) :: isat2  ! Second satellite overlapping isat0

  real :: global_bias,global_stddev ! These are needed bias and trends calls, but are not used.
  integer :: num_overlap_months01, num_overlap_months02
  real :: sat0_trend, sat1_trend, sat2_trend
  real :: relative_trend_01, relative_trend_02
  real :: adj

  ! Get number of overlap month
  call compute_bias_between_sats(sat, isat0, isat1, global_bias, global_stddev, num_overlap_months01, remove_bias = .FALSE. )
  call compute_bias_between_sats(sat, isat0, isat2, global_bias, global_stddev, num_overlap_months02, remove_bias = .FALSE. )

  ! Get relative trend in overlaps
  call compute_global_trend_between_sats(sat,isat0, isat1, relative_trend_01, sat0_trend, sat1_trend, remove_trend=.FALSE. ) 
  call compute_global_trend_between_sats(sat,isat0, isat2, relative_trend_02, sat0_trend, sat2_trend, remove_trend=.FALSE. )

  ! Return if no overlap
  if(num_overlap_months01==0 .and. num_overlap_months02==0)  return

  ! Compute and apply adjustment to isat0
  adj = (num_overlap_months01*relative_trend_01 + num_overlap_months02*relative_trend_02)/(num_overlap_months01 + num_overlap_months02)
  call subtract_trend_adjustment(sat, isat0, adj)

  write(6,'(a,f10.4,a)' ) trim(sat_label(isat0))//" trend reduced by: ",adj," per decade"


end subroutine automated_trend_adjustment

subroutine write_diurnal_drift_slopes( slopes_am, slopes_pm, islopes_am, islopes_pm )
  real(4) :: slopes_am(:,:,:), slopes_pm(:,:,:) ! Dims are (months, nlat,nlon)
  integer :: islopes_am(:,:,:),islopes_pm(:,:,:) ! Dims are (months, nlat,nlon)

  character(len=150),parameter :: outpath = "./diurnal_slopes/"

  character(len=150) :: outfile
  integer :: imonth

  ! Write diurnal drift slopes to monthly files
  
  do imonth=1,12

     write(outfile,'(a2,a,i2.2,a)') channel_label,"_slopes_am_",imonth,".dat"
     outfile = trim(outpath)//trim(outfile)
     open(1,file=trim(outfile),status="replace",form="unformatted",access="stream")
     write(1) slopes_am(imonth,:,:)
     close(1)

     write(outfile,'(a2,a,i2.2,a)') channel_label,"_slopes_pm_",imonth,".dat"
     outfile = trim(outpath)//trim(outfile)
     open(1,file=trim(outfile),status="replace",form="unformatted",access="stream")
     write(1) slopes_pm(imonth,:,:)
     close(1)

  end do   

  ! Write diurnal drift islopes to monthly files
  do imonth=1,12

     write(outfile,'(a2,a,i2.2,a)') channel_label,"_islopes_am_",imonth,".dat"
     outfile = trim(outpath)//trim(outfile)
     open(1,file=trim(outfile),status="replace",form="unformatted",access="stream")
     write(1) real(islopes_am(imonth,:,:),4)
     close(1)

     write(outfile,'(a2,a,i2.2,a)') channel_label,"_islopes_pm_",imonth,".dat"
     outfile = trim(outpath)//trim(outfile)
     open(1,file=trim(outfile),status="replace",form="unformatted",access="stream")
     write(1) real(islopes_pm(imonth,:,:),4)
     close(1)

  end do   

end subroutine write_diurnal_drift_slopes

!--------------------------------------------------------------------------
! Collapse 2-D arrray into 1-D array, the first index is varied the fastest.
subroutine collapse( x, cx )
  real :: x(:,:) ! Input: 2-D arrray to be collapsed
  real :: cx(:)  ! Output: 1-D collapsed array
  integer :: i1,i2,n1,n2,ii

  n1 = size(x,1)
  n2 = size(x,2)
  if(size(cx)/=n1*n2) stop "Error - collapse(): inconsistent sizes for input arrays"
   
  ii=0
  do i2=1,n2
     do i1=1,n1
        ii=ii+1
        cx(ii) = x(i1,i2)
     end do
  end do

end subroutine collapse

!--------------------------------------------------------------------------
! Compoute monthly anomaly time series and annual cycle for the following
! regions: global, northern hemisphere, southern hemisphere, continental US (us48), US + Alaska (us49), 
! and Australia.
subroutine compute_monthly_regional_products( msat, monthly )
  !use REGIONAL_MASKS
  type(merged_sat_structure) :: msat(:,start_year:) ! Dims are (month,year)
  type(monthly_regional_product_collection) :: monthly

  real,dimension(72,144) :: mask, cosgrid
  integer :: ilat

  !call load_regional_masks()

  ! Load cosine grid
  do ilat=1,72
     cosgrid(ilat,:) = COS(latvec(ilat)*degrees)
  end do


  ! USA48
  mask = masks%usa48; call compute_regional_anomaly( monthly%usa48 )

  ! USA49 (with Alaska)
  mask = masks%usa49; call compute_regional_anomaly( monthly%usa49)

  ! Australia
  mask = masks%aus; call compute_regional_anomaly( monthly%aus)

  ! Global
  mask = masks%global%all;   call compute_regional_anomaly( monthly%global%all )
  mask = masks%global%land;  call compute_regional_anomaly( monthly%global%land )
  mask = masks%global%ocean; call compute_regional_anomaly( monthly%global%ocean )

  ! Northern Hemisphere
  mask = masks%NH%all;   call compute_regional_anomaly( monthly%NH%all )
  mask = masks%NH%land;  call compute_regional_anomaly( monthly%NH%land )
  mask = masks%NH%ocean; call compute_regional_anomaly( monthly%NH%ocean )

  ! Southern Hemisphere
  mask = masks%SH%all;   call compute_regional_anomaly( monthly%SH%all )
  mask = masks%SH%land;  call compute_regional_anomaly( monthly%SH%land )
  mask = masks%SH%ocean; call compute_regional_anomaly( monthly%SH%ocean )

  ! Tropics
  mask = masks%trpcs%all;   call compute_regional_anomaly( monthly%trpcs%all )
  mask = masks%trpcs%land;  call compute_regional_anomaly( monthly%trpcs%land )
  mask = masks%trpcs%ocean; call compute_regional_anomaly( monthly%trpcs%ocean )

  ! Northern Extratropics
  mask = masks%NoExt%all;   call compute_regional_anomaly( monthly%NoExt%all )
  mask = masks%NoExt%land;  call compute_regional_anomaly( monthly%NoExt%land )
  mask = masks%NoExt%ocean; call compute_regional_anomaly( monthly%NoExt%ocean )

  ! Southern Extratropics
  mask = masks%SoExt%all;   call compute_regional_anomaly( monthly%SoExt%all )
  mask = masks%SoExt%land;  call compute_regional_anomaly( monthly%SoExt%land )
  mask = masks%SoExt%ocean; call compute_regional_anomaly( monthly%SoExt%ocean )

  ! Northern Polar Region
  mask = masks%NoPol%all;   call compute_regional_anomaly( monthly%NoPol%all )
  mask = masks%NoPol%land;  call compute_regional_anomaly( monthly%NoPol%land )
  mask = masks%NoPol%ocean; call compute_regional_anomaly( monthly%NoPol%ocean )

  ! Southern Polar Region
  mask = masks%SoPol%all;   call compute_regional_anomaly( monthly%SoPol%all )
  mask = masks%SoPol%land;  call compute_regional_anomaly( monthly%SoPol%land )
  mask = masks%SoPol%ocean; call compute_regional_anomaly( monthly%SoPol%ocean )
  

  !===================================================================
  ! Compute 12 month running averages of "all" values for some of
  ! these regions: global, NH, SH, trpcs.
  !===================================================================   
  call compute_12_month_running_average( monthly%global )
  call compute_12_month_running_average( monthly%NH )
  call compute_12_month_running_average( monthly%SH )
  call compute_12_month_running_average( monthly%trpcs )   

CONTAINS

subroutine compute_12_month_running_average( x )
  type(monthly_regional_product_structure) :: x
  integer :: n1,n2,iyear,imonth,ii,n
  real,allocatable :: vec(:)

  n1 = size(x%all,1)
  n2 = size(x%all,2)
  if(n2/=(end_year-start_year+1)) stop "Error - compute_12_month_running_average(): Inconsistent for n2"
  if(n1/=12) stop "Error - compute_12_month_running_average(): Inconsistent value for n1"
  if(allocated(vec)) deallocate(vec)
  allocate(vec(n1*n2))

  call collapse(x%all,vec)
   
  x%all_12 = badvalue
  !x%num_days_all_12 = 0
  ii=0
  do iyear=start_year,end_year
     do imonth=1,12
        ii=ii+1
        if(ii<12) cycle
        n = COUNT(vec(ii-11:ii)/=badvalue)
        if(n==0) cycle
        x%all_12(imonth,iyear) = SUM(vec(ii-11:ii),mask=vec(ii-11:ii)/=badvalue)/n
        !x%num_days_all_12(imonth,iyear) = n
     end do
  end do

end subroutine compute_12_month_running_average

subroutine compute_regional_anomaly( anom )
  real :: anom(12,start_year:end_year)
  integer :: iyear, imonth
  real :: missing_mask(72,144), avg, wt

  anom = badvalue
  do iyear = start_year,end_year
     do imonth = 1,12
        missing_mask = 1
        where( msat(imonth,iyear)%anom(:,:)==badvalue ) missing_mask = 0
        if(ALL(missing_mask(:,:)==0)) cycle
        wt = SUM(cosgrid(:,:)*mask(:,:)*missing_mask(:,:))
        avg = SUM(msat(imonth,iyear)%anom(:,:)*cosgrid(:,:)*mask(:,:)*missing_mask(:,:))/wt
        anom(imonth,iyear) = avg
     end do
  end do

end subroutine compute_regional_anomaly

end subroutine compute_monthly_regional_products

!--------------------------------------------------------------------------
subroutine write_monthly_regional_products( monthly, layer, data_product_directory )
  use TREND_MODULE
  type(monthly_regional_product_collection),intent(IN) :: monthly
  character(len=2),intent(IN) :: layer ! lt, mt, ls, or tp
  character(len=150),intent(IN) :: data_product_directory

  character(len=200) :: header_line = " Year Mo Globe  Land Ocean   NH   Land Ocean   SH   Land Ocean Trpcs  Land Ocean NoExt"//&
                                      "  Land Ocean SoExt  Land Ocean NoPol  Land Ocean SoPol  Land Ocean USA48 USA49  AUST"
  character(len=250) :: outfile
  integer :: iyear, imonth
  real :: data_vec(27), trend_vec(27) ! There are 27 regional products
  character(len=500) :: footer1, footer2
  character(len=150) :: label

  ! Variables use in computing trend
  integer,parameter :: num_months = ((end_year - start_year)+1)*12
  real :: xvec(num_months)
  real :: fvec(num_months), slope, intercept
  real :: farray(1:12,start_year:end_year)
  integer :: num_good_values, iproduct
  real :: rbadvalue
 
  !=======================
  ! FILE: uahncdc_lt_6.0
  !       uahncdc_mt_6.0
  !       uahncdc_ls_6.0
  !       uahncdc_tp_6.0
  !=======================
  select case(layer)
     case("lt"); outfile = "uahncdc_lt_"//trim(version)
     case("mt"); outfile = "uahncdc_mt_"//trim(version)
     case("ls"); outfile = "uahncdc_ls_"//trim(version)
     case("tp"); outfile = "uahncdc_tp_"//trim(version)    
  end select      
  outfile = trim(data_product_directory)//trim(outfile)

  open(10,file=trim(outfile),status="REPLACE")
  write(10,'(a)') trim(header_line)

  do iyear = start_year, end_year
     do imonth = 1,12

        data_vec(:) = (/ monthly%global%all(imonth,iyear),   &
                         monthly%global%land(imonth,iyear),  &
                         monthly%global%ocean(imonth,iyear), &
                         monthly%NH%all(imonth,iyear),       &
                         monthly%NH%land(imonth,iyear),      &
                         monthly%NH%ocean(imonth,iyear),     &
                         monthly%SH%all(imonth,iyear),       &
                         monthly%SH%land(imonth,iyear),      &
                         monthly%SH%ocean(imonth,iyear),     &
                         monthly%trpcs%all(imonth,iyear),    &
                         monthly%trpcs%land(imonth,iyear),   &
                         monthly%trpcs%ocean(imonth,iyear),  &
                         monthly%NoExt%all(imonth,iyear),    &
                         monthly%NoExt%land(imonth,iyear),   &
                         monthly%NoExt%ocean(imonth,iyear),  &
                         monthly%SoExt%all(imonth,iyear),    &
                         monthly%SoExt%land(imonth,iyear),   &
                         monthly%SoExt%ocean(imonth,iyear),  &
                         monthly%NoPol%all(imonth,iyear),    &
                         monthly%NoPol%land(imonth,iyear),   &
                         monthly%NoPol%ocean(imonth,iyear),  &
                         monthly%SoPol%all(imonth,iyear),    &
                         monthly%SoPol%land(imonth,iyear),   &
                         monthly%SoPol%ocean(imonth,iyear),  &
                         monthly%usa48(imonth,iyear),        &
                         monthly%usa49(imonth,iyear),        &
                         monthly%aus(imonth,iyear)          /)
        
        if(ALL(data_vec(:)==badvalue)) cycle

        write(10,'(i5,i3,27f6.2)') iyear, imonth, data_vec(:)

     end do
  end do

  !============================================
  ! Compute trend for each regional product.
  !============================================
  xvec(:) = (/ (imonth,imonth=1,num_months) /)
  rbadvalue = badvalue
  do iproduct=1,27
     select case(iproduct)   
        case( 1); farray = monthly%global%all
        case( 2); farray = monthly%global%land
        case( 3); farray = monthly%global%ocean
        case( 4); farray = monthly%NH%all
        case( 5); farray = monthly%NH%land
        case( 6); farray = monthly%NH%ocean
        case( 7); farray = monthly%SH%all
        case( 8); farray = monthly%SH%land
        case( 9); farray = monthly%SH%ocean
        case(10); farray = monthly%trpcs%all
        case(11); farray = monthly%trpcs%land
        case(12); farray = monthly%trpcs%ocean
        case(13); farray = monthly%NoExt%all
        case(14); farray = monthly%NoExt%land
        case(15); farray = monthly%NoExt%ocean
        case(16); farray = monthly%SoExt%all
        case(17); farray = monthly%SoExt%land
        case(18); farray = monthly%SoExt%ocean
        case(19); farray = monthly%NoPol%all
        case(20); farray = monthly%NoPol%land
        case(21); farray = monthly%NoPol%ocean
        case(22); farray = monthly%SoPol%all
        case(23); farray = monthly%SoPol%land
        case(24); farray = monthly%SoPol%ocean
        case(25); farray = monthly%usa48
        case(26); farray = monthly%usa49
        case(27); farray = monthly%aus
     end select   

     call collapse( farray, fvec )

     call trend_with_badvalues(xvec,fvec,slope,intercept,rbadvalue,num_good_values)
     trend_vec(iproduct) = slope*120 ! 120 Converts to per decade
  end do

  !============================================
  ! Write footer lines
  !============================================
  write(10,'(a)') trim(header_line)
  write(10,*    )
  write(10,'(a6,2x,27f6.2)') " Trend", trend_vec(:) ! TBD
  write(10,*    )
  write(10,'(a)') "  NOTE:  Version 6.0 as of April 2015"
  write(10,'(a)') "  NOTE:  New Reference for annual cycle 1981-2010"
  write(10,'(a)') "  NOTE:  Version 5.6 as of Jun 2013"
  write(10,'(a)') "  NOTE:  3/13 More accurate grid weighting for USA48"
  write(10,'(a)') "  NOTE:  3/13 Alaska added in USA49, Australia added"
  write(10,*    )
  write(10,'(a)') "  GL 90S-90N, NH 0-90N, SH 90S-0, TRPCS 20S-20N"
  write(10,'(a)') "  NoExt 20N-90N, SoExt 90S-20S, NoPol 60N-90N, SoPol 90S-60S"

  close(10)

  !=======================
  ! FILE: tltglhmam_6.0
  !       tmtglhmam_6.0
  !       ttpglhmam_6.0
  !       tlsglhmam_6.0
  !=======================
  select case(layer)
     case("lt"); outfile = "tltglhmam_"//trim(version); label = "MONTHLY MEANS OF LOWER-TROPOSPHERE" 
     case("mt"); outfile = "tmtglhmam_"//trim(version); label = "MONTHLY MEANS OF MID-TROPOSPHERE" 
     case("ls"); outfile = "tlsglhmam_"//trim(version); label = "MONTHLY MEANS OF LOWER-STRATOSPHERE"
     case("tp"); outfile = "ttpglhmam_"//trim(version); label = "MONTHLY MEANS OF TROPOPAUSE" 
  end select      
  outfile = trim(data_product_directory)//trim(outfile)

  open(10,file=trim(outfile),status="replace")
  write(10,'(a,a,a)')trim(label),"  ",trim(version)
  write(10,*)
  write(10,'(a)')" ANNUAL CYCLE BASED ON  81001-110365 12-MON RUNNING MEAN"
  write(10,'(a)')"   YEAR  MON  GLOBAL     NH      SH    TRPC  NO.DAYS   GLOBAL     NH      SH    TRPC  DAYS"

  do iyear = start_year, end_year
     do imonth = 1,12
        write(10,'(i7,i5,4(f8.3),i5,4x,4(f8.3),i5)') iyear, imonth,        &
                                         monthly%global%all(imonth,iyear), &
                                         monthly%NH%all(imonth,iyear),     &
                                         monthly%SH%all(imonth,iyear),     &
                                         monthly%trpcs%all(imonth,iyear),  &
                                         0, & !monthly%global%num_days_all(imonth,iyear), &
                                         monthly%global%all_12(imonth,iyear), &
                                         monthly%NH%all_12(imonth,iyear),     &
                                         monthly%SH%all_12(imonth,iyear),     &
                                         monthly%trpcs%all_12(imonth,iyear),  &
                                         0 !monthly%global%num_days_all_12(imonth,iyear)

     end do
  end do

  write(10,*)
  write(10,'(a)') "DECADAL TREND= " ! TBD

end subroutine write_monthly_regional_products

!--------------------------------------------------------------------------

subroutine get_reference_incidence_angles(sat, msu_ref_ia, amsu_ref_ia)
  type(sat_structure),intent(IN) :: sat(:,start_year:,:) ! Dims are (month,year,sat)
  real,intent(INOUT) :: msu_ref_ia, amsu_ref_ia

  integer :: isat,ilat,iyear,imonth,iprint
  real :: ia5,ia6,ia7,ia

  ! Get MSU reference incidence angle

  select case(channel_label) ! MSU channels 2,3,4 are in array indices 1,2,3.
     case("c2"); iprint = 4  ! MSU ch 2
     case("c3"); iprint = 4  ! MSU ch 3
     case("c4"); iprint = 4  ! MSU ch 4
  end select               
  
  isat = 9 ! N14
  ilat = 36
  iyear = 1996
  imonth = 1
  msu_ref_ia = sat(imonth,iyear,isat)%ia(ilat,iprint)


  ! Get AMSU reference incidence angle

  select case(channel_label) ! AMSU channels 5,6,7,9 are in array indices 1,2,3,4.
     case("c2"); iprint = 6  ! AMSU ch 5  (switch to between 6 & 7)
     case("c3"); iprint = 12 ! AMSU ch 7
     case("c4"); iprint = 6  ! AMSU ch 9
  end select               

  isat = 10 ! N15
  ilat = 36
  iyear = 1999
  imonth = 1
  amsu_ref_ia = sat(imonth,iyear,isat)%ia(ilat,iprint)


  ! Override for AMSU ch2
  if(channel_label=="c2") then
     ia5 =  sat(imonth,iyear,isat)%ia(ilat,5)
     ia6 =  sat(imonth,iyear,isat)%ia(ilat,6)
     ia7 =  sat(imonth,iyear,isat)%ia(ilat,7)

     !amsu_ref_ia = 0.667*ia6 + 0.333*ia7 ! Beta 4
     amsu_ref_ia = 0.50*ia5 + 0.50*ia6   ! Beta 5


     ! DEBUG OVERRIDE (WDB)
     if(version=="6.0sc0") amsu_ref_ia = 0.667*ia6   + 0.333*ia7
     if(version=="6.0sc1") amsu_ref_ia = 0.7*ia6     + 0.3*ia7
     if(version=="6.0sc2") amsu_ref_ia = 0.8*ia6     + 0.2*ia7
     if(version=="6.0sc3") amsu_ref_ia = 0.9*ia6     + 0.1*ia7
     if(version=="6.0sc4") amsu_ref_ia = 1.0*ia6
     if(version=="6.0sc5") amsu_ref_ia = 1.0*ia5

     if(version=="6.0sc6") amsu_ref_ia = 0.25*ia5 + 0.75*ia6
     if(version=="6.0sc7") amsu_ref_ia = 0.50*ia5 + 0.50*ia6
     if(version=="6.0sc8") amsu_ref_ia = 0.75*ia5 + 0.25*ia6
     ! END DEBUG

  end if   

end subroutine get_reference_incidence_angles

!--------------------------------------------------------------------------
subroutine apply_ch3_correction( msat )
  use INTERPOLATE
  type(merged_sat_structure) :: msat(:,start_year:)

  real :: corr0(5), corr(12,1978:2002)
  real :: xvec0(5)
  
  integer :: iyear,imonth,ii
  real :: x


  ! Correction table "peg" points (ch3 only)
  corr0(1) = +0.70; xvec0(1) = 1   ! Dec. 1978
  corr0(2) = +0.70; xvec0(2) = 13  ! Dec. 1979
  corr0(3) = +0.08; xvec0(3) = 25  ! Dec. 1980
  corr0(4) = -0.11; xvec0(4) = 49  ! Dec. 1982
  corr0(5) =  0.00; xvec0(5) = 289 ! Dec. 2002

  ! Compute corrections for each month from the correction table.
  ii=0
  corr = badvalue
  do iyear=1978,2002
     do imonth=1,12
        if(iyear==1978 .and. imonth<12) cycle
        ii=ii+1
        x = ii
        corr(imonth,iyear) = linear_interpolate(corr0,xvec0,x)
     end do
  end do   

  ! Apply correction to data for period Dec 1978 to Dec 2002
  do iyear=1978,2002
     do imonth=1,12
        if(iyear==1978 .and. imonth<12) cycle
        where(msat(imonth,iyear)%anom /=badvalue) msat(imonth,iyear)%anom = msat(imonth,iyear)%anom - corr(imonth,iyear)
     end do
  end do   

end subroutine apply_ch3_correction

subroutine apply_n06_ch3_correction( sat )
  type(sat_structure) :: sat(:,start_year:,:) ! Dims are (month,year,sat)

  integer,parameter :: isat = 2  ! Sat index for NOAA-6

  integer :: iyear,imonth

  real :: error(1978:1981,12)

  error = badvalue

  error(1978,12) =  0.7
  error(1979,1 ) =  0.7
  error(1979,2 ) =  0.7
  error(1979,3 ) =  0.7
  error(1979,4 ) =  0.7
  error(1979,5 ) =  0.7
  error(1979,6 ) =  0.7
  error(1979,7 ) =  0.7
  error(1979,8 ) =  0.7
  error(1979,9 ) =  0.7
  error(1979,10) =  0.7
  error(1979,11) =  0.7
  error(1979,12) =  0.7
  error(1980,1 ) =  0.65
  error(1980,2 ) =  0.6
  error(1980,3 ) =  0.55
  error(1980,4 ) =  0.5
  error(1980,5 ) =  0.45
  error(1980,6 ) =  0.4
  error(1980,7 ) =  0.35
  error(1980,8 ) =  0.3
  error(1980,9 ) =  0.25
  error(1980,10) =  0.2
  error(1980,11) =  0.15
  error(1980,12) =  0.1
  error(1981,1 ) =  0.05
  error(1981,2 ) =  0

  do iyear=1978,1981
     do imonth=1,12

        if(error(iyear,imonth)==badvalue) cycle

        where(sat(imonth,iyear,isat)%anom(:,:) /= badvalue) 
           sat(imonth,iyear,isat)%anom(:,:) = sat(imonth,iyear,isat)%anom(:,:) - error(iyear,imonth)
        end where   

     end do
  end do

end subroutine apply_n06_ch3_correction

!--------------------------------------------------------------------------
subroutine write_monthly_gridded_products( msat, layer, data_product_directory )
  type(merged_sat_structure),intent(IN) :: msat(:,start_year:) ! Strucutre containing anomalies to be written
  character(len=2),intent(IN) :: layer   ! One of lt, mt, ls, or tp.
  character(len=150),intent(IN) :: data_product_directory

  integer :: iarray(72,144) ! Use for integer output
  integer :: ilat, imonth, iyear
  character(len=150) :: outfile
  character(len=2) :: ulayer ! Layer in upper case

  write(6,*)   trim(data_product_directory)
  write(101,*) trim(data_product_directory)

  ulayer = layer
  call upper_case(ulayer)

  ! WRITE PRODUCT DATA SET: gridded monthly anomalies, e.g. tmtmonamg.1998_6.0
  YEAR_LOOP: do iyear  = start_year,end_year

     select case(layer)
     case("lt"); write(outfile,71)"tltmonamg.",iyear,"_"//trim(version)
     case("mt"); write(outfile,71)"tmtmonamg.",iyear,"_"//trim(version)
     case("ls"); write(outfile,71)"tlsmonamg.",iyear,"_"//trim(version)
     case("tp"); write(outfile,71)"ttpmonamg.",iyear,"_"//trim(version)
     end select
     71 format(a,i4.4,a)
      
     outfile = trim(data_product_directory)//trim(outfile)
     open(10,file=trim(outfile),status="replace")

     MONTH_LOOP: do imonth = 1, 12 
      
        write(10,'(i12.12,i12,i12,a12)') 0,iyear,imonth,ulayer

        where(msat(imonth,iyear)%anom(:,:)/=badvalue) 
           iarray = msat(imonth,iyear)%anom(:,:)*100
        elsewhere
           iarray = -9999 ! Previous versions used -9999 for missing value in output files
        end where

        do ilat=1,72
           write(10,'(16i5)') iarray(ilat,:)
        end do

     end do MONTH_LOOP
     close(10)
  end do YEAR_LOOP

end subroutine write_monthly_gridded_products
!--------------------------------------------------------------------------


subroutine apply_diurnal_drift_correction( sat, slopes_am, slopes_pm )
  type(sat_structure),intent(INOUT) :: sat(:,start_year:,:) ! Dims are (imonth, iyear, isat)
  real(4),intent(IN) :: slopes_am(:,:,:), slopes_pm(:,:,:) ! Dims are (months, nlat, nlon)

  integer :: isat,iyear,imonth,ilat,ilon
  real :: slopes(12,nlat,nlon)
  real :: ref_time,time
  

  SAT_LOOP: do isat=1,num_sats

     ! Skip the reference satellites
     if(isat==11) cycle ! AQUA
     if(isat==14) cycle ! N19

     select case(isat)
        case( 2,4,6,8,10,13);      slopes = slopes_am; ref_time = am_ref_time  ! AM sats: N6, N8, N10, N12, N15, MET
        case( 1,3,5,7,9,11,12,14); slopes = slopes_pm; ref_time = pm_ref_time  ! PM sats: TRN, N7, N9, N11, N14, AQU, N18, N19
        case DEFAULT; stop "Error - apply_diurnal_drift_correction(): Invalid ksat"    
     end select

     YEAR_LOOP: do iyear=start_year,end_year
        MONTH_LOOP: do imonth=1,12
           if( .not. sat(imonth,iyear,isat)%success ) cycle

           time = sat(imonth,iyear,isat)%crossing_time
           if(time==badvalue) then
              sat(imonth,iyear,isat)%anom(:,:) = badvalue
              cycle
           end if   

           do ilat=1,nlat
              do ilon=1,nlon

                 if(sat(imonth,iyear,isat)%anom(ilat,ilon)==badvalue) cycle

                 if( slopes(imonth,ilat,ilon)==badvalue ) then
                    sat(imonth,iyear,isat)%anom(ilat,ilon)=badvalue
                    cycle
                 else   
                    sat(imonth,iyear,isat)%anom(ilat,ilon) = sat(imonth,iyear,isat)%anom(ilat,ilon) - slopes(imonth,ilat,ilon)*(time-ref_time)
                 end if
   
              end do   
           end do

        end do MONTH_LOOP
     end do YEAR_LOOP
  end do SAT_LOOP  

end subroutine apply_diurnal_drift_correction

!--------------------------------------------------------------------------

subroutine compute_diurnal_drift_corrections(sat, slopes, islopes, ref_time, isat_ref, isat2 )
  use TREND_MODULE
  use NAN
  type(sat_structure),intent(IN) :: sat(:,start_year:,:) ! Dims are (months,years,sats)
  real(4),allocatable,intent(OUT)  :: slopes(:,:,:)  ! Computed diurnal drift corrections. Dims are (months,nlat,nlon)
  integer,allocatable,intent(OUT)  :: islopes(:,:,:)  ! Num values used in regression at each lat/lon point. Dims are (months,nlat,nlon)
  real,intent(IN)    :: ref_time ! Time the crossing times are referenced too
  integer,intent(IN) :: isat_ref ! ID of the satellite to use as reference
  integer,intent(IN) :: isat2    ! ID of the satellite regressed against the reference


  integer,parameter :: end_year_dd = 2014   ! Diurnal drift is computed using data thru this year
  integer :: ilat,ilon,icount,num_good_values
  integer,parameter :: n = (end_year_dd - start_year + 1)*12*3  ! Max number of months. Factor of 3 accounts for taking values from 3-month windows.
  real :: delta_vec(n) ! Holds anomaly differences
  real :: ct_vec(n)    ! Holds crossing times
  real :: slope, intercept
  integer :: imonth0,imonth,iyear0,iyear,i,istart,istop
  integer :: im(3),iy(3),i1(1),ilow,ihigh

  real(4) :: precip(12,nlat,nlon)

  ! Type to hold the values in 3-month window centered on current year/month
  type window
     real:: v(3)
  end type
  type(window) :: sat_ref, sat2

  ! Read precip data
  open(1,file="precip.dat",action="read",form="unformatted",access="stream")
  read(1) precip
  close(1)

  if(allocated(slopes)) deallocate(slopes)
  allocate(slopes(12,nlat,nlon))
  slopes = badvalue

  if(allocated(islopes)) deallocate(islopes)
  allocate(islopes(12,nlat,nlon))
  islopes = 0

  ! At each lat/lon, regress the difference (anom_isat2 - anom_isat_ref) vs isat2 crossing times.
  do ilat=1,nlat
     do ilon=1,nlon

        MONTH_LOOP: do imonth0=1,12
           icount = 0
           delta_vec = badvalue
           ct_vec = badvalue

           YEAR_LOOP: do iyear0=start_year,end_year_dd

              if(iyear0==start_year  .and. imonth0==1 ) cycle
              if(iyear0==end_year_dd .and. imonth0==12) cycle

              ! Set month/year for 3 month window on current month/year.
              select case(imonth0)
                 case(1   ); im(1)=12;        im(2)=1;       im(3)=2;         iy(1)=iyear0-1; iy(2)=iyear0; iy(3)=iyear0
                 case(2:11); im(1)=imonth0-1; im(2)=imonth0; im(3)=imonth0+1; iy(1)=iyear0;   iy(2)=iyear0; iy(3)=iyear0
                 case(12  ); im(1)=11;        im(2)=12;      im(3)=1;         iy(1)=iyear0;   iy(2)=iyear0; iy(3)=iyear0+1
              end select

              ! If all 3 months in window are available, use them. Otherwise only use the center month.
              if( ALL(sat(im(:),iy(:),isat_ref)%success                  )  .and. &
                  ALL(sat(im(:),iy(:),isat2)%success                     )  .and. &
                  ALL(sat(im(:),iy(:),isat_ref)%anom(ilat,ilon)/=badvalue)  .and. &
                  ALL(sat(im(:),iy(:),isat2)%anom(ilat,ilon)/=badvalue   )) then 
                 istart=1
                 istop=3
              elseif ( (sat(im(2),iy(2),isat_ref)%success                  )  .and. &
                       (sat(im(2),iy(2),isat2)%success                     )  .and. &
                       (sat(im(2),iy(2),isat_ref)%anom(ilat,ilon)/=badvalue)  .and. &
                       (sat(im(2),iy(2),isat2)%anom(ilat,ilon)/=badvalue   )) then 
                 istart =2
                 istop = 2
              else
                 cycle
              end if 

              ! Add values from the window to regression vectors
              THREE_MONTH_WINDOW: do i=istart,istop

                 imonth = im(i)
                 iyear  = iy(i)
                 icount = icount + 1
                 delta_vec(icount) = sat(imonth,iyear,isat2)%anom(ilat,ilon) - sat(imonth,iyear,isat_ref)%anom(ilat,ilon)
                 ct_vec(icount)    = sat(imonth,iyear,isat2)%crossing_time - ref_time

              end do THREE_MONTH_WINDOW

           end do YEAR_LOOP

           ! Compute slope
           if( COUNT(delta_vec/=badvalue)>=4 ) then

              ! Remove the max and min values in delta_vec
              i1  = minloc(delta_vec,mask=delta_vec/=badvalue)
              ilow = i1(1)
              i1  = maxloc(delta_vec,mask=delta_vec/=badvalue)
              ihigh = i1(1)
              delta_vec(ilow)  = badvalue
              delta_vec(ihigh) = badvalue

              call trend_with_badvalues(ct_vec,delta_vec,slope,intercept,badvalue,num_good_values)
              slopes(imonth0,ilat,ilon) = slope
              islopes(imonth0,ilat,ilon) = num_good_values

              ! DEBUG BLOCK
              if(ilon==132 .and. imonth0==12) then
                 write(55,*) "START BLOCK: ilat=",ilat," imonth=",imonth0
                 do i=1,size(delta_vec)
                    if(delta_vec(i)/=badvalue .and. ct_vec(i)/=badvalue) then
                       write(55,*),sat_label(isat2),sat_label(isat_ref),ct_vec(i), delta_vec(i)
                    end if
                 end do
                 write(55,*) "Slope = ",slopes(imonth0,ilat,ilon)
              end if
              ! END DEBUG BLOCK

           end if   

        end do MONTH_LOOP
     end do
  end do

  if(channel_label=="c4") then
     ! For channel 4, replace slope value at each pixel with the latitudinal average.
     do imonth=1,12
        call avg_lat( slopes(imonth,:,:) )
     end do

  else if(channel_label=="c3") then

     ! Over the oceans, replace slope value with the latitudinal average. Compute the
     ! avaerage using 3 lat bands centered on the lat band computing the average for.
     do imonth=1,12
        call avg_lat_over_ocean( slopes(imonth,:,:) )
     end do

     ! Over land, replace slope values with those from regression
     ! of slope vs elevation.
     do imonth=1,12
        call linear_fit_over_land3( slopes(imonth,:,:) )
     end do

  else if(channel_label=="c2") then

     ! Over the oceans, replace slope value with the latitudinal average. Compute the
     ! avaerage using 3 lat bands centered on the lat band computing the average for.
     do imonth=1,12
        call avg_lat_over_ocean( slopes(imonth,:,:) )
     end do

     ! Over land, replace slope values with those from regression
     ! of slope vs elevation and precip.
     do imonth=1,12
        if(use_precip_in_diurnal_correction) then
           call linear_fit_over_land_with_precip( slopes(imonth,:,:), precip(imonth,:,:) )
        else   
           call linear_fit_over_land3( slopes(imonth,:,:) )
        end if   
     end do

  end if

  ! Replace any NaN values with badvalue
  call set_nan_to_badvalue(slopes,badvalue)
  
end subroutine compute_diurnal_drift_corrections

!--------------------------------------------------------------------------
subroutine avg_lat( slope )
  real(4) :: slope(:,:)
  real(4) :: land_mask(nlat,nlon),ocean_mask(nlat,nlon)
  integer :: ilat,n
  real :: avg

  if( size(slope,1)/=nlat .or. size(slope,2)/=nlon ) stop "Error - avg_lat(): Unexpected dims for input array"
  
  ! Compute average each lat band.
  do ilat=1,nlat
     n = COUNT(slope(ilat,:)/=badvalue)
     if(n>0) then
        avg = SUM(slope(ilat,:),mask=slope(ilat,:)/=badvalue)/n
     else
        avg = badvalue
     end if   
     slope(ilat,:) = avg
  end do   

end subroutine avg_lat
!---------------------------------------------------------------------
subroutine avg_lat_over_ocean( slope )
  real(4) :: slope(:,:)
  real(4) :: land_mask(nlat,nlon),ocean_mask(nlat,nlon)
  integer :: ilat
  real :: avg

  if( size(slope,1)/=nlat .or. size(slope,2)/=nlon ) stop "Error - avg_lat_over_ocean(): Unexpected dims for input array"
  
  land_mask = masks%global%land
  ocean_mask = masks%global%ocean
  
  ! Compute average over ocean for each lat band.
  ! Each ocean pixel at that land band gets the average value
  do ilat=1,nlat
     if(COUNT(ocean_mask(ilat,:)==1)>0) then
        avg = SUM(slope(ilat,:)*ocean_mask(ilat,:),mask=slope(ilat,:)/=badvalue)/SUM(ocean_mask(ilat,:),mask=slope(ilat,:)/=badvalue)
        where(ocean_mask(ilat,:)==1) slope(ilat,:) = avg
     end if   
  end do   

end subroutine avg_lat_over_ocean

!---------------------------------------------------------------------
! This version does the fit using 3 lat bands centered on the current band.
! This is done for bands 2..nlat-1. Band is set equal to 2 and band nlat 
! is set equal to band nlat-1.
subroutine linear_fit_over_land3( slope )
  use TREND_MODULE
  real(4) :: slope(:,:)
  real(4) :: land_mask(nlat,nlon), elevation(nlat,nlon)
  integer :: ilat,ilon,ii,icount
  real :: fvec(3*nlon),elev_vec(3*nlon),a,b
  integer :: num_good_values, lat_start, lat_stop


  if( size(slope,1)/=nlat .or. size(slope,2)/=nlon ) stop "Error - linear_fit_over_land(): Unexpected dims for input array"
  
  land_mask = masks%global%land
  elevation = masks%elevation

  LAT_LOOP: do ilat=1,nlat
     fvec = badvalue
     elev_vec = badvalue
     if(count(land_mask(ilat,:)==1)<2) cycle
     
     icount = 0
     lat_start = ilat-1
     lat_stop  = ilat+1
     
     if(ilat==1) lat_start = ilat
     if(ilat==nlat) lat_stop = ilat
        
     do ii=lat_start,lat_stop
        do ilon=1,nlon
           if(land_mask(ii,ilon)==1) then
              icount = icount + 1
              elev_vec(icount) = elevation(ii,ilon)
              if(slope(ii,ilon)/=badvalue) then
                 fvec(icount) = slope(ii,ilon)
              else   
                 fvec(icount) = badvalue
              end if   
           end if
        end do
     end do   

     call trend_with_badvalues(elev_vec,fvec,a,b,badvalue,num_good_values)
     if(num_good_values==0) then
        a=badvalue
        b=badvalue
     end if   

     ! Use a,b to compute new slopes
     LON_LOOP: do ilon=1,nlon
        if(land_mask(ilat,ilon)/=1) cycle
        if(a/=badvalue .and. b/=badvalue) then
           slope(ilat,ilon) = a*elevation(ilat,ilon) + b
        else   
           slope(ilat,ilon) = badvalue
        end if   
     end do LON_LOOP   

  end do LAT_LOOP
  
  ! Set 1st and last lat band equal to its neighbor
  ! TBD
  

end subroutine linear_fit_over_land3

subroutine linear_fit_over_land_with_precip( slope, precip )
  use TREND_MODULE
  real(4) :: slope(:,:),precip(:,:) ! Dims are (nlat,nlon)
  real(4) :: land_mask(nlat,nlon), elevation(nlat,nlon)
  integer :: ilat,ilon,ii,icount
  real :: fvec(3*nlon),elev_vec(3*nlon),a,b,c, precip_vec(3*nlon)
  integer :: num_good_values, lat_start, lat_stop


  if( size(slope,1)/=nlat .or. size(slope,2)/=nlon ) stop "Error - linear_fit_over_land(): Unexpected dims for input array"
  

  land_mask = masks%global%land
  elevation = masks%elevation

  LAT_LOOP: do ilat=1,nlat
     fvec = badvalue
     elev_vec = badvalue
     precip_vec = badvalue
     if(count(land_mask(ilat,:)==1)<2) cycle
     
     icount = 0
     lat_start = ilat-1
     lat_stop  = ilat+1
     
     if(ilat==1) lat_start = ilat
     if(ilat==nlat) lat_stop = ilat
        
     do ii=lat_start,lat_stop
        do ilon=1,nlon
           if(land_mask(ii,ilon)==1) then
              icount = icount + 1
              elev_vec(icount) = elevation(ii,ilon)
              precip_vec(icount) = precip(ii,ilon)

              if(slope(ii,ilon)/=badvalue) then
                 fvec(icount) = slope(ii,ilon)
              else   
                 fvec(icount) = badvalue
              end if   
           end if
        end do
     end do   

     call trend_2var_with_badvalues(elev_vec,precip_vec,fvec,a,b,c,badvalue,num_good_values)

     ! Use a,b to compute new slopes
     LON_LOOP: do ilon=1,nlon
        if(land_mask(ilat,ilon)/=1) cycle
        if(a/=badvalue .and. b/=badvalue .and. c/=badvalue) then
           slope(ilat,ilon) = a*elevation(ilat,ilon) + b*precip(ilat,ilon) + c
        else   
           slope(ilat,ilon) = badvalue
        end if   
     end do LON_LOOP   

  end do LAT_LOOP
  
end subroutine linear_fit_over_land_with_precip

!---------------------------------------------------------------------
subroutine compute_global_time_series_and_trend( msat, time_series, trend )
  use TREND_MODULE
  type(merged_sat_structure),intent(IN) :: msat(:,start_year:) ! Dims are (month,year)
  real,intent(OUT),allocatable :: time_series(:,:) ! Dims are (month,year)
  real,intent(OUT) :: trend  ! Computed trend (K/month)

  integer :: iyear,imonth,ilat,n,i

  real,allocatable :: fvec(:),xvec(:)

  real :: cos_lat_grid(nlat,nlon), wt, slope, intercept
  integer :: num_good_values

  ! Load cos_lat_grid
  do ilat=1,nlat
     cos_lat_grid(ilat,:) = COS(latvec(ilat)*degrees)
  end do   

  if(allocated(time_series)) deallocate(time_series)
  allocate(time_series(12,start_year:end_year))

  ! Compute global time series
  time_series = badvalue
  do iyear=start_year,end_year
     do imonth=1,12
        if(ALL(msat(imonth,iyear)%anom(:,:)==badvalue)) cycle
        
        wt = SUM(cos_lat_grid,mask=msat(imonth,iyear)%anom/=badvalue)
        time_series(imonth,iyear) = SUM(msat(imonth,iyear)%anom*cos_lat_grid,mask=msat(imonth,iyear)%anom/=badvalue)/wt

     end do
  end do   

  ! Compute trend of the time series
  n = size(time_series,1)*size(time_series,2)
  allocate(fvec(n),xvec(n))

  i=0
  do iyear=start_year,end_year
     do imonth=1,12
        i=i+1
        fvec(i) = time_series(imonth,iyear)
     end do
  end do   

  xvec(:) = (/(i,i=1,n)/)

  call trend_with_badvalues(xvec,fvec,trend,intercept,real(badvalue),num_good_values)

  write(6,  '(a,f8.4,a)') "global trend = ",trend*120," K/decade"  ! DEBUG
  write(101,'(a,f8.4,a)') "global trend = ",trend*120," K/decade"  ! DEBUG

  write(6,  *) "num good values = ",num_good_values     ! DEBUG
  write(101,*) "num good values = ",num_good_values     ! DEBUG

  deallocate(fvec,xvec)

end subroutine compute_global_time_series_and_trend

subroutine write_global_time_series_for_sats( sat, isat_list, outfile)
  type(sat_structure),intent(IN) :: sat(:,start_year:,:) ! Dims are (month,year,sat)
  integer,intent(IN)             :: isat_list(:)       ! Vector of IDs of sats to be written to outfile.
  character(len=150),intent(IN)  :: outfile            ! Ouput file for time series

  integer :: iyear,imonth,ilat,n,isat,ii

  real :: cos_lat_grid(nlat,nlon), wt, slope, intercept
  real :: time_series(12,start_year:end_year)

  real,allocatable :: avg(:)

  n=size(isat_list)
  allocate(avg(n))

  ! Load cos_lat_grid
  do ilat=1,nlat
     cos_lat_grid(ilat,:) = COS(latvec(ilat)*degrees)
  end do   

  open(55,file=trim(outfile),status="replace")

  write(55,'(a,14(2x,a3))') "year month ", sat_label(isat_list(:))

  ! Compute global time series
  time_series = badvalue
  YEAR_LOOP: do iyear=start_year,end_year
     MONTH_LOOP: do imonth=1,12
        
        avg = badvalue
        do ii=1,n
           isat = isat_list(ii)
           if(ALL(sat(imonth,iyear,isat)%anom(:,:)==badvalue)) cycle
           wt = SUM(cos_lat_grid,mask=sat(imonth,iyear,isat)%anom/=badvalue)
           avg(ii) = SUM(sat(imonth,iyear,isat)%anom*cos_lat_grid,mask=sat(imonth,iyear,isat)%anom/=badvalue)/wt
        end do
        write(55,10) iyear,imonth,avg(:)
        10 format(i4,1x,i2.2,1x,14(1x,f9.4))

     end do MONTH_LOOP
  end do YEAR_LOOP

  close(55)

  deallocate(avg)

end subroutine write_global_time_series_for_sats
!--------------------------------------------------------------------------
subroutine merge_satellites( sat, msat, ac ) ! Also computes residual annual cycle
  use SMOOTH
  type(sat_structure),intent(IN) :: sat(:,start_year:,:) ! Dims are (months, year, sats)
  type(merged_sat_structure),allocatable,intent(OUT) :: msat(:,:)  ! Allocated within this routine. Dims are(1:12,start_year:end_year)
  type(annual_cycles),intent(INOUT) :: ac ! The am, pm annual cycles (loaded in this structure) are input.
                                          ! The residual, final annual cycles are computed and returned

  integer :: iyear,imonth,ilat,ilon,icount
  real :: rvec(num_sats)
  real :: vec12(12)

  real :: residual(12,nlat,nlon) = badvalue

  if(allocated(msat)) deallocate(msat)
  allocate(msat(12,start_year:end_year))

  ! Merge the satellite record, averaging overlapping satellite values.
  do iyear=start_year,end_year
     do imonth=1,12
        do ilat=1,nlat
           do ilon=1,nlon

              rvec = sat(imonth,iyear,:)%anom(ilat,ilon)

              icount = COUNT(rvec/=badvalue)

              if( icount>0 ) then
                 msat(imonth,iyear)%anom(ilat,ilon) = SUM(rvec,mask=rvec/=badvalue)/icount
              else
                 msat(imonth,iyear)%anom(ilat,ilon) = badvalue
              end if   

           end do
        end do
     end do
  end do



  ! Compute resiudal anomaly annual cycle for the 30-year  period Jan 1981 to Dec 2010
  do imonth=1,12
     do ilat=1,nlat
        do ilon=1,nlon
           icount = COUNT(msat(imonth,1981:2010)%anom(ilat,ilon)/=badvalue)

           if(icount > 20) then
              residual(imonth,ilat,ilon) = SUM(msat(imonth,1981:2010)%anom(ilat,ilon),mask=msat(imonth,1981:2010)%anom(ilat,ilon)/=badvalue)/icount
           else
              residual(imonth,ilat,ilon) = badvalue
           end if

        end do
     end do
  end do

  ! Smooth this just computed residual anomaly annual cycle
  do ilat= 1,72
     do ilon=1,144
        call fit_fourier_series( residual(:,ilat,ilon), 4, vec12, badvalue)
        where( residual(:,ilat,ilon)/=badvalue .and. vec12/=badvalue) residual(:,ilat,ilon) = vec12
     end do
  end do

  ! Remove the residual anomaly annual cycle from the merged anomaliles to create the 
  ! gridded monthly anomalies.
  do iyear=start_year,end_year
     do imonth=1,12
        do ilat=1,nlat
           do ilon=1,nlon

              if( msat(imonth,iyear)%anom(ilat,ilon)/=badvalue .and. residual(imonth,ilat,ilon)/=badvalue ) then
                 msat(imonth,iyear)%anom(ilat,ilon) = msat(imonth,iyear)%anom(ilat,ilon) - residual(imonth,ilat,ilon)
              else
                 msat(imonth,iyear)%anom(ilat,ilon) = badvalue
              end if

           end do
        end do
     end do
  end do

  ! Add the residual anomaly annual cycle to the NOAA-10 AM absolute annual cycle to create the
  ! "gridded monthly annual cycle".
  where( ac%am/=badvalue .and. residual/=badvalue) 
     ac%final = ac%am + residual
  elsewhere
     ac%final = badvalue
  end where

end subroutine merge_satellites

!--------------------------------------------------------------------------
! Remove trend adjustment in satellite anomalies
subroutine subtract_trend_adjustment(sat, isat, adjustment, start_date )
  type(sat_structure),intent(INOUT) :: sat(:,start_year:,:)  ! Dims are (months, years, sats)
  integer,intent(IN)                :: isat       ! Satellite to be adjusted (JC's notation)
  real,intent(IN)                   :: adjustment ! Trend per decade (K) to be subtracted from idsat
  type(date),optional,intent(IN)    :: start_date ! Optional date at which the trend application is started

  real :: offset, monthly_trend_adjustment
  integer :: iday, iyear, imonth, imonth_running

  real,pointer :: ptr(:,:,:,:)

  ! Ajust the monthly grids
  imonth_running = 0
  monthly_trend_adjustment = adjustment/120.
  YEAR_LOOP: do iyear =start_year,end_year
     MONTH_LOOP: do imonth = 1,12

        ! OPTIONAL: delay application of trend
        if(present(start_date)) then
           if(iyear<start_date%year) cycle
           if(iyear==start_date%year .and. imonth<start_date%month) cycle
           if(iyear==start_date%year .and. imonth==start_date%month) imonth_running = 0
        end if

        imonth_running = imonth_running + 1
        offset = (imonth_running-0.5)*monthly_trend_adjustment ! the -0.5 puts value in middle of month

        where( sat(imonth,iyear,isat)%anom/=badvalue )
           sat(imonth,iyear,isat)%anom = sat(imonth,iyear,isat)%anom - offset
        endwhere

     end do MONTH_LOOP
  END DO YEAR_LOOP

  if( present(start_date) ) then
     write(6,  '(a,a,f9.4,a,i4.4,a,i3.3,a)') sat_label(isat), " trend reduced by: ",adjustment," per decade (delayed start to: ", start_date%year, "_",start_date%month,")"
     write(101,'(a,a,f9.4,a,i4.4,a,i3.3,a)') sat_label(isat), " trend reduced by: ",adjustment," per decade (delayed start to: ", start_date%year, "_",start_date%month,")"
  else
     write(6,  '(a,a,f9.4,a)') sat_label(isat), " trend reduced by: ",adjustment," per decade"
     write(101,'(a,a,f9.4,a)') sat_label(isat), " trend reduced by: ",adjustment," per decade"
  end if

end subroutine subtract_trend_adjustment

!--------------------------------------------------------------------------
! For the specified time period, compute the trend in satellite A relative to satellite B.
! If remove_trend=T, then remove trend(A-B) from the entire record of A.

subroutine compute_global_trend_between_sats(sat, isat_a, isat_b, relative_global_trend, sat_a_global_trend, sat_b_global_trend, remove_trend, start_date )
  type(sat_structure),intent(INOUT) :: sat(:,start_year:,:) ! Dims are (months, years, sats)
  integer,intent(IN)                :: isat_a, isat_b ! Trend in satellite A relative to satellite B (per ???)
  real,intent(OUT)                  :: relative_global_trend   ! Global trend of sat A relative to sat B, per decade
  real,intent(OUT)                  :: sat_a_global_trend, sat_b_global_trend ! Per decade
  logical,intent(IN),optional       :: remove_trend ! =T, remove trend from sat A; =F, not removed.
  type(date),intent(IN),optional    :: start_date    ! If remove_trend is present and true, then the trend removal is delayed until start_date (if present).

  integer :: ilat, iyear, imonth

  real    :: avg_a, avg_b, icount_a, icount_b, wt_a, wt_b
  real,parameter :: deg = 3.141592654d0/180.

  real :: bias, wt,trend_per_decade, trend_per_year, trend_per_month

  real :: xmean, fmean, x2, fx ! Summation variables for computing trend
  real :: trend,stddev
  real :: diff, offset
  integer :: icount, imonth_running

  type sat_trend_vars
     real :: xmean
     real :: fmean
     real :: x2
     real :: fx
     real :: trend_per_month
     real :: trend_per_year
     real :: trend_per_decade
  end type  

  type(sat_trend_vars) :: trend_diff, trend_a, trend_b

  icount = 0
  imonth_running = 0

  trend_diff = sat_trend_vars(0.,0.,0.,0.,0.,0.,0.)
  trend_a = sat_trend_vars(0.,0.,0.,0.,0.,0.,0.)
  trend_b = sat_trend_vars(0.,0.,0.,0.,0.,0.,0.)

  YEAR_LOOP: do iyear = start_year,end_year
     MONTH_LOOP: do imonth = 1,12
        imonth_running = imonth_running + 1

        if( .not. sat(imonth,iyear,isat_a)%success .or. .not. sat(imonth,iyear,isat_b)%success ) cycle

        avg_a = compute_global_avg( sat(imonth,iyear,isat_a)%anom )
        avg_b = compute_global_avg( sat(imonth,iyear,isat_b)%anom )
        if( avg_a==badvalue .or. avg_b==badvalue ) cycle

        icount = icount + 1

        diff = avg_a - avg_b

        ! Difference (e.g. sat a - sat b)
        trend_diff%xmean = trend_diff%xmean + imonth_running
        trend_diff%fmean = trend_diff%fmean + diff
        trend_diff%x2    = trend_diff%x2 + imonth_running*imonth_running
        trend_diff%fx    = trend_diff%fx + diff*imonth_running

        ! Sat A only
        trend_a%xmean = trend_a%xmean + imonth_running
        trend_a%fmean = trend_a%fmean + avg_a
        trend_a%x2    = trend_a%x2 + imonth_running*imonth_running
        trend_a%fx    = trend_a%fx + avg_a*imonth_running

        ! Sat B only
        trend_b%xmean = trend_b%xmean + imonth_running
        trend_b%fmean = trend_b%fmean + avg_b
        trend_b%x2    = trend_b%x2 + imonth_running*imonth_running
        trend_b%fx    = trend_b%fx + avg_b*imonth_running

     end do MONTH_LOOP
  END DO YEAR_LOOP

  ! Finalize global trend difference
  trend_diff%xmean = trend_diff%xmean/icount
  trend_diff%fmean = trend_diff%fmean/icount
  trend_diff%trend_per_month  = (trend_diff%fx - icount*trend_diff%fmean*trend_diff%xmean)/(trend_diff%x2-icount*trend_diff%xmean*trend_diff%xmean)
  trend_diff%trend_per_year   = trend_diff%trend_per_month*12
  trend_diff%trend_per_decade = trend_diff%trend_per_year*10
  relative_global_trend  = trend_diff%trend_per_decade

  ! Finalize satellite A trend
  trend_a%xmean = trend_a%xmean/icount
  trend_a%fmean = trend_a%fmean/icount
  trend_a%trend_per_month  = (trend_a%fx - icount*trend_a%fmean*trend_a%xmean)/(trend_a%x2-icount*trend_a%xmean*trend_a%xmean)
  trend_a%trend_per_year   = trend_a%trend_per_month*12
  trend_a%trend_per_decade = trend_a%trend_per_year*10
  sat_a_global_trend  = trend_a%trend_per_decade

  ! Finalize satellite B trend
  trend_b%xmean = trend_b%xmean/icount
  trend_b%fmean = trend_b%fmean/icount
  trend_b%trend_per_month  = (trend_b%fx - icount*trend_b%fmean*trend_b%xmean)/(trend_b%x2-icount*trend_b%xmean*trend_b%xmean)
  trend_b%trend_per_year   = trend_b%trend_per_month*12
  trend_b%trend_per_decade = trend_b%trend_per_year*10
  sat_b_global_trend  = trend_b%trend_per_decade


  ! Remove relative trend from sat A. Note: Removing a trend changes the mean.
  if( present(remove_trend)  .and. remove_trend .eqv. .TRUE.) then
     
     if(present(start_date)) then
        call subtract_trend_adjustment(sat, isat_a, trend_per_year, start_date) ! Delay start of isat_a adjustment until start_date
     else
        call subtract_trend_adjustment(sat, isat_a, trend_per_year)  ! Apply adjustment to entire isat_a valid record      
     end if

  end if   

end subroutine compute_global_trend_between_sats

!--------------------------------------------------------------------------
! Compute bias in satellite A relative to satellite B, and optionally remove the 
! bias from satellite A.
subroutine compute_bias_between_sats(sat, isat_a, isat_b, global_bias, global_stddev, num_overlap_months, remove_bias )
  type(sat_structure),target,intent(INOUT) :: sat(:,start_year:,:) ! Dims are (months,years,sats)
  integer,intent(IN)                       :: isat_a, isat_b
  real,intent(OUT)                         :: global_bias, global_stddev
  integer,intent(OUT)                      :: num_overlap_months
  logical,intent(IN),optional              :: remove_bias  ! =T, compute and remove latitudinal bias from sat A; =F, not removed

  real(8) :: avg_a, avg_b, diff, xmean, fmean, x2, ff, stddev, wt
  integer :: iyear,imonth,imonth_running,icount,ilat

  integer :: lat_icount(nlat)
  real(8) :: lat_mean(nlat),lat_bias(nlat), lat_fmean(nlat)
  real(4),pointer,dimension(:,:) :: ptr_a, ptr_b

  !=================================================================
  ! Compute latitudinal bias between satellites A and B (i.e. A-B)
  !=================================================================
  lat_icount = 0
  lat_fmean = 0
  num_overlap_months = 0
  do iyear=start_year,end_year
     do imonth=1,12
        
        if( .not. sat(imonth,iyear,isat_a)%success .or. .not. sat(imonth,iyear,isat_b)%success ) cycle
        num_overlap_months = num_overlap_months + 1

        ptr_a => sat(imonth,iyear,isat_a)%anom(:,:)
        ptr_b => sat(imonth,iyear,isat_b)%anom(:,:)
        
        do ilat= 1, nlat

           if( ALL(ptr_a(ilat,:)==badvalue) ) cycle
           if( ALL(ptr_b(ilat,:)==badvalue) ) cycle

           lat_icount(ilat) = lat_icount(ilat) + 1

           avg_a = SUM(ptr_a(ilat,:),mask=ptr_a(ilat,:)/=badvalue)/COUNT(ptr_a(ilat,:)/=badvalue)
           avg_b = SUM(ptr_b(ilat,:),mask=ptr_b(ilat,:)/=badvalue)/COUNT(ptr_b(ilat,:)/=badvalue)
        
           diff = avg_a - avg_b 
           lat_fmean(ilat) = lat_fmean(ilat) + diff

        end do
     end do
  end do
  
  where(lat_icount>0) 
     lat_bias(:) = lat_fmean(:)/lat_icount(:)
  elsewhere
     lat_bias(:) = badvalue
  endwhere

  !==========================================================
  ! Compute global bias by averaging the latitudinal biases
  !==========================================================
  if(ALL(lat_bias==badvalue)) then
     global_bias = badvalue
  else   
     wt = SUM(COS(latvec(:)*degrees),mask=lat_bias(:)/=badvalue)
     global_bias = SUM(lat_bias(:)*COS(latvec(:)*degrees),mask=lat_bias(:)/=badvalue)/wt
  end if
  global_stddev = badvalue ! This is no longer computed

  !========================================================
  ! Optionally, remove latitudinal bias from satellite A
  !========================================================
  OPTION_BLOCK: if( present(remove_bias) .and. remove_bias .eqv. .TRUE. ) then

     do iyear=start_year,end_year
        do imonth=1,12
           if(.not. sat(imonth,iyear,isat_a)%success) cycle
           do ilat=1,nlat
              if(ANY(sat(imonth,iyear,isat_a)%anom(ilat,:)/=badvalue)) then
                 if(lat_bias(ilat)==badvalue) stop "Error - compute_bias_between_sats(): unexpected badvalue in lat_bias"
                 where(sat(imonth,iyear,isat_a)%anom(ilat,:)/=badvalue) sat(imonth,iyear,isat_a)%anom(ilat,:) = sat(imonth,iyear,isat_a)%anom(ilat,:) - lat_bias(ilat)
              end if   
           end do
        end do   
     end do

  end if OPTION_BLOCK

end subroutine compute_bias_between_sats


! Function to compute cosine weighted average of input array x.
function compute_global_avg( x ) result(avg)
  real(4) :: x(:,:)
  real :: avg

  integer :: ilat
  real(8),save :: coslat(nlat,nlon)
  logical,save :: first_call = .TRUE.

  if(first_call) then
     do ilat=1,nlat
        coslat(ilat,:) = COS(latvec(ilat)*degrees)
     end do
     first_call = .FALSE.
  end if   
  
  ! Verify size of input array
  if(size(x,1)/=nlat .or. size(x,2)/=nlon) stop "Error - compute_global_avg(): Unexpected dimensions for input array"

  if(ALL(x==badvalue)) then
     avg = badvalue
     return
  end if   
  
  avg = SUM(x*coslat,mask=x/=badvalue)/SUM(coslat,mask=x/=badvalue)

end function compute_global_avg



!--------------------------------------------------------------------------
! Comute monthly grid point anomalies relative to the am sat or pm sat monthly
! grid point annual cycles. These annual cycles are also computed in this routine.
subroutine compute_monthly_grid_anomalies( sat, ac )
  use SMOOTH
  !type(sat_structure),target,intent(INOUT) :: sat(:,start_year:,:)  ! Dimensions are (imonth,iyear,isat)
  type(sat_structure),target :: sat(:,start_year:,:)  ! Dimensions are (imonth,iyear,isat)
  type(annual_cycles),intent(INOUT) :: ac ! The am/pm annual cycles are created and added to this structure.

  ! Set reference satellites for annual cycles
  integer,parameter :: iam = 6 ! AM_ac_sat = 6 ! NOAA-10
  integer,parameter :: ipm = 7 ! PM_ac_sat = 7 ! NOAA-11

  integer,parameter :: n=(end_year-start_year + 1)*12 ! Maximum number of months

  integer :: ilat, ilon, imonth, iyear, isat
  real :: vecAM(12), vecPM(12), fitvec(12)
  integer :: icount(12)
  character(len=2) :: time

  real(4),pointer :: am_ptr, pm_ptr, ptr

  integer :: ii ! DEBUG
  integer :: total, num_grids ! DEBUG
  character(len=50) :: outfile ! DEBUG

  !=================================================================
  ! Compute grid pt annual cycles of the reference AM and PM sats
  ! for their overlap period.
  !=================================================================
  LAT_LOOP: do ilat=1,nlat
     LON_LOOP: do ilon=1,nlon

        ! Compute annual cycles at current grid point
        vecAM = 0
        vecPM = 0
        icount = 0
        do iyear=start_year,end_year
           do imonth=1,12

              if( (.not. sat(imonth,iyear,iam)%success) .or. (.not. sat(imonth,iyear,ipm)%success) ) cycle

              am_ptr=>sat(imonth,iyear,iam)%anom(ilat,ilon)
              pm_ptr=>sat(imonth,iyear,ipm)%anom(ilat,ilon)

              if( am_ptr/=badvalue .and. pm_ptr/=badvalue ) then
                 vecAM(imonth) = vecAM(imonth) + am_ptr
                 vecPM(imonth) = vecPM(imonth) + pm_ptr
                 icount(imonth) = icount(imonth) + 1
              end if

           end do
        end do

        where( icount(:) > 0 )
           vecAM(:) = vecAM(:)/icount(:)
           vecPM(:) = vecPM(:)/icount(:)
        else where
           vecAM(:) = badvalue
           vecPM(:) = badvalue
        end where

        ! Smooth annual cycles at current grid point. Load into annual cycle grids.
        call fit_fourier_series( vecAM, 4, fitvec, badvalue )
        ac%am(:,ilat,ilon) = fitvec

        call fit_fourier_series( vecPM, 4, fitvec, badvalue )
        ac%pm(:,ilat,ilon) = fitvec

     end do LON_LOOP
  end do LAT_LOOP


  !=================================================================
  ! Compute anomalies by removing appropriate (AM or PM) annual cycle
  !=================================================================
  SAT_LOOP: do isat = 1, 14
     select case(isat)
        case( 1); time = "PM"  ! PM          trn
        case( 2); time = "AM"  ! AM          n06
        case( 3); time = "PM"  ! PM          n07
        case( 4); time = "AM"  ! AM          n08
        case( 5); time = "PM"  ! PM          n09
        case( 6); time = "AM"  ! AM          n10
        case( 7); time = "PM"  ! PM          n11
        case( 8); time = "AM"  ! AM          n12
        case( 9); time = "PM"  ! PM          n14
        case(10); time = "AM"  ! AM  AMSU    n15
        case(11); time = "PM"  ! PM  AMSU    aqu
        case(12); time = "PM"  ! PM  AMSU    n18
        case(13); time = "AM"  ! AM  AMSU    met
        case(14); time = "PM"  ! PM  AMSU    n19
     end select

     do ilat=1,nlat
        do ilon=1,nlon
           do iyear=start_year,end_year
              do imonth=1,12

                 if( .not. sat(imonth,iyear,isat)%success ) cycle

                 ptr=>sat(imonth,iyear,isat)%anom(ilat,ilon)

                 if(time == "AM") then
                    if( (ptr/=badvalue) .and. (ac%am(imonth,ilat,ilon)/=badvalue) ) then
                       ptr = ptr - ac%am(imonth,ilat,ilon)               
                    else
                       ptr = badvalue
                    end if
                 end if

                 if(time == "PM") then
                    if( (ptr/=badvalue) .and. (ac%pm(imonth,ilat,ilon)/=badvalue) ) then
                       ptr = ptr - ac%pm(imonth,ilat,ilon)               
                    else
                       ptr = badvalue
                    end if
                 end if
               
              end do
           end do
        end do
     end do

  end do SAT_LOOP


  ! Write AM/PM annual cycles
  open(1,file=trim(grid_path_ac)//channel_label//"_am_ann_cyc.dat",form="unformatted",access="stream",status="replace")
  write(1) real(ac%am,4)
  close(1)
  open(1,file=trim(grid_path_ac)//channel_label//"_pm_ann_cyc.dat",form="unformatted",access="stream",status="replace")
  write(1) real(ac%pm,4)
  close(1)

end subroutine compute_monthly_grid_anomalies

!--------------------------------------------------------------------------

subroutine deallocate_sat_structure( x )
  type(sat_structure),intent(INOUT) :: x

  if(allocated(x%grid )) deallocate(x%grid )
  if(allocated(x%igrid)) deallocate(x%igrid)
  if(allocated(x%ia   )) deallocate(x%ia   )

  x%success = .FALSE.
  
end subroutine deallocate_sat_structure

subroutine apply_valid_ranges(x)
  use VALID_RANGE_MODULE
  type(sat_structure),intent(INOUT) :: x(:,start_year:,:) ! Dims are (month,year,sat)

  integer :: imonth,iyear,isat
  logical :: flag

  write(6,  *) "Running: apply_valid_ranges()"
  write(101,*) "Running: apply_valid_ranges()"

  do isat=1,num_sats
     do iyear=start_year,end_year
        do imonth=1,12

           flag = valid_sat_month(isat,iyear,imonth,channel_label)
           if(.not. flag) call deallocate_sat_structure( x(imonth,iyear,isat) )

           write(222,*) isat,iyear,imonth,channel_label,flag  !!! DEBUG

        end do
     end do
  end do    

end subroutine apply_valid_ranges

subroutine write_grid_files( x , path)
  type(sat_structure),intent(IN) :: x(:,start_year:,:) ! Dims are (month,year,sat)
  character(len=150),intent(IN) :: path  ! Path to the directory where the grids are written (must be already created).
  integer :: isat,iyear,imonth
  integer :: num_months,num_years,num_sats
  integer :: icount
  character(len=150) :: grid_file

  num_months = size(x,1)
  num_years  = size(x,2)
  num_sats   = size(x,3)

  if((start_year+num_years-1)/=end_year) stop "Error - write_grid_files(): Unexpected year range in input variable"
  
  ! Open the for writing the product (lt,mt,ls) grid file list
  open(15,file="grid_file_list.txt",status="replace")

  icount = 0
  do isat=1,num_sats
     do iyear=start_year,end_year
        do imonth=1,12
           if(x(imonth,iyear,isat)%success) then

              ! Create file name (e.g. mt_n15_2014_08.dat)
              write(grid_file,'(a2,"_",a3,"_",i4.4,"_",i2.2,".dat")') channel_label,sat_label(isat),iyear,imonth
              open(33,file=trim(path)//"/"//trim(grid_file),status="replace",form="unformatted",access="stream")

              icount = icount + 1
              write(33) x(imonth,iyear,isat)%anom

              write(15,*) icount,trim(path)//trim(grid_file)
              close(33)
      
           end if   
        end do
     end do
  end do   
  close(15)

end subroutine write_grid_files

subroutine write_merged_grid_files( x , path)
  type(merged_sat_structure),intent(IN),target :: x(:,start_year:) ! Dims are (month,year,sat)
  character(len=150),intent(IN) :: path  ! Path to the directory where the grids are written (must be already created).
  integer :: iyear,imonth
  integer :: num_months,num_years,num_sats
  character(len=150) :: grid_file

  real(4),pointer :: ptr(:,:)

  num_months = size(x,1)
  num_years  = size(x,2)

  if((start_year+num_years-1)/=end_year) stop "Error - write_grid_files(): Unexpected year range in input variable"
  
  do iyear=start_year,end_year
     do imonth=1,12

        ptr => x(imonth,iyear)%anom

        if(ALL(ptr==badvalue)) then
           cycle
        else

           ! Create file name (e.g. mt_n15_2014_08.dat)
           write(grid_file,'(a2,"_",a3,"_",i4.4,"_",i2.2,".dat")') channel_label,"mrg",iyear,imonth
           open(33,file=trim(path)//"/"//trim(grid_file),status="replace",form="unformatted",access="stream")
           write(33) ptr
           close(33)
           
        end if   
     end do
  end do
  close(15)

end subroutine write_merged_grid_files

subroutine compute_ch_tb( x , ref_ia )
  use LINEAR_ALGEBRA

  type(sat_structure),intent(INOUT) :: x
  real,intent(IN) :: ref_ia ! Reference incidence angle

  real,allocatable :: tb_vec(:) ! Vector of tb's at each print for a fixed lat,lon
  real,allocatable :: ia_vec(:) ! Vector of incidence angles at each print for a fixed lat,lon
  !real,parameter :: ref_ia = 28.4663d0  ! N15 incidence angle for print 8 at equator for Jan 1999.

  integer :: ichannel,num_prints,ilat,ilon,icoeff
  real :: rdum
  real,allocatable :: coeff(:)
  integer :: iprint

  x%anom = badvalue

  if(.not. x%success) return

  if(trim(x%instr_type)=="msu") then

     select case(channel_label) ! MSU channels 2,3,4 are in array indices 1,2,3.
        case("c2"); ichannel = 1 ! MSU ch 2
        case("c3"); ichannel = 2 ! MSU ch 3
        case("c4"); ichannel = 3 ! MSU ch 4
     end select               

  elseif(trim(x%instr_type)=="amsu") then

     select case(channel_label) ! AMSU channels 5,6,7,9 are in array indices 1,2,3,4.
        case("c2"); ichannel = 1 ! AMSU ch 5
        case("c3"); ichannel = 3 ! AMSU ch 7
        case("c4"); ichannel = 4 ! AMSU ch 9
     end select               

  else
     stop "Error - compute_ch_tb(): invlaid instr_type in input sat_structure variable x"
  end if   

  num_prints = x%num_prints

  if(allocated(tb_vec)) deallocate(tb_vec)
  allocate(tb_vec(num_prints))

  if(allocated(ia_vec)) deallocate(ia_vec)
  allocate(ia_vec(num_prints))

  do ilat=1,nlat
     ia_vec(:) = x%ia(ilat,:)
     
     if(any(ia_vec==badvalue)) cycle

     do ilon=1,nlon
        
        tb_vec(:) = x%grid(ilat,ilon,:,ichannel)

        if(any(tb_vec==badvalue)) cycle

        ! Fit polynomial of order "iorder" to tb_vec vs ia_vec.
        ! Vector coeff is allocated within this call.
        call curve_fit(tb_vec,ia_vec,iorder,coeff)

        ! Apply coefficients and compute tb at reference incidence angle
        rdum = 0
        do icoeff=1,size(coeff)
           rdum = rdum + coeff(icoeff)*ref_ia**(icoeff-1)
        end do
        x%anom(ilat,ilon) = rdum
        
     end do
  end do   

  deallocate(tb_vec)
  deallocate(ia_vec)

end subroutine compute_ch_tb

subroutine read_sat(isat, iyear, imonth, x)
  use valid_range_module
  integer,intent(IN) :: isat ! Satellite id 1..14
  integer,intent(IN) :: iyear
  integer,intent(IN) :: imonth
  type(sat_structure),intent(OUT) :: x

  character(len=150) :: infile, path
  integer :: num_prints,num_channels
  integer :: year1,month1,year2,month2 ! Use to hold valild date ranges for a sat

  ! Incidence angle. Dims are (month,year,lat,print,sat). For msu, only prints 1..6 are used.
  real(4) :: ia( 12,start_year:end_year,nlat,max_num_prints,num_sats)

  logical,save :: first_call = .TRUE.

  ! Variables need for call to "add_trend_to_grids"
  integer :: ich,iyear_ref,imonth_ref
  real    :: adj

  if(first_call) then
     ! Read satellite incidence angle file
     open(1,file="incidence_angles.dat",action="read",form="unformatted",access="stream",status="old")
     read(1) ia
     close(1)
     first_call = .FALSE.
  end if   

  if(allocated(x%grid)) deallocate(x%grid)
  if(allocated(x%igrid)) deallocate(x%igrid)
  if(allocated(x%ia)) deallocate(x%ia)

  if(isat<=9) then
     x%instr_type = "msu"
  else
     x%instr_type = "amsu"
  end if   

  if(trim(x%instr_type)=="msu") then
     num_prints = num_msu_prints
     num_channels = num_msu_channels
     path = msu_path
  elseif(trim(x%instr_type)=="amsu") then
     num_prints = num_amsu_prints
     num_channels = num_amsu_channels
     path = amsu_path
  else
     stop "Error - read_sat(): This branch should never be reached"
  end if

  if(trim(x%instr_type)=="msu" .and. (iyear<msu_start_year .or. iyear>msu_end_year)) then
     x%success = .FALSE.
     return
  end if   
  if(trim(x%instr_type)=="amsu" .and. (iyear<amsu_start_year .or. iyear>amsu_end_year)) then
     x%success = .FALSE.
     return
  end if   

  allocate(x%grid(nlat,nlon,num_prints,num_channels))
  allocate(x%igrid(nlat,nlon,num_prints,num_channels))
  allocate(x%ia(nlat,num_prints))
  x%grid = badvalue
  x%igrid = badvalue
  x%ia = badvalue
  x%num_prints = num_prints
  x%num_channels = num_channels
  x%crossing_time = badvalue

  ! Reconstruct name for input monthly grid file (e.g. msu_n14_1985_07.dat, amsu_n19_2000_09.dat)
  write(infile,'(a,"_",a3,"_",i4.4,"_",i2.2,".dat")') trim(x%instr_type),sat_label(isat),iyear,imonth

  ! Read grids
  open(1,file=trim(path)//trim(infile),action="read",status="old",form="unformatted",access="stream",err=99)
  read(1) x%crossing_time
  read(1) x%grid,x%igrid
  close(1)

  !=================================================================================
  ! To test impact of calibration drift, add a trend to the N15 ch5 grids of "adj" K per decade.
  !$$$if(isat==10) then
  !$$$   ich = 1  ! 1,2,3,4 => AMSU 5,6,7,9
  !$$$   iyear_ref = 1998
  !$$$   imonth_ref = 7
  !$$$   adj = 0.2 ! K/decade
  !$$$   write(6,*) "TREND ADDED TO N15 GRIDS using: add_trend_to_grids():"
  !$$$   call add_trend_to_grids(iyear,imonth,x,ich,iyear_ref,imonth_ref,adj)
  !$$$end if   
  !=================================================================================

  ! Copy incidence angles for current isat,imonth,iyear
  x%ia(:,:) = ia(imonth,iyear,:,1:num_prints,isat)

  ! Set success flag
  x%success = .TRUE.
  return

  99 continue
  deallocate(x%grid, x%igrid, x%ia)
  x%success = .FALSE.

end subroutine read_sat

subroutine compute_trend_grid(x, gtrend)
  use TREND_MODULE
  type(merged_sat_structure),intent(IN) :: x(:,:) ! Dims are (month,year)
  real(4) :: gtrend(:,:)! Output. Compute trend grid. Dims are (nlat,nlon). Trends are per month.

  integer :: nyear,nmonth,n
  integer :: ilat,ilon,iyear,imonth,icount,i
  real(8),allocatable :: fvec(:,:,:) ! Dims are num_years*num_months,nlat,nlon)
  real(8),allocatable :: xvec(:)     ! Dims are num_years*num_months
  real(8) :: slope, intercept
  integer :: num_good_values


  !if(ALL(product/=(/"lt","mt","ls"/))) stop "Error - compute_trend_grid(): Input var product not lt,mt, or ls"

  nmonth = size(x,1)
  nyear = size(x,2)
  n = nyear*nmonth

  if(allocated(fvec)) deallocate(fvec)
  allocate(fvec(n,nlat,nlon))

  if(allocated(xvec)) deallocate(xvec)
  allocate(xvec(n))

  xvec = (/(i,i=1,n)/) ! Total months number consecutively

  ! Load input grid x0 into a new grid with the year,month dimensions rolled into 1 dim
  icount = 0
  do iyear=1,nyear
     do imonth=1,nmonth
        icount = icount + 1

        fvec(icount,:,:) = x(imonth,iyear)%anom(:,:)

     end do
  end do

  ! Compute trends at each grid point
  gtrend = badvalue
  do ilat=1,nlat
     do ilon=1,nlon
        call trend_with_badvalues(xvec,fvec(:,ilat,ilon),slope,intercept,badvalue,num_good_values)
        gtrend(ilat,ilon) = slope
     enddo
  enddo   

end subroutine compute_trend_grid

!=============================================================================

subroutine lower_case(word)
  ! convert a word to lower case
  character (len=*) , intent(in out) :: word
  integer                            :: i,ic,nlen
  nlen = len(word)
  do i=1,nlen
     ic = ichar(word(i:i))
     if (ic >= 65 .and. ic <= 90) word(i:i) = char(ic+32)
  end do
end subroutine lower_case

subroutine upper_case(word)
  ! convert a word to upper case
  character (len=*) , intent(in out) :: word
  integer                            :: i,ic,nlen
  nlen = len(word)
  do i=1,nlen
     ic = ichar(word(i:i))
     if (ic >= 97 .and. ic <= 122) word(i:i) = char(ic-32)
  end do
end subroutine upper_case

subroutine make_lt(c2_msat, c3_msat, c4_msat, lt_msat, c2_ac, c3_ac, c4_ac, lt_ac)
  type(merged_sat_structure),intent(IN) :: c2_msat(:,start_year:), c3_msat(:,start_year:),c4_msat(:,start_year:)
  type(merged_sat_structure),allocatable,intent(out) :: lt_msat(:,:)
  type(annual_cycles) :: c2_ac, c3_ac, c4_ac, lt_ac

  real,parameter :: c2_coeff =  1.53846 ! Note these sum to 1.
  real,parameter :: c3_coeff = -0.54808
  real,parameter :: c4_coeff =  0.00962

  integer :: iyear,imonth

  if(allocated(lt_msat)) deallocate(lt_msat)
  allocate(lt_msat(12,start_year:end_year))

  ! Compute lt
  do iyear=start_year,end_year
     do imonth=1,12
        where( c2_msat(imonth,iyear)%anom/=badvalue .and. &
               c3_msat(imonth,iyear)%anom/=badvalue .and. &
               c4_msat(imonth,iyear)%anom/=badvalue )
           lt_msat(imonth,iyear)%anom = c2_coeff*c2_msat(imonth,iyear)%anom + &
                                        c3_coeff*c3_msat(imonth,iyear)%anom + &
                                        c4_coeff*c4_msat(imonth,iyear)%anom
        elsewhere
           lt_msat(imonth,iyear)%anom = badvalue
        end where   
     end do
  end do   

  ! Compute lt annual cycle
  where(c2_ac%final/=badvalue .and. c3_ac%final/=badvalue .and. c4_ac%final/=badvalue)
     lt_ac%final = c2_coeff*c2_ac%final + c3_coeff*c3_ac%final + c4_coeff*c4_ac%final
  elsewhere
     lt_ac%final = badvalue
  endwhere   

end subroutine make_lt

!=============================================================================

subroutine process_channel( channel, msat, ac )
  character(len=2),intent(IN) :: channel
  type(merged_sat_structure),allocatable :: msat(:,:) ! Dims are (imonth,iyear) 
  type(annual_cycles) :: ac

  ! Select satellites to use
  logical,parameter :: use_sat(num_sats) = (/ .TRUE., & ! trn 
                                              .TRUE., & ! n06 
                                              .TRUE., & ! n07 
                                              .TRUE., & ! n08 
                                              .TRUE., & ! n09 
                                              .TRUE., & ! n10 
                                              .TRUE., & ! n11 
                                              .TRUE., & ! n12 
                                              .TRUE., & ! n14 
                                              .TRUE., & ! n15 
                                              .TRUE., & ! aqu 
                                              .TRUE., & ! n18 
                                              .TRUE., & ! met 
                                              .TRUE. /) ! n19 

  !===================================================================


  character(len=150) :: infile,outfile
  integer(4) :: isat,iyear,imonth,ilat

  type(sat_structure),allocatable :: sat(:,:,:)  ! Dims are (imonth, iyear, isat)

  real(4) :: fitted_grid(nlat,nlon,start_year:end_year,12,num_sats)

  real(4) :: trend_grid(nlat,nlon)

  real :: amsu_ref_ia, msu_ref_ia
  real :: ref_ia
  integer :: icount
  character(len=150) :: grid_file

  integer :: icase,isat_a, isat_b
  real :: global_bias, global_stddev, sat_a_trend, sat_b_trend, relative_trend
  integer :: num_overlap_months
  real,allocatable :: global_time_series(:,:) ! Dims are (year,month)
  real :: global_trend, adjustment



  real(4),allocatable :: slopes_am(:,:,:), slopes_pm(:,:,:) ! Dims are (months, nlat,nlon)
  integer,allocatable :: islopes_am(:,:,:),islopes_pm(:,:,:) ! Dims are (months, nlat,nlon)

  integer :: isat0,isat1,isat2



  !===========================================================
  ! Set global variables in module merge_module:
  ! product, version, data_product_directory
  !
  ! Set local variables: layer, make_trend_adjustments
  !===========================================================
  channel_label = channel

  write(6,  *) "make_trend_ajustments = ", make_trend_adjustments
  write(101,*) "make_trend_ajustments = ", make_trend_adjustments

  ! Allocate sat structure for holding grids
  if(allocated(sat)) deallocate(sat)
  allocate(sat(12,start_year:end_year,num_sats))

  fitted_grid = badvalue

  ! Read grid files into sat_structure sat.
  write(6,  *) "Reading monthly grid files"
  write(101,*) "Reading monthly grid files"
  do isat=1,num_sats
     if(.not. use_sat(isat)) cycle
     write(6,  *) isat,sat_label(isat)
     write(101,*) isat,sat_label(isat)
     do iyear=start_year,end_year
        do imonth=1,12
           call read_sat(isat,iyear,imonth,sat(imonth,iyear,isat))
           !write(44,*) isat,iyear+imonth/12.,sat(imonth,iyear,isat)%crossing_time ! DEBUG
        enddo
     end do
  end do 

  ! Apply valid ranges
  call apply_valid_ranges(sat)

  ! Get reference incidence angles
  call get_reference_incidence_angles(sat, msu_ref_ia, amsu_ref_ia)

  ! Write reference ia
  write(6,*)   "channel= ",channel
  write(6,111) "msu ref ia  = ",msu_ref_ia
  write(6,111) "amsu ref ia = ",amsu_ref_ia
  write(101,*)   "channel= ",channel
  write(101,111) "msu ref ia  = ",msu_ref_ia
  write(101,111) "amsu ref ia = ",amsu_ref_ia
  111 format(a,f7.2)


  ! Compute tb for specified channel at reference incidence angle
  write(6,  *) "Computing ",trim(channel_label)
  write(101,*) "Computing ",trim(channel_label)
  do isat=1,num_sats
     if(.not. use_sat(isat)) cycle
     write(6,  *) isat,sat_label(isat)
     write(101,*) isat,sat_label(isat)
     do iyear=start_year,end_year
        do imonth=1,12

           if(.not. sat(imonth,iyear,isat)%success) cycle

           select case(trim(sat(imonth,iyear,isat)%instr_type))
              case("msu");  ref_ia = msu_ref_ia
              case("amsu"); ref_ia = amsu_ref_ia
              case default; stop "Error - invalid instrument type encountered"
           end select

           ! Compute channel value at specified incidence angle. Load "anom" grid.
           call compute_ch_tb(sat(imonth,iyear,isat),ref_ia)

        end do
     end do
  end do 


  ! Apply elevation mask to channel 2
  !if(channel_label == "c2") call apply_elevation_mask( sat )

  ! Write grid files
  call write_grid_files( sat, grid_path_0 )


  ! Compute diurnal drift corrections
  if(make_diurnal_correction) then 
     write(6,  *) "Computing diurnal drift corrections"
     write(101,*) "Computing diurnal drift corrections"
     call compute_diurnal_drift_corrections( sat, slopes_am, islopes_am, am_ref_time, isat_ref=11, isat2=10 )
     call compute_diurnal_drift_corrections( sat, slopes_pm, islopes_pm, pm_ref_time, isat_ref=14, isat2=12 )

     call write_diurnal_drift_slopes( slopes_am, slopes_pm, islopes_am, islopes_pm )

     call apply_diurnal_drift_correction( sat, slopes_am, slopes_pm )

  else
     write(6,  *) "Diurnal drift corrections not computed"
     write(101,*) "Diurnal drift corrections not computed"

  end if


  call write_grid_files( sat, grid_path_1 )

  call compute_monthly_grid_anomalies( sat,ac ) ! This also computes the AM/PM annual cycles

  call write_grid_files( sat, grid_path_2 )

  ! Write time series for N6 and N9
  outfile = "n06_n09_time_series."//trim(channel_label)//".txt" ! DEBUG
  call write_global_time_series_for_sats( sat, (/2,5/), outfile ) ! DEBUG

  ! Write time series for N11 and N12
  outfile = "n11_n12_time_series."//trim(channel_label)//".txt" ! DEBUG
  call write_global_time_series_for_sats( sat, (/7,8/), outfile ) ! DEBUG

  ! Write time series for N12 and N14
  outfile = "n12_n14_time_series."//trim(channel_label)//".txt" ! DEBUG
  call write_global_time_series_for_sats( sat, (/8,9/), outfile ) ! DEBUG


  !=====================================
  ! WRITE OVERLAP TABLE
  write(6,    *) "BEFORE TREND ADJUSTMENT AND BIAS REMOVAL"
  write(101,  *) "BEFORE TREND ADJUSTMENT AND BIAS REMOVAL"
  write(6,  '(15x,a,a,a,a,a,a,a)') "bias","     trend "," stddev ","#_months_overlap ","sat_a_trend ","sat_b_trend   (note: global, trend is K/decade)"
  write(101,'(15x,a,a,a,a,a,a,a)') "bias","     trend "," stddev ","#_months_overlap ","sat_a_trend ","sat_b_trend   (note: global, trend is K/decade)"

  do icase = 1,13

     SELECT CASE(icase) ! isat_a and isat_b use John Christy's satellite designations
        case( 1); isat_a = 1;  isat_b = 2  ! TN - N6
        case( 2); isat_a = 3;  isat_b = 2  ! N7 - N6
        case( 3); isat_a = 5;  isat_b = 2  ! N9 - N6

        case( 4); isat_a = 4;  isat_b = 3  ! N8 - N7
        case( 5); isat_a = 6;  isat_b = 5  ! N10 - N9
        case( 6); isat_a = 7;  isat_b = 6  ! N11 - N10 

        case( 7); isat_a = 8;  isat_b = 7  ! N12 - N11
        case( 8); isat_a = 9;  isat_b = 8  ! N14 - N12
        case( 9); isat_a = 10; isat_b = 9  ! N15 - N14

        case(10); isat_a = 11; isat_b = 10 ! AQUA - N15
        case(11); isat_a = 12; isat_b = 11 ! N18 - AQUA 
        case(12); isat_a = 13; isat_b = 11 ! METOP - AQUA

        case(13); isat_a = 14; isat_b = 12 ! N19 - N18
     END SELECT

     call compute_bias_between_sats(sat, isat_a, isat_b, global_bias, global_stddev, num_overlap_months, remove_bias = .FALSE. )
     call compute_global_trend_between_sats(sat,isat_a, isat_b, relative_trend, sat_a_trend, sat_b_trend, remove_trend=.FALSE. ) 

     write(6,  '(a,3(1x,f8.3),1x,i6,1x,f8.3,1x,f8.3,1x,f8.3)') sat_label(isat_a)//" - "//sat_label(isat_b)//": ", global_bias, relative_trend, global_stddev, num_overlap_months, sat_a_trend, sat_b_trend
     write(101,'(a,3(1x,f8.3),1x,i6,1x,f8.3,1x,f8.3,1x,f8.3)') sat_label(isat_a)//" - "//sat_label(isat_b)//": ", global_bias, relative_trend, global_stddev, num_overlap_months, sat_a_trend, sat_b_trend

  end do
  !=====================================

  ! Trend Adjustments
  if( make_trend_adjustments ) then
     ! Adjust trend in isat0 such that the slope difference
     ! with the overlap periods is mininmized (weighted by number
     ! of days of in each overlap).

     ! N11 adjustment relative to N10 and N12
     isat0 = 7 ! N11
     isat1 = 6 ! N10
     isat2 = 8 ! N12
     call automated_trend_adjustment(sat,isat0,isat1,isat2)

     ! N14 adjustment relative to N12 and N15
     isat0 = 9  ! N14
     isat1 = 8  ! N12
     isat2 = 10 ! N15

     call automated_trend_adjustment(sat,isat0,isat1,isat2)
     !write(6,*) "N14 TREND ADJUSTMENT TURNED OFF!!!!!!!!!!!!!" ! DEBUG
  else
     write(6,*) "TREND ADJUSTMENTS TURNED OFF!!!"
  end if   


  !=====================================
  ! Remove intersatellite relative biases. Bias removals are cumulative; first case 1 removal, then case 2, etc.
  write(6,    *) "AFTER TREND ADJUSTMENT AND BIAS REMOVAL"
  write(101,  *) "AFTER  TREND ADJUSTMENT AND BIAS REMOVAL"
  write(6,  '(15x,a,a,a,a,a,a,a)') "bias","     trend "," stddev ","#_months_overlap ","sat_a_trend ","sat_b_trend   (note: global, trend is K/decade)"
  write(101,'(15x,a,a,a,a,a,a,a)') "bias","     trend "," stddev ","#_months_overlap ","sat_a_trend ","sat_b_trend   (note: global, trend is K/decade)"

  BIAS_REMOVAL_LOOP: do icase = 1,13

     SELECT CASE(icase) ! isat_a and isat_b use John Christy's satellite designations
        case( 1); isat_a = 1;  isat_b = 2  ! TN - N6
        case( 2); isat_a = 3;  isat_b = 2  ! N7 - N6
        case( 3); isat_a = 5;  isat_b = 2  ! N9 - N6

        case( 4); isat_a = 4;  isat_b = 3  ! N8 - N7
        case( 5); isat_a = 6;  isat_b = 5  ! N10 - N9
        case( 6); isat_a = 7;  isat_b = 6  ! N11 - N10 

        case( 7); isat_a = 8;  isat_b = 7  ! N12 - N11
        case( 8); isat_a = 9;  isat_b = 8  ! N14 - N12
        case( 9); isat_a = 10; isat_b = 9  ! N15 - N14

        case(10); isat_a = 11; isat_b = 10 ! AQUA - N15
        case(11); isat_a = 12; isat_b = 11 ! N18 - AQUA 
        case(12); isat_a = 13; isat_b = 11 ! METOP - AQUA

        case(13); isat_a = 14; isat_b = 12 ! N19 - N18
     END SELECT

     call compute_bias_between_sats(sat, isat_a, isat_b, global_bias, global_stddev, num_overlap_months, remove_bias = .TRUE. )
     call compute_bias_between_sats(sat, isat_a, isat_b, global_bias, global_stddev, num_overlap_months, remove_bias = .FALSE. ) ! This is called to get the final bias values
     call compute_global_trend_between_sats(sat,isat_a, isat_b, relative_trend, sat_a_trend, sat_b_trend, remove_trend=.FALSE. ) 

     write(6,  '(a,3(1x,f8.3),1x,i6,1x,f8.3,1x,f8.3,1x,f8.3)') sat_label(isat_a)//" - "//sat_label(isat_b)//": ", global_bias, relative_trend, global_stddev, num_overlap_months, sat_a_trend, sat_b_trend
     write(101,'(a,3(1x,f8.3),1x,i6,1x,f8.3,1x,f8.3,1x,f8.3)') sat_label(isat_a)//" - "//sat_label(isat_b)//": ", global_bias, relative_trend, global_stddev, num_overlap_months, sat_a_trend, sat_b_trend

  end do BIAS_REMOVAL_LOOP
  !=====================================


  ! Write grid files
  call write_grid_files( sat, grid_path_3 )

  ! Merge satellites
  call merge_satellites(sat, msat, ac) ! This also computes the residual and final annual cycles.

  ! If ch3, apply ch3 correction
  if(channel_label=="c3") call apply_ch3_correction( msat )

  call write_merged_grid_files( msat, grid_path_merged)

  ! Compute trends for specificed satellite (trends are deg K per month)
  call compute_trend_grid( msat, trend_grid ) ! Merged product in msat

    ! Convert trend from K/month to K/decade
  where(trend_grid/=badvalue) trend_grid = trend_grid*120
  
  ! Write trend grid to file
  write(outfile,'("trend_",a2,"_",a,".dat")') channel_label,trim(version)
  write(6,  *) "trend file = ",trim(outfile)
  write(101,*) "trend file = ",trim(outfile)

  open(10,file=trim(outfile),form="unformatted",access="stream",status="replace")
  write(10) real(trend_grid,4)
  close(10)

  ! Compute global time series of the merged anomalies and the trend in them. Write the time series to a file.
  call compute_global_time_series_and_trend( msat, global_time_series, global_trend ) ! Returned trend in K/month
  write(6,  '(a,f8.4,a)') "global trend = ",global_trend*120," K/decade"
  write(101,'(a,f8.4,a)') "global trend = ",global_trend*120," K/decade"


end subroutine process_channel

!=============================================================================

subroutine write_monthly_annual_cycle_product(ac, layer, data_product_directory)
  type(annual_cycles),intent(IN) :: ac
  character(len=2),intent(IN) :: layer ! lt, mt, ls, or tp
  character(len=150),intent(IN) :: data_product_directory

  character(len=150) :: outfile
  character(len=2) :: upcase_layer
  integer :: iarray(nlat,nlon)! Used for integer output
  integer :: imonth

  !=======================
  ! FILE: tltmonacg_6.0
  !       tmtmonacg_6.0
  !       tlsmonacg_6.0
  !       ttpmonacg_6.0
  !=======================
  select case(layer)
     case("lt"); outfile = "tltmonacg_"//trim(version); upcase_layer = "LT"
     case("mt"); outfile = "tmtmonacg_"//trim(version); upcase_layer = "MT"
     case("ls"); outfile = "tlsmonacg_"//trim(version); upcase_layer = "LS"
     case("tp"); outfile = "ttpmonacg_"//trim(version); upcase_layer = "TP"
  end select      
  outfile = trim(data_product_directory)//trim(outfile)

  open(10,file=trim(outfile),status="replace")

   ! WRITE PRODUCT DATA SET: gridded monthly annual cycle, e.g. tmtmonacg_6.0
   open(10,file=trim(outfile),status="replace")
   do imonth = 1, 12
      ! Write header
      write(10,'(i12,i10,i2.2,i10,i2.2,a12)') imonth, 1981-1900, 1, 2010-1900, 12, upcase_layer

      ! Write data
      where(ac%final(imonth,:,:)/=badvalue)
         iarray = ac%final(imonth,:,:)*100
      elsewhere
         iarray = -9999
      endwhere

      do ilat=1,72
         write(10,'(16i5)') iarray(ilat,:)
      end do
   
   end do   

  
  close(10)

end subroutine write_monthly_annual_cycle_product

END MODULE MERGE_MODULE


!========================
! Main program
!========================
program main
  use MERGE_MODULE
  use TREND_MODULE
  implicit none

  ! Variables to hold the merged product for each channel.
  ! Dims are (imonth,iyear) .
  ! These are allocated in call to process_channel().
  type(merged_sat_structure),allocatable,target :: c2_msat(:,:), c3_msat(:,:), c4_msat(:,:), lt_msat(:,:)
  type(merged_sat_structure),pointer :: ptr_msat(:,:)
  type(annual_cycles),target :: c2_ac, c3_ac, c4_ac, lt_ac
  type(annual_cycles),pointer :: ptr_ac

  type(monthly_regional_product_collection),target :: mt_monthly, lt_monthly, ls_monthly, tp_monthly
  type(monthly_regional_product_collection),pointer :: ptr_monthly

  character(len=2),parameter :: layers(4) = (/"mt","lt","ls","tp"/)


  character(len=150) :: data_product_directory

  character(len=2) :: layer

  integer :: ilayer
  real(4) :: trend_grid(nlat,nlon)
  character(len=150) :: outfile

  real,allocatable,dimension(:,:) :: global_time_series_c2, global_time_series_c3, global_time_series_c4, global_time_series_lt ! Dims are (year,month)
  real :: global_trend
  integer :: imonth, iyear

  character(len=5) :: surface_type

  real :: harvest ! DEBUG
  real(4) :: ac(nlat,nlon,12)

  character(len=150) :: dump_file

  call setup()

  !==============================================
  ! Global variables in module MERGE_MODULE

  version                          = "6.0"
  make_trend_adjustments           = .TRUE.
  make_diurnal_correction          = .TRUE.
  use_precip_in_diurnal_correction = .TRUE.  ! This is only used if above is also TRUE.

  !version                          = "6.0test1"
  !make_trend_adjustments           = .FALSE.
  !make_diurnal_correction          = .TRUE.
  !use_precip_in_diurnal_correction = .TRUE.  ! This is only used if above is also TRUE.


  !version                          = "6.0test2"
  !make_trend_adjustments           = .FALSE.
  !make_diurnal_correction          = .FALSE.
  !use_precip_in_diurnal_correction = .FALSE.  ! This is only used if above is also TRUE.

  !==============================================


  ! Open 101 dump file
  dump_file = "fort.101_"//trim(version)
  write(6,*) "dump_file= ",trim(dump_file)
  open(101,file=trim(dump_file),status="replace")


  !==============================================
  ! Process channels 2,3,4. Make LT.
  !==============================================
  call process_channel( "c2", c2_msat, c2_ac )
  call process_channel( "c3", c3_msat, c3_ac )
  call process_channel( "c4", c4_msat, c4_ac )

  ! Generate LT from combination of c2, c3, and c4
  call make_lt(c2_msat, c3_msat, c4_msat, lt_msat, c2_ac, c3_ac, c4_ac, lt_ac)


  !==============================================
  ! Smooth ocean and land (in longitude)
  !==============================================
  surface_type = "ocean"
  call smooth_ocean_or_land(c2_msat,surface_type)
  call smooth_ocean_or_land(c3_msat,surface_type)
  call smooth_ocean_or_land(c4_msat,surface_type)
  call smooth_ocean_or_land(lt_msat,surface_type)
   
  surface_type = "land"
  call smooth_ocean_or_land(c2_msat,surface_type)
  call smooth_ocean_or_land(c3_msat,surface_type)
  call smooth_ocean_or_land(c4_msat,surface_type)
  call smooth_ocean_or_land(lt_msat,surface_type)


  !==============================================
  ! Remove any remaining resiudal annual cycle,
  ! computed for period 1981 to 2010.
  ! Update "final" annual cycles in c2_ac etc.
  !==============================================
  call remove_residual_annual_cycle( c2_msat, 1981, 2010, c2_ac )
  call remove_residual_annual_cycle( c3_msat, 1981, 2010, c3_ac )
  call remove_residual_annual_cycle( c4_msat, 1981, 2010, c4_ac )
  call remove_residual_annual_cycle( lt_msat, 1981, 2010, lt_ac )


  !==============================================
  ! Compute and write trend grids
  !==============================================
  do ilayer=1,4

     layer = layers(ilayer)
     select case(layer)
        case("mt"); ptr_msat=>c2_msat; outfile = "trend.mt."//trim(version)//".dat"
        case("lt"); ptr_msat=>lt_msat; outfile = "trend.lt."//trim(version)//".dat"
        case("ls"); ptr_msat=>c4_msat; outfile = "trend.ls."//trim(version)//".dat"
        case("tp"); ptr_msat=>c3_msat; outfile = "trend.tp."//trim(version)//".dat"
     end select      

     ! Make grid map of trends
     call compute_trend_grid(ptr_msat, trend_grid)
      
     ! Convert trend from K/month to K/decade
     where(trend_grid/=badvalue) trend_grid = trend_grid*120
      
     ! Write trend grid to file
     write(6,  *) "trend file = ",trim(outfile)
     write(101,*) "trend file = ",trim(outfile)
     open(10,file=trim(outfile),form="unformatted",access="stream",status="replace")
     write(10) real(trend_grid,4)
     close(10)

  end do

  ! Compute LT global trend Write the time series to a file.
  call compute_global_time_series_and_trend( lt_msat, global_time_series_lt, global_trend ) ! Returned trend in K/month
  write(6,  '(a,f8.4,a)') "LT global trend = ",global_trend*120," K/decade"
  write(101,'(a,f8.4,a)') "LT global trend = ",global_trend*120," K/decade"


  !==============================================
  ! Compute and write monthly products for each layer
  !==============================================
  do ilayer = 1,4
     layer = layers(ilayer)
     
     select case(layer)
        case("mt"); ptr_monthly=>mt_monthly; ptr_msat=>c2_msat; ptr_ac=>c2_ac
        case("lt"); ptr_monthly=>lt_monthly; ptr_msat=>lt_msat; ptr_ac=>lt_ac
        case("ls"); ptr_monthly=>ls_monthly; ptr_msat=>c4_msat; ptr_ac=>c4_ac
        case("tp"); ptr_monthly=>tp_monthly; ptr_msat=>c3_msat; ptr_ac=>c3_ac
     end select      

     call compute_monthly_regional_products( ptr_msat, ptr_monthly )
     write(data_product_directory,'(a2,a,a,a)') layer,"_data_products_",trim(version),"/"
     call write_monthly_regional_products(ptr_monthly, layer, data_product_directory)
     call write_monthly_gridded_products( ptr_msat,    layer, data_product_directory)

     call write_monthly_annual_cycle_product(ptr_ac, layer, data_product_directory)

  end do   



  !==============================================  
  ! Compute and write c2,c3,c4 global trends
  !==============================================
  call compute_global_time_series_and_trend( c2_msat, global_time_series_c2, global_trend ) ! Returned trend in K/month
  write(6,  '(a,f8.4,a)') "C2 global trend = ",global_trend*120," K/decade"
  write(101,'(a,f8.4,a)') "C2 global trend = ",global_trend*120," K/decade"

  call compute_global_time_series_and_trend( c3_msat, global_time_series_c3, global_trend ) ! Returned trend in K/month
  write(6,  '(a,f8.4,a)') "C3 global trend = ",global_trend*120," K/decade"
  write(101,'(a,f8.4,a)') "C3 global trend = ",global_trend*120," K/decade"


  call compute_global_time_series_and_trend( c4_msat, global_time_series_c4, global_trend ) ! Returned trend in K/month
  write(6,  '(a,f8.4,a)') "C4 global trend = ",global_trend*120," K/decade"
  write(101,'(a,f8.4,a)') "C4 global trend = ",global_trend*120," K/decade"



  !==============================================
  ! Write global time series to txt file
  !==============================================
  outfile = "global_time_series."//trim(version)//".txt"
  open(22,file=trim(outfile),status="replace")
  write(22,76) "year","mn","ch2","ch3","ch4","lt"
  do iyear=start_year,end_year
     do imonth=1,12
        write(22,75) iyear,imonth, global_time_series_c2(imonth,iyear), &
                                   global_time_series_c3(imonth,iyear), &
                                   global_time_series_c4(imonth,iyear), &
                                   global_time_series_lt(imonth,iyear)
        75 format(i4,1x,i2,4(1x,f9.4))
        76 format(a4,1x,a2,4(1x,a9))

     end do
  end do   
  close(22)

end program main
