!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: main.flyer.f90
!
! PURPOSE: Fortran 95 main program and supporting modules
!
!
! DESCRIPTION: This main program takes as input 2-line element sets in the various
!              satellite files (e.g. trn.txt, n15.txt, etc.) and computes satellite altitudes
!              and incidence angles as a function of time.
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS: Satellite 2-line element (TLE) files: e.g. trn.txt, n15.txt, etc.
!
! OUTPUTS:   File: sat_altitudes.dat
!            File: incidence_angles.dat
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE FLYER
  use valid_range_module
  implicit none
  private
  public:: alt,ia,compute_sat_alt_and_ia,num_sats,start_year,end_year,badvalue,sat_label,apply_valid_ranges

  integer,parameter :: nlat=72
  real,parameter :: badvalue = -999
  
  integer,parameter :: start_year = 1978
  integer,parameter :: end_year   = 2017

  integer,parameter :: num_msu_prints=6
  integer,parameter :: num_amsu_prints=15

  integer,parameter :: num_msu_sats=9
  integer,parameter :: num_amsu_sats=5
  integer,parameter :: num_sats=num_msu_sats+num_amsu_sats

  real :: alt(12,start_year:end_year,nlat,num_sats) ! Altitude. Dims are (month,year,lat,sat)
  real :: ia( 12,start_year:end_year,nlat,num_amsu_prints,num_sats) ! Incidence angle. Dims are (month,year,lat,print,sat). For msu, only prints 1..6 are used
  
  real,parameter :: G=6.673d-11   ! Gravitational constant, N-m^2/kg^2
  real,parameter :: Me=5.97219d24 ! Mass of earth, kg
  real,parameter :: Re=6378.1d3   ! Earth equatorial radiius (m)
  real,parameter :: Rp=6356.8d3   ! Earth polar radiius (m)
  real,parameter :: Rm=6371.0d3   ! Earth mean radiius (m)
  real,parameter  :: Pi = 3.1415926535897932385d0
  real,parameter :: degrees=pi/180.d0
  real,parameter :: radians=180.d0/pi
  
  character(len=3),parameter :: sat_label(num_sats)=(/"trn","n06","n07","n08","n09","n10","n11","n12","n14","n15","aqu","n18","met","n19"/)

  integer :: i
  real,parameter :: msu_va(num_msu_prints) = (/ (9.47*(6-i),i=1,6) /)
  real,parameter :: amsu_va(num_amsu_prints) = (/ (48.333333 - (i-1)*3.333333,i=1,15) /)

CONTAINS

! Computes satellite altitudes and incidence angles.
subroutine compute_sat_alt_and_ia()
  use INTERPOLATE
  use CALENDAR
  character(len=150),parameter :: path = "./data_files/" ! Path to two-line-element (TLE) files
  character(len=150) :: infile
  character(len=100) :: line1, line2
  character(len=100),allocatable :: buffer(:,:)
  integer :: num_tle,i,year,day,min_year,max_year,iecc,ilat,month
  real :: T ! Period of orbit
  real :: ecc ! Eccentricity
  real :: mm  ! Mean motion (revs per day)
  real :: a,b ! Semi-major/minor axis
  real,allocatable :: rorbit(:,:) ! Average orbit radius (m), Dims are (year,jday)
  real :: mid_month(12) ! Holds julian day of 15th of each month for a given year
  real :: numerator,denominator,lat
  real :: earth_radius_at_lat(nlat) ! meter
  integer :: first,last,icount,num_prints
  real :: avg_orbit_radius
  character(len=4) :: inst_type
  integer :: iprint,isat
  real :: sat_va

  ! Initialize the altitude and incidence angle arrays
  alt = badvalue
  ia  = badvalue

  ! Compute Earth's radius at geodetic latitude
  do ilat=1,nlat
     lat = -88.75 + (ilat-1)*2.5
     numerator   = (Re**2*cos(lat*degrees))**2 + (Rp**2*sin(lat*degrees))**2
     denominator = (Re*cos(lat*degrees))**2 + (Rp*sin(lat*degrees))**2
     earth_radius_at_lat(ilat) = sqrt(numerator/denominator)
  end do

  SAT_LOOP: do isat=1,num_sats

     ! Select input TLE file
     select case(isat)
     case( 1); infile = "trn.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case( 2); infile = "n06.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case( 3); infile = "n07.txt"; num_prints = num_msu_prints ; inst_type = "msu"  
     case( 4); infile = "n08.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case( 5); infile = "n09.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case( 6); infile = "n10.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case( 7); infile = "n11.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case( 8); infile = "n12.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case( 9); infile = "n14.txt"; num_prints = num_msu_prints ; inst_type = "msu" 
     case(10); infile = "n15.txt"; num_prints = num_amsu_prints; inst_type = "amsu"
     case(11); infile = "aqu.txt"; num_prints = num_amsu_prints; inst_type = "amsu"  
     case(12); infile = "n18.txt"; num_prints = num_amsu_prints; inst_type = "amsu"  
     case(13); infile = "met.txt"; num_prints = num_amsu_prints; inst_type = "amsu"  
     case(14); infile = "n19.txt"; num_prints = num_amsu_prints; inst_type = "amsu"  
     case default; stop "Error - compute_sat_altitudes(): Invalid isat"
     end select

     ! Open TLE file and count number of entries
     open(1,file=trim(path)//trim(infile),action="read",status="old")
     num_tle = 0
     do 
        read(1,'(a100)',end=99) line1
        read(1,'(a100)',end=99) line2
        num_tle = num_tle + 1
     end do
     99   continue
     close(1)
  
     ! Allocate buffer array used to hold TLE's, read the TLE file into it.
     if(allocated(buffer)) deallocate(buffer)
     allocate(buffer(num_tle,2))
     open(1,file=trim(path)//trim(infile),action="read",status="old")
     do i=1,num_tle
        read(1,'(a100)') buffer(i,1)
        read(1,'(a100)') buffer(i,2)
     end do
     close(1)

     ! Find max and min year in TLE's
     min_year = +999999
     max_year = -999999
     do i=1,num_tle
        read(buffer(i,1),'(18x,i2,i3)') year,day
        if(year>70) then
           year = year + 1900
        else
           year = year + 2000
        end if
        if(year<min_year) min_year = year
        if(year>max_year) max_year = year
     end do

     if(allocated(rorbit)) deallocate(rorbit)
     allocate(rorbit(min_year:max_year,366))
     rorbit = badvalue

     ! Read year, day, eccentricity, mean motion from TLE, then compute other parameters.
     do i=1,num_tle
        read(buffer(i,1),'(18x,i2,i3)') year,day
        if(year>70) then
           year = year + 1900
        else
           year = year + 2000
        end if
        
        ! Read ecccentricity and mean motion from TLE
        read(buffer(i,2),'(26x,i7,19x,f11.0)') iecc, mm
        ecc = iecc/10.e6

        ! Compute period and semi-major/minor axis
        T = 86400/mm
        a = ((T/(2*Pi))**2*G*Me)**(1.d0/3)
        b = a*sqrt(1-ecc**2)   
     
        ! Compute orbit radius as average of a & b
        rorbit(year,day) = (a+b)*0.5
     end do

     ! Fill missing days in year with linear interpolation. Interpolation is only done.
     ! No extrapolation is performed. This results in days at beginning/end of
     ! yeas before/after first/last existing value remaining missing.
      call fill_missing_data_2D(rorbit,real(badvalue))

      !$$$! DEBUG BLOCK
      !$$$if(isat==5) then
      !$$$   do year=min_year,max_year
      !$$$      do day=1,366
      !$$$         if(rorbit(year,day)/=badvalue) then
      !$$$            write(44,'(i2,1x,i4.4,1x,i3.3,1x,f,1x,f)') isat,year,day,year+day/366.,rorbit(year,day)/1000.
      !$$$         else
      !$$$            write(44,'(i2,1x,i4.4,1x,i3.3,1x,f,1x,f)') isat,year,day,year+day/366.
      !$$$         end if   
      !$$$      end do
      !$$$   end do
      !$$$end if   

     do year=min_year,max_year
        do month=1,12

           ! Compute first/last julian day of current month
           first = first_of_month(year,month)
           last =  last_of_month(year,month)

           ! Compute average orbit radius over the month.
           icount = 0
           avg_orbit_radius = 0
           do day=first,last
              if(rorbit(year,day)==badvalue) cycle
              avg_orbit_radius = avg_orbit_radius + rorbit(year,day)
              icount = icount + 1
           end do
           if(icount>=1) then ! Require at values in month
              avg_orbit_radius = avg_orbit_radius/icount
           else
              !write(6,*) isat,year,month,"icount=",icount ! DEBUG
              avg_orbit_radius = badvalue
           end if

           write(6,*) sat_label(isat),year,month," icount = ",icount

           ! Compute altitude and incidence angle at latidue from averaged orbit radius and earth
           ! radius at geodetic latitude.
           do ilat=1,nlat
              if(avg_orbit_radius/=badvalue) then
                 alt(month,year,ilat,isat) = (avg_orbit_radius - earth_radius_at_lat(ilat))/1000. ! 1000 converts to km

                 do iprint=1,num_prints
                    if(trim(inst_type)=="msu" ) sat_va = msu_va(iprint)
                    if(trim(inst_type)=="amsu") sat_va = amsu_va(iprint)
                    if(sat_va == badvalue) cycle
                    call compute_surface_incidence_angle(avg_orbit_radius, earth_radius_at_lat(ilat), sat_va, ia(month,year,ilat,iprint,isat))
                 end do

              end if
           end do

        end do
     end do
  
     ! Clean up
     if(allocated(buffer)) deallocate(buffer)
     if(allocated(rorbit)) deallocate(rorbit)

  end do SAT_LOOP

end subroutine compute_sat_alt_and_ia

subroutine compute_surface_incidence_angle( rorbit, rearth, sat_va, surf_ia )
  real,intent(IN) :: rorbit  ! Radius of satellite orbit (units for rorbit and rearth must be same)
  real,intent(IN) :: rearth  ! Radius of earth 
  real,intent(IN) :: sat_va  ! View angle at satellite(deg)
  real,intent(INOUT):: surf_ia ! Incidence angle at surface(deg)

  surf_ia = asin( Rorbit/Rearth * sin(sat_va*degrees))/degrees

end subroutine compute_surface_incidence_angle

subroutine apply_valid_ranges()
  integer :: iyear,imonth,isat
  integer :: year1,month1,year2,month2

  ! This routines applies the valid ranges to arrays alt and ia
  do isat=1,num_sats

     ! Get start/end of record for current satellite. Use product "ls" since it has
     ! a longer record for N9.
     call start_of_record(isat,year1,month1,"ls")
     call end_of_record(isat,year2,month2,"ls")

     do iyear=start_year,end_year
        if(iyear<year1 .or. iyear>year2) then
           alt(:,iyear,:,isat) = badvalue
           ia( :,iyear,:,:,isat) = badvalue
           cycle
        end if   
        do imonth= 1,12
           if(iyear==year1 .and. imonth<month1) then
              alt(imonth,iyear,:,isat) = badvalue
              ia( imonth,iyear,:,:,isat) = badvalue
              cycle
           end if
           if(iyear==year2 .and. imonth>month2) then
              alt(imonth,iyear,:,isat) = badvalue
              ia( imonth,iyear,:,:,isat) = badvalue
              cycle
           end if
        end do
     end do
  end do

end subroutine apply_valid_ranges


!$$$subroutine compute_surface_incidence_angle( altitude, sat_va, surf_ia )
!$$$  ! Given the satellite altitude and the view_angle at the satellite,
!$$$  ! compute the incidence angle at the surface.
!$$$  real(8),intent(IN) :: altitude ! Satellite altitude (km)
!$$$  real(8),intent(IN) :: sat_va   ! Satellite view angle (deg)
!$$$  real(8),intent(OUT):: surf_ia  ! Incidence angle at the surface (deg)
!$$$
!$$$  real(8),parameter  :: R = 6371 ! Earth's mean readius (km)
!$$$  real(8),parameter  :: Pi = 3.1415926535897932385d0
!$$$  real(8),parameter  :: degrees=Pi/180
!$$$
!$$$  surf_ia = asin( (R+altitude)/R * sin(sat_va*degrees))/degrees
!$$$
!$$$end subroutine compute_surface_incidence_angle

END MODULE FLYER

!==========================
! Main program
!==========================

program main
  use FLYER
  implicit none
  integer :: isat,ilat,iyear,imonth,iprint
  real :: min0(15),max0(15)

  ! Compute satellite altitudes and incidences angles. Loaded in to 
  ! variables alt and ia in module FLYER.
  call compute_sat_alt_and_ia()
  call apply_valid_ranges()

  !do isat=1,num_sats
  !   write(6,'(12x,72(1x,f8.2))') (/(-88.75+(ilat-1)*2.5,ilat=1,36)/)    
  !   do iyear=start_year,end_year
  !      do imonth=1,12
  !         write(6,10) sat_label(isat),iyear,imonth,alt(imonth,iyear,1:36,isat)
  !         10 format(a3,1x,i4.4,1x,i2.2,1x,72(1x,f8.2))
  !      end do
  !   end do   
  !end do   

  ! Write altitude file
  open(1,file="sat_altitudes.dat",form="unformatted",access="stream",status="replace")
  write(1) real(alt,4)
  close(1)

  ! Write incidence angle file
  open(1,file="incidence_angles.dat",form="unformatted",access="stream",status="replace")
  write(1) real(ia,4)
  close(1)

  ! Make table of min/max incidence angles for each sat at each print
  do isat=1,14
     do iprint=1,15
        if( ALL(ia(:,:,:,iprint,isat)==badvalue) ) then
           min0(iprint) = badvalue
           max0(iprint) = badvalue
        else   
           min0(iprint) = minval(ia(:,:,:,iprint,isat),mask=ia(:,:,:,iprint,isat)/=badvalue)
           max0(iprint) = maxval(ia(:,:,:,iprint,isat),mask=ia(:,:,:,iprint,isat)/=badvalue)
        end if   
     end do
     if(isat<=9) then ! MSU
        write(6,22) "min: ",sat_label(isat),min0(1:6)
        write(6,22) "max: ",sat_label(isat),max0(1:6)
     else ! AMSU
        write(6,23) "min: ",sat_label(isat),min0(:)
        write(6,23) "max: ",sat_label(isat),max0(:)
     end if   
     22 format(a5,1x,a3,1x,6( 1x,f8.3))
     23 format(a5,1x,a3,1x,15(1x,f8.3))

  end do 

end program main

! MSU view angles
!#   At satellite           Including geometry effect (833 km orbit)
!1      47.35                  56.27
!2      37.88                  43.97
!3      28.41                  32.55
!4      18.94                  21.53
!5        9.47                  10.72
!6        0.00                    0.00
!Note the at satellite angles are in multiples of 9.47.

! AMSU view angle prints 1..15
!view_angle_at_sat = 48.33333 - (print_number -1)*3.33333

!msu_va(num_msu_prints) = (/47.35, 37.88, 28.41, 18.94, 9.47, 0.0 /)


!$$$!--------------------------
!$$$! Given the satellite altitude and the view_angle at the satellite,
!$$$! compute the incidence angle at the surface.
!$$$subroutine compute_surface_incidence_angle( altitude, sat_va, surf_ia )
!$$$   real(8),intent(IN) :: altitude ! Satellite altitude (km)
!$$$   real(8),intent(IN) :: sat_va   ! Satellite view angle (deg)
!$$$   real(8),intent(OUT):: surf_ia  ! Incidence angle at the surface (deg)
!$$$
!$$$   real(8),parameter  :: R = 6371 ! Earth's mean readius (km)
!$$$   real(8),parameter  :: Pi = 3.1415926535897932385d0
!$$$   real(8),parameter  :: degrees=Pi/180
!$$$
!$$$   surf_ia = asin( (R+altitude)/R * sin(sat_va*degrees))/degrees
!$$$
!$$$end subroutine compute_surface_incidence_angle
!$$$!--------------------------

