!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: regional_masks.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION: Routines used to mask out unwanted areas when computing regional products.
!              0's are used for masked areas, 1's are used for non-masked areas.
!              The names given to the masks are of the regions that are NOT masked out.
!              For example the "tropical mask" masks all areas except the tropics, i.e. it has 1's in
!              the tropics and 0's elsewhere, hence when the mask is multiplied by a global grid, only
!              the tropics has non-zero values.
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE REGIONAL_MASKS
  implicit none
  
  type mask_structure
     real(4),dimension(72,144) :: all, land, ocean
  end type   

  type mask_collection
     type(mask_structure) :: global, NH, SH, Trpcs, NoExt, SoExt, NoPol, SoPol
     real(4),dimension(72,144) :: usa48, usa49, aus, elevation
  end type  

  type(mask_collection) :: masks

CONTAINS
! Load or generate lat/lon masks for the regional products.
! This consists of masks for USA48, USA49,  and AUST,
! and  "all, land and ocean" masks for: Globe, NH, SH, Trpcs, NoExt, SoExt, NoPol, SoPol.
! These are places in the above variables. For example, the global land mask is in masks%global%land and the
! mask for usa48 is in masks%usa48.
subroutine load_regional_masks()
   integer :: ix,iy
   real,dimension(144,72) :: usa, usak, aus ! Working arrays
   integer :: ielev(144,72)
   real(4) :: elev(72,144)
   character(len=150),parameter :: path = "./masks/" 

   !===================================================
   ! Load masks for USA, USA with Alaska, and Australia
   !===================================================

   !(1,1) is -178.75, -88.75
   ! ix = 1,144,  iy = 1,72

   usa = 0
   usak = 0
   aus = 0

   ! USA48 mask
   do ix = 24,35
      usa(ix,55) = 1.0
   end do
   do ix = 24,37
      usa(ix,54) = 1.0
      usa(ix,53) = 1.0
   end do
   usa(43,54) = 1.0
   usa(44,54) = 1.0
   usa(41,53) = 1.0
   usa(42,53) = 1.0
   do ix = 24,41
      usa(ix,52) = 1.0
   end do
   do ix = 25,41
      usa(ix,51) = 1.0
   end do
   do ix = 26,40
      usa(ix,50) = 1.0
   end do
   do ix = 31,36
      usa(ix,49) = 1.0
   end do
   usa(39,49) = 1.0
   usa(33,48) = 1.0
   ! North border
   do ix = 24,34
      usa(ix,56) = 0.8
   end do
   usa(35,56) = 0.7
   usa(36,56) = 0.3
   usa(36,55) = 0.9
   usa(37,55) = 0.9
   usa(38,55) = 0.8
   usa(38,54) = 0.5
   usa(38,53) = 0.9
   usa(39,55) = 0.3
   usa(39,54) = 0.9
   usa(39,53) = 0.9
   usa(40,53) = 0.8
   usa(41,54) = 0.3
   usa(42,54) = 0.7
   usa(44,55) = 0.1
   usa(45,55) = 0.9
   ! West Coast
   usa(23,56) = 0.4
   usa(23,55) = 0.8
   usa(23,54) = 0.9
   usa(23,53) = 0.9
   usa(23,52) = 0.5
   usa(24,51) = 0.7
   usa(24,50) = 0.1
   usa(25,50) = 0.6
   ! Southern Border
   usa(27,49) = 0.2
   usa(28,49) = 0.5
   usa(29,49) = 0.4
   usa(30,49) = 0.5
   usa(31,48) = 0.2
   usa(32,48) = 0.4
   usa(33,47) = 0.5
   usa(34,48) = 0.6
   usa(35,48) = 0.1
   usa(36,48) = 0.2
   usa(37,49) = 0.9
   usa(38,49) = 0.9
   usa(39,48) = 0.1
   usa(40,48) = 0.7
   usa(40,47) = 0.7
   ! East Coast
   usa(46,55) = 0.2
   usa(45,54) = 0.8
   usa(46,54) = 0.1
   usa(43,53) = 0.8
   usa(44,53) = 0.4
   usa(42,52) = 0.8
   usa(43,52) = 0.1
   usa(42,51) = 0.7
   usa(41,50) = 0.5
   usa(40,49) = 0.4

   ! USA48 plus Alaska
   do iy = 61,64
      do ix = 9,15
         usak(ix,iy) = 1.0
      end do
   end do
   usak(8,64) = 1.
   usak(7,61) = 1.
   usak(8,61) = 1.
   usak(10,60) = 1.
   usak(7,58) = 0.1
   usak(8,59) = 0.2
   usak(9,59) = 0.3
   usak(10,59) = 0.1
   usak(11,59) = 0.1
   usak(18,59) = 0.1
   usak(19,59) = 0.7
   usak(20,59) = 0.6
   usak(21,59) = 0.2
   usak(8,60) = 0.4
   usak(9,60) = 0.5
   usak(11,60) = 0.5
   usak(12,60) = 0.2
   usak(16,60) = 0.1
   usak(17,60) = 0.2
   usak(18,60) = 0.5
   usak(19,60) = 0.4
   usak(6,61) = 0.3
   usak(12,61) = 0.8
   usak(13,61) = 0.9
   usak(14,61) = 0.7
   usak(16,61) = 0.6
   usak(6,62) = 0.1
   usak(7,62) = 0.4
   usak(8,62) = 0.7
   usak(16,62) = 0.6
   usak(5,63) = 0.5
   usak(6,63) = 0.4
   usak(7,63) = 0.6
   usak(8,63) = 0.8
   usak(16,63) = 0.6
   usak(6,64) = 0.2
   usak(7,64) = 0.7
   usak(8,64) = 1.0
   usak(16,64) = 0.6
   usak(8,65) = 0.1
   usak(9,65) = 0.3
   usak(10,65) = 0.4
   usak(11,65) = 0.2
   usak(12,65) = 0.1
   usak(13,65) = 0.1
   do ix = 24,35
      usak(ix,55) = 1.0
   end do
   do ix = 24,37
      usak(ix,54) = 1.0
      usak(ix,53) = 1.0
   end do
   usak(43,54) = 1.0
   usak(44,54) = 1.0
   usak(41,53) = 1.0
   usak(42,53) = 1.0
   do ix = 24,41
      usak(ix,52) = 1.0
   end do
   do ix = 25,41
      usak(ix,51) = 1.0
   end do
   do ix = 26,40
      usak(ix,50) = 1.0
   end do
   do ix = 31,36
      usak(ix,49) = 1.0
   end do
   usak(39,49) = 1.0
   usak(33,48) = 1.0
   ! North border
   do ix = 24,34
      usak(ix,56) = 0.8
   end do
   usak(35,56) = 0.7
   usak(36,56) = 0.3
   usak(36,55) = 0.9
   usak(37,55) = 0.9
   usak(38,55) = 0.8
   usak(38,54) = 0.5
   usak(38,53) = 0.9
   usak(39,55) = 0.3
   usak(39,54) = 0.9
   usak(39,53) = 0.9
   usak(40,53) = 0.8
   usak(41,54) = 0.3
   usak(42,54) = 0.7
   usak(44,55) = 0.1
   usak(45,55) = 0.9
   ! West Coast
   usak(23,56) = 0.4
   usak(23,55) = 0.8
   usak(23,54) = 0.9
   usak(23,53) = 0.9
   usak(23,52) = 0.5
   usak(24,51) = 0.7
   usak(24,50) = 0.1
   usak(25,50) = 0.6
   ! Southern Border
   usak(27,49) = 0.2
   usak(28,49) = 0.5
   usak(29,49) = 0.4
   usak(30,49) = 0.5
   usak(31,48) = 0.2
   usak(32,48) = 0.4
   usak(33,47) = 0.5
   usak(34,48) = 0.6
   usak(35,48) = 0.1
   usak(36,48) = 0.2
   usak(37,49) = 0.9
   usak(38,49) = 0.9
   usak(39,48) = 0.1
   usak(40,48) = 0.7
   usak(40,47) = 0.7
   ! East Coast
   usak(46,55) = 0.2
   usak(45,54) = 0.8
   usak(46,54) = 0.1
   usak(43,53) = 0.8
   usak(44,53) = 0.4
   usak(42,52) = 0.8
   usak(43,52) = 0.1
   usak(42,51) = 0.7
   usak(41,50) = 0.5
   usak(40,49) = 0.4

   ! Australia (with Tasmania)
   aus(131,19) = 0.7
   aus(132,19) = 0.1
   aus(130,20) = 0.1
   aus(131,20) = 0.5
   aus(132,20) = 0.2
   aus(129,21) = 0.25
   aus(130,21) = 0.3
   aus(131,21) = 0.5
   aus(132,21) = 0.2
   aus(127,22) = 0.1
   aus(128,22) = 0.3
   aus(133,22) = 0.1
   do iy = 22,29
      do ix = 129,132
         aus(ix,iy) = 1.0
      end do
   end do
   aus(131,29) = 0.6
   aus(132,29) = 0.
   aus(132,28) = 0.7
   aus(133,23) = 0.6
   aus(133,24) = 1.0
   aus(133,25) = 1.0
   aus(133,26) = 1.0
   aus(133,27) = 0.6
   aus(134,24) = 0.2
   aus(134,25) = 0.3
   aus(134,26) = 0.2
   aus(119,23) = 0.8
   aus(120,23) = 0.8
   aus(121,23) = 0.3
   aus(122,23) = 0.3
   aus(126,23) = 0.1
   aus(127,23) = 0.6
   aus(128,23) = 0.8
   do ix = 120,128
      aus(ix,24) = 1.0
   end do
   aus(118,24) = 0.9
   aus(123,24) = 0.9
   aus(124,24) = 0.7
   aus(125,24) = 0.6
   aus(126,24) = 0.9
   do ix = 119,128
      do iy = 25,28
         aus(ix,iy) = 1.0
      end do
   end do
   aus(119,28) = 0.7
   aus(120,28) = 0.9
   aus(118,25) = 0.2
   aus(118,26) = 0.5
   aus(118,27) = 0.5
   aus(118,28) = 0.1
   aus(121,29) = 0.4
   do ix = 122,128
      aus(ix,29) = 1.0
   end do
   aus(122,30) = 0.4
   do ix = 123,130
      aus(ix,30) = 1.0
   end do
   aus(127,30) = 0.8
   aus(128,30) = 0.3
   aus(129,30) = 0.5
   aus(131,30) = 0.2
   aus(123,31) = 0.3
   aus(124,31) = 0.1
   aus(125,31) = 0.95
   aus(126,31) = 1.0
   aus(127,31) = 0.4
   aus(129,31) = 0.35
   aus(130,31) = 0.5
   aus(125,32) = 0.2
   aus(126,32) = 0.3
   aus(127,32) = 0.1
   aus(129,32) = 0.2
   aus(130,32) = 0.1

   ! Transpose arrays and store in structure variables
   masks%usa48 = transpose(usa)
   masks%usa49 = transpose(usak)
   masks%aus   = transpose(aus)

   ! Read global elevation file
   !open(55,file="elev_ascii.txt",action="read",status="old")
   !read(55,'(24i5)') ielev
   !close(55)
   !elev = transpose(ielev)
   
   ! Alternate version of elevation
   open(55,file="elev_72x144.dat",action="read",form="unformatted",access="stream",status="old")
   read(55) elev
   close(55)
   
   masks%elevation = elev

   !=================
   ! Global masks
   ! lat=90S..90N
   ! lat index = 1..72
   !=================
   masks%global%all = 1

   ! Global land mask is based on areas with less than 50% water coverage 
   open(1,file="land_from_wc_72x144.dat",action="read",form="unformatted",access="stream")
   read(1) masks%global%land
   close(1)


   masks%global%ocean = 0
   where(masks%global%land==0) masks%global%ocean = 1

   !=================
   ! NH masks
   ! lat=0..90N
   ! lat index = 37..72
   !=================   
   masks%NH%all = 0
   masks%NH%all(37:72,:) = 1
   masks%NH%land = masks%NH%all * masks%global%land
   masks%NH%ocean = masks%NH%all * masks%global%ocean

   !=================
   ! SH masks
   ! lat=90S..0
   ! lat index = 1..36
   !=================   
   masks%SH%all = 0
   masks%SH%all(1:36,:) = 1
   masks%SH%land = masks%SH%all * masks%global%land
   masks%SH%ocean = masks%SH%all * masks%global%ocean

   !=================
   ! Tropical masks
   ! lat=20S..20N
   ! lat index = 29..44
   !================= 
   masks%trpcs%all = 0
   masks%trpcs%all(29:44,:) = 1
   masks%trpcs%land = masks%trpcs%all * masks%global%land
   masks%trpcs%ocean = masks%trpcs%all * masks%global%ocean

   !=================
   ! NoExt masks
   ! lat=20N..90N
   ! lat index = 45..72
   !================= 
   masks%NoExt%all = 0
   masks%NoExt%all(45:72,:) = 1
   masks%NoExt%land = masks%NoExt%all * masks%global%land
   masks%NoExt%ocean = masks%NoExt%all * masks%global%ocean

   !=================
   ! SoExt masks
   ! lat=90S..20S
   ! lat index = 1..28
   !=================
   masks%SoExt%all = 0
   masks%SoExt%all(1:28,:) = 1
   masks%SoExt%land = masks%SoExt%all * masks%global%land
   masks%SoExt%ocean = masks%SoExt%all * masks%global%ocean

   !=================
   ! NoPol masks
   ! lat=60N..90N
   ! lat index = 61..72
   !================= 
   masks%NoPol%all = 0
   masks%NoPol%all(61:72,:) = 1
   masks%NoPol%land = masks%NoPol%all * masks%global%land
   masks%NoPol%ocean = masks%NoPol%all * masks%global%ocean

   !=================
   ! SoPol masks
   ! lat=90S..60S
   ! lat index = 1..12
   !================= 
   masks%SoPol%all = 0
   masks%SoPol%all(1:12,:) = 1
   masks%SoPol%land = masks%SoPol%all * masks%global%land
   masks%SoPol%ocean = masks%SoPol%all * masks%global%ocean


!/////// DEBUG BLOCK
   ! WRITE EACH MASK TO A FILE
   open(1,file=trim(path)//"/masks%elevation"     ,access="stream",form="unformatted",status="replace"); write(1) real(masks%elevation   ,4) ; close(1)
   open(1,file=trim(path)//"/masks%usa48"         ,access="stream",form="unformatted",status="replace"); write(1) real(masks%usa48       ,4) ; close(1)
   open(1,file=trim(path)//"/masks%usa49"         ,access="stream",form="unformatted",status="replace"); write(1) real(masks%usa49       ,4) ; close(1)
   open(1,file=trim(path)//"/masks%aus"           ,access="stream",form="unformatted",status="replace"); write(1) real(masks%aus         ,4) ; close(1)
   open(1,file=trim(path)//"/masks%global%all"    ,access="stream",form="unformatted",status="replace"); write(1) real(masks%global%all  ,4) ; close(1)
   open(1,file=trim(path)//"/masks%global%land"   ,access="stream",form="unformatted",status="replace"); write(1) real(masks%global%land ,4) ; close(1)
   open(1,file=trim(path)//"/masks%global%ocean"  ,access="stream",form="unformatted",status="replace"); write(1) real(masks%global%ocean,4) ; close(1)
   open(1,file=trim(path)//"/masks%NH%all"        ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NH%all      ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NH%land"       ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NH%land     ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NH%ocean"      ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NH%ocean    ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SH%all"        ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SH%all      ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SH%land"       ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SH%land     ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SH%ocean"      ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SH%ocean    ,4) ; close(1)
   open(1,file=trim(path)//"/masks%trpcs%all"     ,access="stream",form="unformatted",status="replace"); write(1) real(masks%trpcs%all   ,4) ; close(1)
   open(1,file=trim(path)//"/masks%trpcs%land"    ,access="stream",form="unformatted",status="replace"); write(1) real(masks%trpcs%land  ,4) ; close(1)
   open(1,file=trim(path)//"/masks%trpcs%ocean"   ,access="stream",form="unformatted",status="replace"); write(1) real(masks%trpcs%ocean ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NoExt%all"     ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NoExt%all   ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NoExt%land"    ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NoExt%land  ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NoExt%ocean"   ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NoExt%ocean ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SoExt%all"     ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SoExt%all   ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SoExt%land"    ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SoExt%land  ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SoExt%ocean"   ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SoExt%ocean ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NoPol%all"     ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NoPol%all   ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NoPol%land"    ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NoPol%land  ,4) ; close(1)
   open(1,file=trim(path)//"/masks%NoPol%ocean"   ,access="stream",form="unformatted",status="replace"); write(1) real(masks%NoPol%ocean ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SoPol%all"     ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SoPol%all   ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SoPol%land"    ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SoPol%land  ,4) ; close(1)
   open(1,file=trim(path)//"/masks%SoPol%ocean"   ,access="stream",form="unformatted",status="replace"); write(1) real(masks%SoPol%ocean ,4) ; close(1)
!/////// END DEBUG BLOCK

end subroutine load_regional_masks
END MODULE REGIONAL_MASKS
