!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: smooth.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION: This module contains subprograms used for data series smoothing.
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE SMOOTH
   use LINEAR_ALGEBRA
   implicit none

CONTAINS

! Fit Fourier series of specified order to input vector F.
subroutine fit_fourier_series( f, order, fit, badvalue )
   real    :: f(:)   ! Input: function to be fit
   integer :: order  ! Input: Order of the Fourier series to be fit
   real    :: fit(:) ! Output: Computed fit to the function f
   real    :: badvalue

   ! Internal variables
   integer :: n, nn, i, j, iorder, ii
   real,allocatable :: A(:,:), B(:), X(:), A0(:,:)
   real,parameter :: pi = 3.141592654


   !-------------------------------
   ! Begin executable statements
   !-------------------------------
   
   ! Check size of argument list vectors
   n = size(f)
   if(size(fit)/=n) STOP "ERROR fit_fourier_series(): Argument list vectors of unequal sizes"

   ! Verify order >= 1
   if( order < 1 ) stop "Error - fit_fourier_series(): Order must be >= 1"

   ! Set fit to badvalue if more than 9 (.0625*144) missing values (WDB NOTE: this was raised from 5 used in 2012 version)
   if( COUNT(f==badvalue) > 9 ) then
      !write(6,*) "From fit_fouries_series(): COUNT(f==badvalue) = ",COUNT(f==badvalue) ! DEBUG
      fit = badvalue
      return
   end if


   ! Allocate arrays used in regression fit
   nn = order*2 + 1 ! Number of coefficients for xth order fourier fit
   allocate( A(n,nn), A0(n,nn), B(n), X(nn) )

   ! Load regression arrays
   B = f
   do i=1,n  ! loop over days
      A(i,1) = 1

      ! Loop over order
      ii=1
      do iorder = 1, order
         ii=ii+1
         A(i,ii) = SIN(iorder * 2*Pi * (i-1.)/n)
         ii=ii+1
         A(i,ii) = COS(iorder * 2*Pi * (i-1.)/n)
      end do

   end do

   ! Handle missing values
   where(f==badvalue) B = 0
   A0 = A
   do i=1,nn
      where(f==badvalue) A0(:,i) = 0
   end do

   ! Solves A*X=B for X (using Lapack based routine).
   ! Use matrix A with 0's for missing value days in computing coefficients.
   call regress(A0,B,X)

   ! Apply coefficients to get fitted curve. Use matrix A, with no missing days, for this.
   fit = MATMUL(A,X)

   ! Clean-up
   deallocate( A, A0, B, X )

end subroutine fit_fourier_series

subroutine smooth_121( x, xs, badvalue )
   real :: x(:), xs(:)
   real :: badvalue
   integer :: n
   real,parameter :: filter(3)= (/.25,.5,.25/)
   real,allocatable :: r(:), r2(:)
   integer :: ilon

   n=size(x)
   allocate(r(1:n))
   allocate(r2(0:n+1))

   r = x
   r2(1:n) = r

   r2(0) = r(n)  ! This handles wrap
   r2(n+1) = r(1)

   r = badvalue
   do ilon=1,n
      if(ANY(r2(ilon-1:ilon+1)==badvalue)) cycle
      r(ilon) = SUM( r2(ilon-1:ilon+1)*filter)
   end do   

   xs = r
   deallocate(r,r2)

end subroutine smooth_121

end module SMOOTH

