!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: trend.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION: Routines used in compouting time series trends.
!
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE TREND_MODULE
   implicit none

   private
   public trend, trend_with_badvalues, detrend
   public trend_2var_with_badvalues

   interface trend_2var_with_badvalues
      module procedure trend_2var_with_badvalues4, trend_2var_with_badvalues8
   end interface

   interface detrend
      module procedure detrend_1arg, detrend_2arg
   end interface

   interface trend_with_badvalues
      module procedure trend_with_badvalues4, trend_with_badvalues8
   end interface   

   interface trend
      module procedure trend4, trend8
   end interface   

CONTAINS

subroutine trend8(x,f,a,b)
   implicit none
   real(8) :: x(:),f(:)  ! Input:  Vectors x,f in f=a*x + b, vector f being fit to
   real(8) :: a,b        ! Output: Slope, intercept

   real(8) :: xmean,fmean,x2,fx
   integer :: i,n

   n=size(x)
   if(size(f)/=n) stop "Error - trend(): inconsistent input vector lengths"

   xmean = sum(x)/n
   fmean = sum(f)/n
   x2    = sum(x*x)
   fx    = sum(f*x)
   
   a = (fx - n*fmean*xmean)/(x2-N*xmean*xmean)
   b = fmean - a*xmean

end subroutine trend8

subroutine trend4(x,f,a,b)
   implicit none
   real(4) :: x(:),f(:)  ! Input:  Vectors x,f in f=a*x + b, vector f being fit to
   real(4) :: a,b        ! Output: Slope, intercept

   real(8) :: xmean,fmean,x2,fx
   integer :: i,n

   n=size(x)
   if(size(f)/=n) stop "Error - trend(): inconsistent input vector lengths"

   xmean = sum(x)/n
   fmean = sum(f)/n
   x2    = sum(x*x)
   fx    = sum(f*x)
   
   a = (fx - n*fmean*xmean)/(x2-N*xmean*xmean)
   b = fmean - a*xmean

end subroutine trend4

subroutine trend_with_badvalues8(x,f,a,b,badvalue,num_good_values)
   implicit none
   real(8) :: x(:),f(:)  ! Input:  Vectors x,f in f=a*x + b, vector f being fit to
   real(8) :: a,b        ! Output: Slope, intercept
   real(8) :: badvalue
   integer :: num_good_values

   real(8) :: xmean,fmean,x2,fx
   integer :: i,n

   if(size(f)/=size(x) ) stop "Error - trend_with_badvalues(): inconsistent input vector lengths"


   n=COUNT(f/=badvalue)
   num_good_values = n

   if(n<2) then
      a=badvalue
      b=badvalue
      RETURN
   end if

   xmean = sum(x,mask=f/=badvalue)/n
   fmean = sum(f,mask=f/=badvalue)/n
   x2    = sum(x*x,mask=f/=badvalue)
   fx    = sum(f*x,mask=f/=badvalue)
   
   a = (fx - n*fmean*xmean)/(x2-N*xmean*xmean)
   b = fmean - a*xmean

end subroutine trend_with_badvalues8

subroutine trend_with_badvalues4(x,f,a,b,badvalue,num_good_values)
   implicit none
   real(4) :: x(:),f(:)  ! Input:  Vectors x,f in f=a*x + b, vector f being fit to
   real(4) :: a,b        ! Output: Slope, intercept
   real(4) :: badvalue
   integer :: num_good_values

   real(8) :: xmean,fmean,x2,fx
   integer :: i,n

   if(size(f)/=size(x) ) stop "Error - trend_with_badvalues(): inconsistent input vector lengths"


   n=COUNT(f/=badvalue)
   num_good_values = n

   if(n<2) then
      a=badvalue
      b=badvalue
      RETURN
   end if

   xmean = sum(x,mask=f/=badvalue)/n
   fmean = sum(f,mask=f/=badvalue)/n
   x2    = sum(x*x,mask=f/=badvalue)
   fx    = sum(f*x,mask=f/=badvalue)
   
   a = (fx - n*fmean*xmean)/(x2-N*xmean*xmean)
   b = fmean - a*xmean

end subroutine trend_with_badvalues4

subroutine trend_2var_with_badvalues4(x1,x2,f,b1,b2,c,badvalue,num_good_values)
   use LINEAR_ALGEBRA
   implicit none
   real(4) :: x1(:),x2(:),f(:)  ! Input:  Vectors f,x1,x2 in f=b1*x1 + b2*x2 + c. f is vector being fit to.
   real(4) :: b1,b2, c        ! Output: Slope, slope, intercept
   real(4) :: badvalue
   integer :: num_good_values

   logical :: mask(size(f))
   integer :: i,ii,n

   real(8),allocatable :: A(:,:),X(:),B(:)

   if(size(x1)/=size(f) .or. size(x2)/=size(f)) stop "Error - trend_2var_with_badvalues(): inconsistent input vector lengths"
   
   ! Set mask
   mask = .FALSE.
   n = 0
   do i=1,size(f)
      if(x1(i)==badvalue .or. x2(i)==badvalue .or. f(i)==badvalue) cycle
      n=n+1
      mask(i) = .TRUE.
   end do  

   if(n<3) then
      c= badvalue
      b1 = badvalue
      b2 = badvalue
      return
   end if   

   ! Allocate matrics A,X,B
   allocate(A(n,3),X(3),B(n))

   ! Load matrices A,B
   ii=0
   do i=1,size(f)
      if(.not. mask(i)) cycle
      ii=ii+1
      A(ii,1) = 1
      A(ii,2) = x1(i)
      A(ii,3) = x2(i)
      B(ii)   = f(i)
   end do   
   
   ! Call LAPACK based routine to perform the regression
   call regress(A,B,X)


   ! Get resulting slopes and intercept
   c  = X(1)
   b1 = X(2)
   b2 = X(3)
            
   num_good_values = n

   deallocate(A,X,B)

end subroutine trend_2var_with_badvalues4

subroutine trend_2var_with_badvalues8(x1,x2,f,b1,b2,c,badvalue,num_good_values)
   use LINEAR_ALGEBRA
   implicit none
   real(8) :: x1(:),x2(:),f(:)  ! Input:  Vectors f,x1,x2 in f=b1*x1 + b2*x2 + c. f is vector being fit to.
   real(8) :: b1,b2, c        ! Output: Slope, slope, intercept
   real(8) :: badvalue
   integer :: num_good_values

   logical :: mask(size(f))
   integer :: i,ii,n

   real(8),allocatable :: A(:,:),X(:),B(:)

   if(size(x1)/=size(f) .or. size(x2)/=size(f)) stop "Error - trend_2var_with_badvalues(): inconsistent input vector lengths"
   
   ! Set mask
   mask = .FALSE.
   n = 0
   do i=1,size(f)
      if(x1(i)==badvalue .or. x2(i)==badvalue .or. f(i)==badvalue) cycle
      n=n+1
      mask(i) = .TRUE.
   end do  

   if(n<3) then
      c= badvalue
      b1 = badvalue
      b2 = badvalue
      return
   end if   

   ! Allocate matrics A,X,B
   allocate(A(n,3),X(3),B(n))

   ! Load matrices A,B
   ii=0
   do i=1,size(f)
      if(.not. mask(i)) cycle
      ii=ii+1
      A(ii,1) = 1
      A(ii,2) = x1(i)
      A(ii,3) = x2(i)
      B(ii)   = f(i)
   end do   
   
   ! Call LAPACK based routine to perform the regression
   call regress(A,B,X)


   ! Get resulting slopes and intercept
   c  = X(1)
   b1 = X(2)
   b2 = X(3)
            
   num_good_values = n

   deallocate(A,X,B)

end subroutine trend_2var_with_badvalues8

subroutine detrend_2arg( x, f )  ! Does not handle badvalues
   implicit none
   real(8) :: x(:), f(:)
   real(8) :: a,b ! Slope, intercept
   integer :: n,i

   n = size(x)
   if(size(f)/=n) stop "Error - detrend(): inconsistent input vector lengths"

   call trend(x,f,a,b) ! a=slope, b=intercept

   do i=1,n
      f(i) = f(i) - a*(x(i)-x(1))
   end do

end subroutine detrend_2arg


subroutine detrend_1arg( f )  ! Does not handle badvalues
   implicit none
   real(8) :: f(:)
   real(8) :: a,b ! Slope, intercept
   integer :: n,i
   real(8),allocatable :: x(:)

   n = size(f)
   allocate(x(n))

   do i=1,n
      x(i) = i
   end do


   call trend(x,f,a,b) ! a=slope, b=intercept

   do i=1,n
      f(i) = f(i) - a*(x(i)-x(1))
   end do

   deallocate(x)

end subroutine detrend_1arg


END MODULE

!==========================
! Main program for testing
!==========================
!program main
!   use TREND_MODULE
!   implicit none
!   real(8) :: x(5),f(5)
!   real(8) :: a,b
!
!   x = (/1,2,3,4,5/)
!   f = (/10.2,10.4,10.6,10.8,11./)
!
!   call trend(x,f,a,b)
!
!   write(6,*) "a=",a
!   write(6,*) "b=",b
!
!   call detrend(f)
!   !call detrend(x,f)
!
!   call trend(x,f,a,b)
!
!   write(6,*) "a=",a
!   write(6,*) "b=",b
!
!end    
   
   
