!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: valid_range.mod.f90
!
! PURPOSE: FORTRAN 95 MODULE
!
! DESCRIPTION: This module contains subprograms used to read and interpret the file "valid_sat_months.txt".
!              That file contains a table designating for each satellite which months are to be used in the
!              processing for that satellite.
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS: File - valid_sat_months.txt
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================

MODULE VALID_RANGE_MODULE
  implicit none
  integer,parameter,private :: badvalue = -999
  integer,parameter,private :: start_year=1978, stop_year=2020, num_sats=14

CONTAINS

function valid_sat_month(ksat, kyear, kmonth, channel) result(flag)
  integer :: ksat   ! satellite designation (JC's) 
  integer :: kyear  ! 4-digit year
  integer :: kmonth ! month(1..12)
  character(len=2),optional :: channel ! One of "lt","mt", or "ls"
  logical :: flag   ! Returned as TRUE if input sat, year, month is flagged
                    ! in valid sat month input file. Returned as FALSE otherwise.


  character(len=150),parameter :: infile = "valid_sat_months.txt"

  integer :: iyear, imonth, ivec(num_sats)
  logical,save :: first_call = .TRUE.
  
  integer,save :: sat_array(start_year:stop_year,1:12, num_sats)
  integer,save :: ls_sat_array(start_year:stop_year,1:12, num_sats)

  if(first_call) then
     sat_array = 0
     open(1,file=trim(infile),action="read",status="old")
     read(1,*) ! Read header line
     do 
        read(1,*,END=99) iyear,imonth,ivec(:)
        sat_array(iyear,imonth,:) = ivec(:)
     end do

     99 continue
     close(1)

     ! Extend LS sat array to include 3/87 to 10/88 for NOAA-9 (ksat=5)
     ls_sat_array = sat_array
     iyear = 1987
     do imonth=3,12
        ls_sat_array(iyear,imonth,5) = 1
     end do
     iyear = 1988
     do imonth=1,10
        ls_sat_array(iyear,imonth,5) = 1
     end do        

     first_call = .FALSE.
  end if

  if(kyear<start_year .or. kyear>stop_year .or. &
     kmonth<1         .or. kmonth>12       .or. &
     ksat<1           .or. ksat>num_sats )  then
     write(6,*) "ksat  = ",ksat
     write(6,*) "kyear = ",kyear
     write(6,*) "kmonth= ",kmonth
     stop "Error - valid_sat_month(): month, year, or sat is out of range"
  end if
   
  if(sat_array(kyear,kmonth,ksat)==1)then
     flag = .TRUE.
  else
     flag = .FALSE.
  end if

  ! Override if LS
  if(present(channel)) then
     if(channel=="ls" .or. channel=="c4") then
        if(ls_sat_array(kyear,kmonth,ksat)==1)then
           flag = .TRUE.
        else
           flag = .FALSE.
        end if
     end if
  end if   

end function valid_sat_month

! Gets the year/month of the first valid entry in the record for the specified satellite.
subroutine start_of_record(ksat,year,month,channel)
  integer,intent(IN) :: ksat
  integer,intent(OUT) :: year,month
  character(len=2),optional,intent(IN) :: channel ! One of mt,lt,ls (only used if set to ls)

  integer :: iyear,imonth,i1,i2
  logical :: flag

  year = badvalue
  month = badvalue

  do iyear=start_year,stop_year
     i1=1; i2=12
     if(iyear==1978) i1=12
     do imonth=i1,i2
        if(present(channel)) then
           if(channel=="ls" .or. channel=="c4") then
              flag = valid_sat_month(ksat,iyear,imonth,channel)
           else
              flag = valid_sat_month(ksat,iyear,imonth)            
           end if
        else
           flag = valid_sat_month(ksat,iyear,imonth)
        end if

        if(flag) then
           year = iyear
           month = imonth
           return
        end if
     end do
  end do

end subroutine start_of_record

! Gets the year/month of the last valid entry in the record for the specified satellite.
subroutine end_of_record(ksat,year,month,channel)
  integer,intent(IN) :: ksat
  integer,intent(OUT) :: year,month
  character(len=2),optional,intent(IN) :: channel ! One of mt,lt,ls (only used if set to ls)

  integer :: iyear,imonth,i1,i2
  logical :: flag

  year = badvalue
  month = badvalue

  do iyear=stop_year,start_year,-1
     i1=1; i2=12
     if(iyear==1978) i1=12
     do imonth=i2,i1,-1
        if(present(channel)) then
           if(channel=="ls" .or. channel=="c4") then
              flag = valid_sat_month(ksat,iyear,imonth,channel)
           else
              flag = valid_sat_month(ksat,iyear,imonth)            
           end if
        else
           flag = valid_sat_month(ksat,iyear,imonth)
        end if
        
        if(flag) then
           year = iyear
           month = imonth
           return
        end if
        
     end do
  end do

end subroutine end_of_record

END MODULE VALID_RANGE_MODULE
