!====================================================================================================
! NAME: MSU Monthly Grid File Generator
!       Version 6.0 of the University of Alabama in Huntsville (UAH)
!       Microwave Sounding Unit(MSU/AMSU) Processing Software
!
! PURPOSE: Generate monthly averaged grids of brightness temperatures from MSU channels 2,3,4 at 
!          each view angle. 
!
!
! DESCRIPTION: This program reads monthly files previously created that contain all the MSU oribit file day for
!              for a single month. From these it creates separate monthly files for each satellite containing
!              gridded brightness temperatures at each view angle position and channel.

!
! AUTHOR(S): Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: April, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS: 1) MSU monthly collections of orbit file data. 
!         2) runparms_all (the list of the files in 1)
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================
MODULE MSU_MODULE
  implicit none
  real,parameter :: badvalue = -999
  integer,parameter :: num_prints = 11 ! Number of prints across the scan (prints 1 and 11 are limbmost, 6 is nadir)
  integer,parameter :: num_angles = 6  ! Number of view angles in scan (prints with same angle on opposite
                                       ! sides of scan are binned together). I.e. angle 1 is for prints 1,11.
                                       ! Angle 6 is for nadr (print 6).

  integer,parameter :: num_channels = 3 ! MSU channels 2,3,4

  ! This structure holds the entries read from file "runparms_all"
  type monthly_file_structure
     character(len=150) :: monthly_file
     integer            :: ksat ! Satellite id (Roy Spencer satellite designatioins)
     integer            :: kday1  ! Starting julian day
     integer            :: kday2  ! Ending julian day
     integer            :: kyear  ! Year
     integer            :: kmon   ! Month
  end type monthly_file_structure


CONTAINS

! Input grid is averaged and resized (igrid too)
subroutine average_monthly_grid_to_72_144(grid, igrid)
  real(4),allocatable  :: grid(:,:,:,:)  ! Dims are (nlat,nlon,num_angles,2:4)
  integer(2),allocatable :: igrid(:,:,:,:)

  integer,parameter :: nlat=360,nlon=720
  integer,parameter :: nlat2=72, nlon2=144

  real(4),allocatable :: grid2(:,:,:,:)  ! Working grids
  integer(2),allocatable :: igrid2(:,:,:,:)
  real :: value
  integer :: iwt

  integer :: ilat,ilon,ilat2,ilon2,iprint,ichannel

  allocate(grid2(nlat2,nlon2,num_angles,2:4))
  allocate(igrid2(nlat2,nlon2,num_angles,2:4))

  if(size(grid,1)/=nlat .or. size(grid,2)/=nlon .or. size(grid,3)/=num_angles .or. size(grid,4)/=num_channels) then
     stop "Error - average_monthly_grid_to_72_144(): Unexpected dimensions for input array grid"
  endif  

  grid2 = badvalue
  igrid2 = badvalue

  do ichannel = 2,4
     do iprint=1,num_angles
        ilat2 = 0
        do ilat=1,nlat,5
           ilat2 = ilat2 + 1
           ilon2 = 0
           do ilon=1,nlon,5
              ilon2 = ilon2 + 1
              value  = SUM( grid(ilat:ilat+4,ilon:ilon+4,iprint,ichannel)* &
                            igrid(ilat:ilat+4,ilon:ilon+4,iprint,ichannel), &
                            mask=grid(ilat:ilat+4,ilon:ilon+4,iprint,ichannel)/=badvalue .and. &
                            igrid(ilat:ilat+4,ilon:ilon+4,iprint,ichannel)/=badvalue )

              iwt = SUM(igrid(ilat:ilat+4,ilon:ilon+4,iprint,ichannel), &
                    mask=grid(ilat:ilat+4,ilon:ilon+4,iprint,ichannel)/=badvalue .and. &
                    igrid(ilat:ilat+4,ilon:ilon+4,iprint,ichannel)/=badvalue  )

              if(iwt>=1) then
                 grid2(ilat2,ilon2,iprint,ichannel) = value/iwt
              else
                 grid2(ilat2,ilon2,iprint,ichannel) = badvalue
              end if   
                                     
           enddo
        enddo
     enddo
  enddo   

  if(allocated(grid)) deallocate(grid)
  if(allocated(igrid)) deallocate(igrid)
  allocate(grid(nlat2,nlon2,num_angles,2:4))
  allocate(igrid(nlat2,nlon2,num_angles,2:4))

  grid = grid2
  igrid = igrid2

  deallocate(grid2,igrid2)

end subroutine average_monthly_grid_to_72_144

subroutine build_monthly_grid_360x720( mfs, lat, lon, rtb, day, year, grid, igrid )
  type(monthly_file_structure),intent(in) :: mfs
  real,intent(in) :: lat(:,:),lon(:,:) ! Dims (#prints, #scans)
  real,intent(in) :: rtb(:,:,:) ! Dims are 4 channels, #prints, #scans)
  integer(4),intent(in) :: day(:), year(:) ! Julian day, 4-digit year. Dims are (#scans)

  real(4),allocatable  :: grid(:,:,:,:)  ! Dims are (nlat,nlon,num_angles,2:4)
  integer(2),allocatable :: igrid(:,:,:,:)

  integer,parameter :: nlat=360, nlon=720
  real(8),parameter :: dlat=180.d0/nlat, dlon=360.d0/nlon

  ! Working array used to accumulate the grids
  real(8) :: grid0(nlat,nlon,num_angles,2:4)

  integer :: iscan,num_scans,iprint,ichannel,ilat,ilon,iangle
  real :: latitude, longitude, tb
  integer :: ilon1,ilon2,ilat1,ilat2
  integer :: ilon_start,ilon_stop,iilat,iilon


  if(allocated(grid)) deallocate(grid)
  allocate(grid(nlat,nlon,num_angles,2:4))
  if(allocated(igrid)) deallocate(igrid)
  allocate(igrid(nlat,nlon,num_angles,2:4))

  num_scans = size(day)

  grid0 = 0
  igrid = 0
  do iscan=1,num_scans
     do iprint=1,num_prints
        latitude = lat(iprint,iscan)
        longitude = lon(iprint,iscan)
        if(latitude<-90. .or. latitude>90. .or. longitude<-180. .or. longitude>180.) cycle
        if(latitude==0.0 .and. longitude==0) cycle ! Denotes bad orbit file value

        ilat = floor( (latitude + 90.)/dlat) + 1
        ilon = floor( (longitude +180.)/dlon) + 1
        if(ilon==nlon+1) ilon = 1
        if(ilat==nlat+1) ilat = nlat

        if(iprint<=num_angles) then
           iangle = iprint
        else
           iangle = num_prints+1-iprint
        end if
        
        do ichannel=2,4
           tb = rtb(ichannel,iprint,iscan)
           if(tb==badvalue) cycle

           !$$$! Accumulate tb in grid cells.
           !$$$! Near poles the tb values are replicated over multiple cells to account for the smaller cell size (in latitude).
           !$$$if(ilat<=25 .or. ilat>=696) then
           !$$$   select case(ilat)
           !$$$      case(1,716); ilon_start=ilon-9*5; ilon_stop=ilon+9*5
           !$$$      case(6,711); ilon_start=ilon-3*5; ilon_stop=ilon+3*5
           !$$$      case(11,706); ilon_start=ilon-2*5; ilon_stop=ilon+2*5
           !$$$      case(16,701); ilon_start=ilon-1*5; ilon_stop=ilon+1*5
           !$$$      case(21,696); ilon_start=ilon-1*5; ilon_stop=ilon+1*5
           !$$$      !case default; stop "Error - build_monthly_grids(): This statment should never be executed"    ! DEBUG, COMMENTED OUT
           !$$$   end select
           !$$$
           !$$$   do ilon2=ilon_start,ilon_stop
           !$$$
           !$$$      ilon3 = ilon2
           !$$$      if(ilon3<1) then 
           !$$$         ilon3 = nlon + ilon3
           !$$$      elseif(ilon3>nlon) then
           !$$$         ilon3 = ilon3 - nlon
           !$$$      end if   
           !$$$
           !$$$      grid0(ilat,ilon3,iangle,ichannel) = grid0(ilat,ilon3,iangle,ichannel) + tb
           !$$$      igrid(ilat,ilon3,iangle,ichannel) = igrid(ilat,ilon3,iangle,ichannel) + 1              
           !$$$   end do
           !$$$else
           
           ! Get start/stop indices for 3X3 grid around current point
           ilat1 = ilat-1
           ilat2 = ilat+1
           ilon1 = ilon-1
           ilon2 = ilon+1

           if(ilat1<1) then
              ilat1 = 1
           elseif(ilat2>nlat) then
              ilat2 = nlat
           end if

           if(ilon1<1) then 
              ilon1 =  nlon + ilon1
           elseif(ilon2>nlon) then
              ilon2 = ilon2 - nlon
           end if   

           ! Distribut tb to 3x3 grid around current ilat,ilon.
           do iilat=ilat1,ilat2
              do iilon=ilon1,ilon2
                 grid0(iilat,iilon,iangle,ichannel) = grid0(iilat,iilon,iangle,ichannel) + tb
                 igrid(iilat,iilon,iangle,ichannel) = igrid(iilat,iilon,iangle,ichannel) + 1
              end do
           end do   
           !$$$end if


   
        end do
     end do   
  end do   

  ! Finalize grids
  where(igrid>0)
     grid = grid0/igrid
  elsewhere
     grid = badvalue
  endwhere   


end subroutine build_monthly_grid_360x720

! This routine reads the file paths and descriptors from "runaparms_all".
subroutine read_file_locations( list )
  type(monthly_file_structure),allocatable :: list(:)
  integer,parameter :: num_entries = 538 ! Number of entries in input file "runparms_all"


  integer :: i
  character(len=150),parameter :: param_file = "runparms_all"

  if(allocated(list)) deallocate(list)
  allocate(list(num_entries))

  open(1,file=trim(param_file),action="read",status="old")

  do i=1,num_entries
     read(1,'(a)') list(i)%monthly_file
     read(1,*) list(i)%ksat, list(i)%kday1, list(i)%kday2, list(i)%kyear, list(i)%kmon
   end do

   close(1)

end subroutine read_file_locations

subroutine read_and_calibrate_msu( mfs, lat0, lon0, rtb0, time0, secs0, day0, year0 )
  implicit none

  type(monthly_file_structure) :: mfs                                  ! Input
  real,allocatable             :: lat0(:,:), lon0(:,:), rtb0(:,:,:)    ! Output, allocated within this routine
  integer(4),allocatable       :: time0(:), secs0(:), day0(:), year0(:)! Output,  allocated within this routine

  integer :: imonth, iscan, channel

  ! Valid ranges on values for channels 2,3,4
  real :: limits(2,2:4)     ! Channel
  data limits/ 150.,280., & ! 2
               150.,250., & ! 3
               150.,250. /  ! 4


  !-----------------------------------------------------------
  ! Argument list declarations for unpack_msu_monthly_file()
  !-----------------------------------------------------------

  ! Input
  character(len=150)   :: monthly_file ! Name of orbit file
  integer(4),parameter :: max_num_scans = 110000 ! last dimension of output arrays


  integer(4) :: reference_time           ! This doesn't appear to be an input
  integer    :: ksat,kday1,kday2
  integer :: iprint = 0    ! =1 for diagnostic prints when something goes wrong, =0 otherwise.

  ! Output  
  integer(4) :: num_scans      ! Number of scans in orbit file

  integer(4) :: index_start    ! The range of scans used in max_num_scans
  integer(4) :: index_stop     

  integer(4) :: sat_id         ! Satellite id
  integer(4) :: year(max_num_scans)
  integer(4) :: day(max_num_scans)
  integer(4) :: secs(max_num_scans) ! Scan time, seconds since Greenwich 0Z on current Julian day
  integer(4) :: time78(max_num_scans) ! Scan time, seconds since Greenwich 0Z on Jan 1, 1978
  integer(2) :: counts(4,11,max_num_scans)   ! counts for channels 1..4, 11 footprints
  integer(2) :: lat(11,max_num_scans)        ! lat & lon, units are tenths of a degree
  integer(2) :: lon(11,max_num_scans)
  integer(2) :: cal_counts(16,max_num_scans) ! The 16 calibration count values
  integer(2) :: iqual1(max_num_scans)        ! Quality check 

  !-----------------------------------------------------------
  ! Argument list declarations for calibrate_msu()
  !-----------------------------------------------------------
  real    :: rtb(4,11,max_num_scans)  ! 4 channels, 11 prints, max number of scans
  logical :: badflag(max_num_scans)

  ! set caltyp=0 for linear calibration (with Tw depndence adjustments)
  ! set caltyp=1 NESDIS original nonlinear calibration technique (then Christy does adj's)
  ! set caltyp=2 for Tsan Mo's new calibration technique (1995 IEEE Trans.
  !  Geosci. & Rem. Sens., pg. 1151)
  ! set caltyp=3 for Tsan Mo's AMSU-type cal technique (Mo et al, 2000)
  integer :: caltyp=0

  ! for caltyp=0, can set linadj=1 for linear empirical adj. of ch 2 Tb
  !  linadj=2 for nonlinear adj (change coef() value declarations above)
  integer :: linadj=0  !...=1 for warm target coefficint adjustments of Tb  

  !real  :: Twcoef(9,4) = 0.04

  ! Hot Target Coefficients based on 20N-20S oceans
  real  :: Twcoef(9,4)
  !data Twcoef/9*0.04, &
  !            .048, .033, .047, .048, .033, .033, .072, .033, .072, &
  !            9*.04, &
  !            9*.04  /

  ! Updated on 9/19/13
  !data Twcoef/9*0.04, &
  !            .048, .037, .046, .048, .033, .057, .057, .029, .057, &
  !            9*.04, &
  !            9*.04  /
  ! Above replaced on 3/3/2015 with following (wdb)
  data Twcoef/9*0.04, &
              0.046, 0.038, 0.046, 0.048, 0.032, 0.057, 0.032, 0.029, 0.059, &
              0.062, 0.054, 0.035, 0.091, 0.041, 0.059, 0.062, 0.026, 0.060, &
              0.024, 0.045, 0.022, 0.032, 0.031, 0.039, 0.024, 0.045, 0.017 /

  real :: temp1av_array(max_num_scans) = 0
  real :: temp2av_array(max_num_scans) 
  real :: Qterm_array(11,max_num_scans)

  real :: temp1diff_avg, temp2diff_avg

  real :: emiss(9,4)
  !data emiss/9*0.95,                                                         &
  !   0.954,  0.956,  0.949,  0.950,  0.964,  0.931,  0.952,  0.969,  0.938,  &
  !   0.989,  0.957,  0.958,  0.949,  0.948,  0.949,  0.950,  0.973,  0.921,  &
  !   0.960,  0.940,  0.973,  0.973,  0.952,  0.956,  0.987,  0.949,  0.953/

  ! New channel 2 coefficients to replace above (wdb, 8/16/2013)
  data emiss/9*0.95,                                                         &
     0.954,  0.954,  0.943,  0.943,  0.967,  0.930,  0.941,  0.971,  0.920,  &
     0.989,  0.957,  0.958,  0.949,  0.948,  0.949,  0.950,  0.973,  0.921,  &
     0.960,  0.940,  0.973,  0.973,  0.952,  0.956,  0.987,  0.949,  0.953/

  real :: calterm(4,11,max_num_scans) 

  real :: avg_gain(9)
  real :: tw_gain(9) 

  real :: scanline_avg

  real :: min_limit, max_limit

  integer :: nn
  
  !///////////////////////////////////////////////////////////////////////////
  ! BEGIN EXECUTABLE STATEMENTS
  !///////////////////////////////////////////////////////////////////////////

  ! Argument list input assignments
  monthly_file = mfs%monthly_file
  ksat         = mfs%ksat
  kday1        = mfs%kday1
  kday2        = mfs%kday2
  iprint       = 0 ! =0 normally, =1 for diagnostic prints when something goes wrong



  year = badvalue
  day  = badvalue
  secs = badvalue
  time78 = badvalue
  lon    = badvalue
  lon    = badvalue

  CALL UNPACK_MSU_MONTHLY_FILE(monthly_file,max_num_scans,                       &                          
                               reference_time,num_scans,index_start,index_stop,  &
                               sat_id,year,day,secs,time78,counts,lat,lon,       &
                               cal_counts,iqual1,ksat,kday1,kday2,iprint )

  write(6,*) "reference_time = ",reference_time
  write(6,*) "num_scans      = ",num_scans     
  write(6,*) "index_start    = ",index_start   
  write(6,*) "index_stop     = ",index_stop    
  write(6,*) "sat_id         = ",sat_id 
  write(6,*) "ksat           = ",ksat          
  write(6,*) "kday1          = ",kday1         
  write(6,*) "kday2          = ",kday2         
  write(6,*) "iprint         = ",iprint        

  badflag(:)=.TRUE.

  CALL CALIBRATE_MSU(index_start,index_stop,sat_id,year,day,secs,  &
        time78,counts,lat,lon,cal_counts,badvalue,rtb,badflag,     &
        iprint,caltyp,linadj,twcoef,temp1av_array,temp2av_array,   &
        Qterm_array,temp1diff_avg,temp2diff_avg,emiss,calterm, avg_gain, tw_gain)

  where(rtb == 0) rtb = badvalue


  do channel=2,4

     min_limit = limits(1,channel)
     max_limit = limits(2,channel)

     ! Check for values out of range. If any values in scan are bad, throw out entire scan.
     do iscan=1,max_num_scans
        if( ANY(rtb(channel,:,iscan)<min_limit) .or. ANY(rtb(channel,:,iscan)>max_limit) ) then
           badflag(iscan) = .TRUE.
           rtb(channel,:,iscan) = badvalue
        end if
     end do

  end do

  nn = index_stop - index_start + 1

  if(allocated(lat0)) deallocate(lat0)
  if(allocated(lon0)) deallocate(lon0)
  if(allocated(rtb0)) deallocate(rtb0)
  if(allocated(time0))deallocate(time0)
  if(allocated(secs0))deallocate(secs0)
  if(allocated(day0)) deallocate(day0)
  if(allocated(year0))deallocate(year0)

  allocate(lat0(size(lat,1), nn))
  allocate(lon0(size(lon,1), nn))
  allocate(rtb0(size(rtb,1), size(rtb,2), nn))
  allocate(time0(nn))
  allocate(secs0(nn))
  allocate(day0(nn))
  allocate(year0(nn))

  lat0 = lat(:,index_start:index_stop)
  where(lat0/=badvalue) lat0=lat0/10.
  where(lat0<-90 .or. lat0>90) lat0=badvalue

  lon0 = lon(:,index_start:index_stop)
  where(lon0/=badvalue) lon0=lon0/10.
  where(lon0<-180 .or. lon0>180) lon0=badvalue

  rtb0 = rtb(:,:,index_start:index_stop)
  
  time0=time78(index_start:index_stop)
  secs0= secs(index_start:index_stop)
  day0 = day(index_start:index_stop)
  year0= year(index_start:index_stop)

  ! NOAA-9 (ksat=7) oscillator went bad on julain day 67 of 1987. Set all tb values
  ! from that day on to badvalue.
  if(ksat==7) then
     where(year0==87 .and. day0>66) 
        rtb0(2,1,:) = badvalue 
        rtb0(2,2,:) = badvalue 
        rtb0(2,3,:) = badvalue 
        rtb0(2,4,:) = badvalue 
        rtb0(2,5,:) = badvalue 
        rtb0(2,6,:) = badvalue 
        rtb0(2,7,:) = badvalue 
        rtb0(2,8,:) = badvalue 
        rtb0(2,9,:) = badvalue 
        rtb0(2,10,:)= badvalue 
        rtb0(2,11,:)= badvalue
     end where   

     where(year0>87)
        rtb0(2,1,:) = badvalue 
        rtb0(2,2,:) = badvalue 
        rtb0(2,3,:) = badvalue 
        rtb0(2,4,:) = badvalue 
        rtb0(2,5,:) = badvalue 
        rtb0(2,6,:) = badvalue 
        rtb0(2,7,:) = badvalue 
        rtb0(2,8,:) = badvalue 
        rtb0(2,9,:) = badvalue 
        rtb0(2,10,:)= badvalue 
        rtb0(2,11,:)= badvalue
     end where   
  end if  

  ! If rtb ch2 nadir tb is bad, set year to bad.
  where(rtb0(2,6,:)==badvalue) 
     year0=badvalue
     day0 =badvalue
     secs0=badvalue
  end where   
     
  
  ! Convert to 4-digit year
  where(year0/=badvalue) year0 = 1900 + year0

  ! Data where lat=0 and lon=0 is bad.
  ! Also set to bad, data for lon <-180 or >180 and
  ! also for lat<-90 or lat>90.
  where(lat0==0 .and. lon0==0) 
     rtb0(1,:,:) = badvalue
     rtb0(2,:,:) = badvalue
     rtb0(3,:,:) = badvalue
     rtb0(4,:,:) = badvalue      
  end where   
  where(lon0>180 .or. lon0<-180 .or. lat0<-90 .or. lat0>90) 
     rtb0(1,:,:) = badvalue
     rtb0(2,:,:) = badvalue
     rtb0(3,:,:) = badvalue
     rtb0(4,:,:) = badvalue      
  end where         
  
end subroutine read_and_calibrate_msu

function rs_id_to_jc_id( rs_id ) result( jc_id )
   ! Satellite ID translation table.
   ! Translate satellite IDs used by Roy Spencer (RS) to the ordering used by John Christy (JC).
   ! Note JC does not use indices 3 and 5.

   integer,intent(IN) :: rs_id
   integer :: jc_id

   select case(rs_id)
   !    RS          JC    SATELLITE
   case( 1); jc_id = 1   ! TIROS-N  PM (1:30)
   case( 2); jc_id = 2   ! NOAA-6   AM (7:30)
   case( 4); jc_id = 3   ! NOAA-7   PM     
   case( 6); jc_id = 4   ! NOAA-8   AM
   case( 7); jc_id = 5   ! NOAA-9   PM
   case( 8); jc_id = 6   ! NOAA-10  AM
   case( 9); jc_id = 7   ! NOAA-11  PM
   case( 5); jc_id = 8   ! NOAA-12  AM
   case( 3); jc_id = 9   ! NOAA-14  PM
   case(10); jc_id = 10  ! NOAA-15  AM  AMSU
   case(21); jc_id = 11  ! AQUA     PM  AMSU
   case(13); jc_id = 12  ! NOAA-18  PM  AMSU  
   case(20); jc_id = 13  ! METOP        AMSU
   case(14); jc_id = 14  ! NOAA-19  PM  AMSU   
   case DEFAULT; STOP "ERROR - rs_id_to_jc_id(): invalid value for input rs_id"
   end select

end function rs_id_to_jc_id

function jc_id_to_label( jc_id ) result( label )
   ! Satellite ID translation table.
   ! Translate satellite IDs used by John Christy (JC) to the 3 character satellite label.
   integer,intent(IN) :: jc_id
   character(len=3) :: label

   select case(jc_id)
   case(1  ); label =  "trn" ! TIROS-N  PM (1:30)
   case(2  ); label =  "n06" ! NOAA-6   AM (7:30)
   case(3  ); label =  "n07" ! NOAA-7   PM     
   case(4  ); label =  "n08" ! NOAA-8   AM
   case(5  ); label =  "n09" ! NOAA-9   PM
   case(6  ); label =  "n10" ! NOAA-10  AM
   case(7  ); label =  "n11" ! NOAA-11  PM
   case(8  ); label =  "n12" ! NOAA-12  AM
   case(9  ); label =  "n14" ! NOAA-14  PM
   case(10 ); label =  "n15" ! NOAA-15  AM  AMSU
   case(11 ); label =  "aqu" ! AQUA     PM  AMSU
   case(12 ); label =  "n18" ! NOAA-18  PM  AMSU  
   case(13 ); label =  "met" ! METOP        AMSU
   case(14 ); label =  "n19" ! NOAA-19  PM  AMSU   
   case DEFAULT; STOP "ERROR - jc_id_to_label(): invalid value for input jc_id"
   end select

end function jc_id_to_label

END MODULE MSU_MODULE

!=================================
program main
  use MSU_MODULE
  use CROSSING_TIME  ! In shared_routines directory
  implicit none

  integer :: i

  real,allocatable :: lat(:,:), lon(:,:) ! Dims are (11 prints, # scans)
  real,allocatable :: rtb(:,:,:) ! Dims are (4 channels,  11 prints, # scans)
  integer(4),allocatable :: time(:), secs(:) ! Dims are (# scans)
  integer(4),allocatable :: day(:), year(:) ! Julian day, 4-digit year. Dims are(#scans)

  character(len=150) :: outfile

  real(4),allocatable :: grid(:,:,:,:)  ! Tb grid. Dims are (nlat,nlon, 6 angles, channels 2,3,4)
  integer(2),allocatable :: igrid(:,:,:,:) ! Number observations grid. Dims are (nlat,nlon, 6 angles, channels 2,3,4)
  integer :: rs_id, jc_id
  character(len=3) :: sat_label

  character(len=150),parameter :: outpath = "./monthly_grids/" ! Path to output grid files

  type(monthly_file_structure),allocatable :: list(:)
  real(4) :: eq_cr_time

  ! Read the file "runparms_all" into the global monthly_file_structure variable "list".
  ! Variable list is allocated in the called routine.
  call read_file_locations( list )

  ! Read MSU monthly orbit files
  FILE_LOOP: do i=1,size(list)
     write(6,*) i

     !! DEBUG BLOCK
     !if( (list(i)%ksat==3 .and. list(i)%kyear==100 .and. ANY(list(i)%kmon==(/3,7,11/)) ) .or. &
     !    (list(i)%ksat==3 .and. list(i)%kyear==96  .and. list(i)%kmon==1 )) then
     !   continue
     !else
     !   cycle
     !end if   


     ! list(i) is input. The remaining variables are output (and allocated within call).
     ! day is jday, year is 4-digit.
     call read_and_calibrate_msu(list(i), lat, lon, rtb, time, secs, day, year )

     ! Construct monthly averaged grid for each print and channel(2,3,4) for current month and satellite.
     ! Variable grid is allocated within call.
     call build_monthly_grid_360x720( list(i), lat, lon, rtb, day, year, grid, igrid)
     call average_monthly_grid_to_72_144(grid, igrid)

     ! Get satellite label (e.g. trn, n06, n14 etc)
     rs_id = list(i)%ksat ! Roy's satellite designation
     jc_id = rs_id_to_jc_id( rs_id ) ! John's satellite designation
     sat_label = jc_id_to_label( jc_id )

     ! Compute ascending equatorial crossing time (based on print 6 crossing the equator). Routine returns median value.
     call compute_equatorial_crossing_time( lat(6,:), lon(6,:), real(secs(:)), badvalue, eq_cr_time )
     write(6,'(a,1x,f6.3)') "eq_cr_time=",eq_cr_time

     ! Create output file name (e.g. msu_n14_1985_07.dat)
     write(outfile,'("msu_",a3,"_",i4.4,"_",i2.2,".dat")') sat_label, list(i)%kyear+1900, list(i)%kmon

     ! Write grid to file
     open(11,file=trim(outpath)//trim(outfile),status="replace",form="unformatted",access="stream")
     write(11) eq_cr_time
     write(11) grid
     write(11) igrid
     close(11)

  end do FILE_LOOP   


end program main


