MODULE crossing_time
  implicit none

CONTAINS

subroutine compute_equatorial_crossing_time( lat, lon, secs, badvalue, eq_cr_time )
  use MEDIAN_MODULE
  implicit none
  real,intent(IN)  :: lat(:)       ! latitude, dim (#scans)
  real,intent(IN)  :: lon(:)       ! longitude
  real,intent(IN)  :: secs(:)      ! GMT seconds in current day
  real,intent(IN)  :: badvalue  
  real,intent(OUT) :: eq_cr_time   ! Local equatorial crossing time averaged over the month
                                               

  integer :: n ! Vector lengths
  real :: lct ! Most recent ASC equatorial crossing time
  integer :: i
  real :: hr ! Fractional hour
  real :: rsecs ! Secs interpolated to latitude=0

  real :: avg
  integer :: avg_count
  real,allocatable :: lct_vec(:)
  real :: median

  ! Check input vector sizes
  n = size(lat)
  if(size(lon)/=n .or. size(secs)/=n) stop "Error - compute_equatorial_crossing_times(): Inconsistent sizes for arg list vectors"

  allocate(lct_vec(size(lat)))
  
  lct_vec = badvalue

  ! Accumulate local crossing time (LCT) for average
  do i=2,n
     if(lat(i)==badvalue  .or. lat(i-1)==badvalue) cycle
     if(lon(i)==badvalue  .or. lon(i-1)==badvalue) cycle
     if(secs(i)==badvalue .or. secs(i-1)==badvalue) cycle

     if(lat(i-1)<0 .and. lat(i)>=0) then ! ASC equatorial crossing
        if( (lat(i) - lat(i-1)) > 2. ) cycle ! Scans must be within 2 degree of each other
        rsecs = secs(i-1) + (0.-lat(i-1))/(lat(i)-lat(i-1)) * (secs(i)-secs(i-1)) ! Interpolate time to the 0 crossing
        hr = rsecs*24./86400.
        lct = hr + lon(i)*24./360. ! Local equatorial crossing time (fractional hour)
        if(lct<0) lct = lct + 24.
        lct_vec(i) = lct
     end if

  end do   

  ! Use median value of lct_vec for crossing time
  call find_median(lct_vec,median,badvalue)
  eq_cr_time = median

  deallocate(lct_vec)

end subroutine compute_equatorial_crossing_time

end module crossing_time
